@@ -16,7 +16,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, see <https://www.gnu.org/licenses/>.
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
 
 #include <config.h>
@@ -33,9 +33,8 @@
 #include <npth.h> /* (we use pth_sleep) */
 
 #include "agent.h"
-#include "../common/i18n.h"
+#include "i18n.h"
 #include "../common/ssh-utils.h"
-#include "../common/name-value.h"
 
 #ifndef O_BINARY
 #define O_BINARY 0
@@ -52,89 +51,6 @@ struct try_unprotect_arg_s
 };
 
 
-/* Note: Ownership of FNAME and FP are moved to this function.  */
-static gpg_error_t
-write_extended_private_key (char *fname, estream_t fp, int update,
-                            const void *buf, size_t len)
-{
-  gpg_error_t err;
-  nvc_t pk = NULL;
-  gcry_sexp_t key = NULL;
-  int remove = 0;
-
-  if (update)
-    {
-      int line;
-
-      err = nvc_parse_private_key (&pk, &line, fp);
-      if (err && gpg_err_code (err) != GPG_ERR_ENOENT)
-        {
-          log_error ("error parsing '%s' line %d: %s\n",
-                     fname, line, gpg_strerror (err));
-          goto leave;
-        }
-    }
-  else
-    {
-      pk = nvc_new_private_key ();
-      if (!pk)
-        {
-          err = gpg_error_from_syserror ();
-          goto leave;
-        }
-    }
-  es_clearerr (fp);
-
-  err = gcry_sexp_sscan (&key, NULL, buf, len);
-  if (err)
-    goto leave;
-
-  err = nvc_set_private_key (pk, key);
-  if (err)
-    goto leave;
-
-  err = es_fseek (fp, 0, SEEK_SET);
-  if (err)
-    goto leave;
-
-  err = nvc_write (pk, fp);
-  if (err)
-    {
-      log_error ("error writing '%s': %s\n", fname, gpg_strerror (err));
-      remove = 1;
-      goto leave;
-    }
-
-  if (ftruncate (es_fileno (fp), es_ftello (fp)))
-    {
-      err = gpg_error_from_syserror ();
-      log_error ("error truncating '%s': %s\n", fname, gpg_strerror (err));
-      remove = 1;
-      goto leave;
-    }
-
-  if (es_fclose (fp))
-    {
-      err = gpg_error_from_syserror ();
-      log_error ("error closing '%s': %s\n", fname, gpg_strerror (err));
-      remove = 1;
-      goto leave;
-    }
-  else
-    fp = NULL;
-
-  bump_key_eventcounter ();
-
- leave:
-  es_fclose (fp);
-  if (remove)
-    gnupg_remove (fname);
-  xfree (fname);
-  gcry_sexp_release (key);
-  nvc_release (pk);
-  return err;
-}
-
 /* Write an S-expression formatted key to our key storage.  With FORCE
    passed as true an existing key with the given GRIP will get
    overwritten.  */
@@ -149,8 +65,7 @@ agent_write_private_key (const unsigned char *grip,
   bin2hex (grip, 20, hexgrip);
   strcpy (hexgrip+40, ".key");
 
-  fname = make_filename (gnupg_homedir (), GNUPG_PRIVATE_KEYS_DIR,
-                         hexgrip, NULL);
+  fname = make_filename (opt.homedir, GNUPG_PRIVATE_KEYS_DIR, hexgrip, NULL);
 
   /* FIXME: Write to a temp file first so that write failures during
      key updates won't lead to a key loss.  */
@@ -162,63 +77,14 @@ agent_write_private_key (const unsigned char *grip,
       return gpg_error (GPG_ERR_EEXIST);
     }
 
-  fp = es_fopen (fname, force? "rb+,mode=-rw" : "wbx,mode=-rw");
+  fp = es_fopen (fname, force? "wb,mode=-rw" : "wbx,mode=-rw");
   if (!fp)
     {
       gpg_error_t tmperr = gpg_error_from_syserror ();
-
-      if (force && gpg_err_code (tmperr) == GPG_ERR_ENOENT)
-        {
-          fp = es_fopen (fname, "wbx,mode=-rw");
-          if (!fp)
-            tmperr = gpg_error_from_syserror ();
-        }
-      if (!fp)
-        {
-          log_error ("can't create '%s': %s\n", fname, gpg_strerror (tmperr));
-          xfree (fname);
-          return tmperr;
-        }
+      log_error ("can't create '%s': %s\n", fname, gpg_strerror (tmperr));
+      xfree (fname);
+      return tmperr;
     }
-  else if (force)
-    {
-      gpg_error_t rc;
-      char first;
-
-      /* See if an existing key is in extended format.  */
-      if (es_fread (&first, 1, 1, fp) != 1)
-        {
-          rc = gpg_error_from_syserror ();
-          log_error ("error reading first byte from '%s': %s\n",
-                     fname, strerror (errno));
-          xfree (fname);
-          es_fclose (fp);
-          return rc;
-        }
-
-      rc = es_fseek (fp, 0, SEEK_SET);
-      if (rc)
-        {
-          log_error ("error seeking in '%s': %s\n", fname, strerror (errno));
-          xfree (fname);
-          es_fclose (fp);
-          return rc;
-        }
-
-      if (first != '(')
-        {
-          /* Key is already in the extended format.  */
-          return write_extended_private_key (fname, fp, 1, buffer, length);
-        }
-      if (first == '(' && opt.enable_extended_key_format)
-        {
-          /* Key is in the old format - but we want the extended format.  */
-          return write_extended_private_key (fname, fp, 0, buffer, length);
-        }
-    }
-
-  if (opt.enable_extended_key_format)
-    return write_extended_private_key (fname, fp, 0, buffer, length);
 
   if (es_fwrite (buffer, length, 1, fp) != 1)
     {
@@ -229,18 +95,6 @@ agent_write_private_key (const unsigned char *grip,
       xfree (fname);
       return tmperr;
     }
-
-  /* When force is given, the file might have to be truncated.  */
-  if (force && ftruncate (es_fileno (fp), es_ftello (fp)))
-    {
-      gpg_error_t tmperr = gpg_error_from_syserror ();
-      log_error ("error truncating '%s': %s\n", fname, gpg_strerror (tmperr));
-      es_fclose (fp);
-      gnupg_remove (fname);
-      xfree (fname);
-      return tmperr;
-    }
-
   if (es_fclose (fp))
     {
       gpg_error_t tmperr = gpg_error_from_syserror ();
@@ -328,7 +182,7 @@ try_unprotect_cb (struct pin_entry_info_s *pi)
       xfree (desc);
     }
 
-  return err;
+  return 0;
 }
 
 
@@ -343,9 +197,9 @@ try_unprotect_cb (struct pin_entry_info_s *pi)
    The functions returns 0 on success or an error code.  On success a
    newly allocated string is stored at the address of RESULT.
  */
-gpg_error_t
-agent_modify_description (const char *in, const char *comment,
-                          const gcry_sexp_t key, char **result)
+static gpg_error_t
+modify_description (const char *in, const char *comment, const gcry_sexp_t key,
+                    char **result)
 {
   size_t comment_length;
   size_t in_len;
@@ -354,19 +208,12 @@ agent_modify_description (const char *in, const char *comment,
   size_t i;
   int special, pass;
   char *ssh_fpr = NULL;
-  char *p;
-
-  *result = NULL;
-
-  if (!comment)
-    comment = "";
 
   comment_length = strlen (comment);
   in_len  = strlen (in);
 
   /* First pass calculates the length, second pass does the actual
      copying.  */
-  /* FIXME: This can be simplified by using es_fopenmem.  */
   out = NULL;
   out_len = 0;
   for (pass=0; pass < 2; pass++)
@@ -412,8 +259,7 @@ agent_modify_description (const char *in, const char *comment,
 
                 case 'F': /* SSH style fingerprint.  */
                   if (!ssh_fpr && key)
-                    ssh_get_fingerprint_string (key, opt.ssh_fingerprint_digest,
-                                                &ssh_fpr);
+                    ssh_get_fingerprint_string (key, &ssh_fpr);
                   if (ssh_fpr)
                     {
                       if (out)
@@ -457,23 +303,8 @@ agent_modify_description (const char *in, const char *comment,
     }
 
   *out = 0;
-  log_assert (*result + out_len == out);
+  assert (*result + out_len == out);
   xfree (ssh_fpr);
-
-  /* The ssh prompt may sometimes end in
-   *    "...%0A  ()"
-   * The empty parentheses doesn't look very good.  We use this hack
-   * here to remove them as well as the indentation spaces. */
-  p = *result;
-  i = strlen (p);
-  if (i > 2 && !strcmp (p + i - 2, "()"))
-    {
-      p += i - 2;
-      *p-- = 0;
-      while (p > *result && spacep (p))
-        *p-- = 0;
-    }
-
   return 0;
 }
 
@@ -488,7 +319,7 @@ agent_modify_description (const char *in, const char *comment,
    passphrase (entered or from the cache) is stored there; if not NULL
    will be stored.  The caller needs to free the returned
    passphrase. */
-static gpg_error_t
+static int
 unprotect (ctrl_t ctrl, const char *cache_nonce, const char *desc_text,
            unsigned char **keybuf, const unsigned char *grip,
            cache_mode_t cache_mode, lookup_ttl_t lookup_ttl,
@@ -553,6 +384,7 @@ unprotect (ctrl_t ctrl, const char *cache_nonce, const char *desc_text,
               return 0;
             }
           xfree (pw);
+          rc  = 0;
         }
       else if (cache_mode == CACHE_MODE_NORMAL)
         {
@@ -590,6 +422,7 @@ unprotect (ctrl_t ctrl, const char *cache_nonce, const char *desc_text,
                   return 0;
                 }
               xfree (pw);
+              rc  = 0;
             }
         }
 
@@ -690,7 +523,7 @@ unprotect (ctrl_t ctrl, const char *cache_nonce, const char *desc_text,
 static gpg_error_t
 read_key_file (const unsigned char *grip, gcry_sexp_t *result)
 {
-  gpg_error_t err;
+  int rc;
   char *fname;
   estream_t fp;
   struct stat st;
@@ -698,113 +531,67 @@ read_key_file (const unsigned char *grip, gcry_sexp_t *result)
   size_t buflen, erroff;
   gcry_sexp_t s_skey;
   char hexgrip[40+4+1];
-  char first;
 
   *result = NULL;
 
   bin2hex (grip, 20, hexgrip);
   strcpy (hexgrip+40, ".key");
 
-  fname = make_filename (gnupg_homedir (), GNUPG_PRIVATE_KEYS_DIR,
-                         hexgrip, NULL);
+  fname = make_filename (opt.homedir, GNUPG_PRIVATE_KEYS_DIR, hexgrip, NULL);
   fp = es_fopen (fname, "rb");
   if (!fp)
     {
-      err = gpg_error_from_syserror ();
-      if (gpg_err_code (err) != GPG_ERR_ENOENT)
-        log_error ("can't open '%s': %s\n", fname, gpg_strerror (err));
+      rc = gpg_error_from_syserror ();
+      if (gpg_err_code (rc) != GPG_ERR_ENOENT)
+        log_error ("can't open '%s': %s\n", fname, strerror (errno));
       xfree (fname);
-      return err;
-    }
-
-  if (es_fread (&first, 1, 1, fp) != 1)
-    {
-      err = gpg_error_from_syserror ();
-      log_error ("error reading first byte from '%s': %s\n",
-                 fname, gpg_strerror (err));
-      xfree (fname);
-      es_fclose (fp);
-      return err;
-    }
-
-  if (es_fseek (fp, 0, SEEK_SET))
-    {
-      err = gpg_error_from_syserror ();
-      log_error ("error seeking in '%s': %s\n", fname, gpg_strerror (err));
-      xfree (fname);
-      es_fclose (fp);
-      return err;
-    }
-
-  if (first != '(')
-    {
-      /* Key is in extended format.  */
-      nvc_t pk;
-      int line;
-
-      err = nvc_parse_private_key (&pk, &line, fp);
-      es_fclose (fp);
-
-      if (err)
-        log_error ("error parsing '%s' line %d: %s\n",
-                   fname, line, gpg_strerror (err));
-      else
-        {
-          err = nvc_get_private_key (pk, result);
-          nvc_release (pk);
-          if (err)
-            log_error ("error getting private key from '%s': %s\n",
-                       fname, gpg_strerror (err));
-        }
-
-      xfree (fname);
-      return err;
+      return rc;
     }
 
   if (fstat (es_fileno (fp), &st))
     {
-      err = gpg_error_from_syserror ();
-      log_error ("can't stat '%s': %s\n", fname, gpg_strerror (err));
+      rc = gpg_error_from_syserror ();
+      log_error ("can't stat '%s': %s\n", fname, strerror (errno));
       xfree (fname);
       es_fclose (fp);
-      return err;
+      return rc;
     }
 
   buflen = st.st_size;
   buf = xtrymalloc (buflen+1);
   if (!buf)
     {
-      err = gpg_error_from_syserror ();
+      rc = gpg_error_from_syserror ();
       log_error ("error allocating %zu bytes for '%s': %s\n",
-                 buflen, fname, gpg_strerror (err));
+                 buflen, fname, strerror (errno));
       xfree (fname);
       es_fclose (fp);
       xfree (buf);
-      return err;
+      return rc;
 
     }
 
   if (es_fread (buf, buflen, 1, fp) != 1)
     {
-      err = gpg_error_from_syserror ();
+      rc = gpg_error_from_syserror ();
       log_error ("error reading %zu bytes from '%s': %s\n",
-                 buflen, fname, gpg_strerror (err));
+                 buflen, fname, strerror (errno));
       xfree (fname);
       es_fclose (fp);
       xfree (buf);
-      return err;
+      return rc;
     }
 
   /* Convert the file into a gcrypt S-expression object.  */
-  err = gcry_sexp_sscan (&s_skey, &erroff, (char*)buf, buflen);
+  rc = gcry_sexp_sscan (&s_skey, &erroff, (char*)buf, buflen);
   xfree (fname);
   es_fclose (fp);
   xfree (buf);
-  if (err)
+  if (rc)
     {
       log_error ("failed to build S-Exp (off=%u): %s\n",
-                 (unsigned int)erroff, gpg_strerror (err));
-      return err;
+                 (unsigned int)erroff, gpg_strerror (rc));
+      return rc;
     }
   *result = s_skey;
   return 0;
@@ -821,8 +608,7 @@ remove_key_file (const unsigned char *grip)
 
   bin2hex (grip, 20, hexgrip);
   strcpy (hexgrip+40, ".key");
-  fname = make_filename (gnupg_homedir (), GNUPG_PRIVATE_KEYS_DIR,
-                         hexgrip, NULL);
+  fname = make_filename (opt.homedir, GNUPG_PRIVATE_KEYS_DIR, hexgrip, NULL);
   if (gnupg_remove (fname))
     err = gpg_error_from_syserror ();
   xfree (fname);
@@ -852,7 +638,7 @@ agent_key_from_file (ctrl_t ctrl, const char *cache_nonce,
                      cache_mode_t cache_mode, lookup_ttl_t lookup_ttl,
                      gcry_sexp_t *result, char **r_passphrase)
 {
-  gpg_error_t err;
+  int rc;
   unsigned char *buf;
   size_t len, buflen, erroff;
   gcry_sexp_t s_skey;
@@ -863,20 +649,20 @@ agent_key_from_file (ctrl_t ctrl, const char *cache_nonce,
   if (r_passphrase)
     *r_passphrase = NULL;
 
-  err = read_key_file (grip, &s_skey);
-  if (err)
+  rc = read_key_file (grip, &s_skey);
+  if (rc)
     {
-      if (gpg_err_code (err) == GPG_ERR_ENOENT)
-        err = gpg_error (GPG_ERR_NO_SECKEY);
-      return err;
+      if (gpg_err_code (rc) == GPG_ERR_ENOENT)
+        rc = gpg_error (GPG_ERR_NO_SECKEY);
+      return rc;
     }
 
   /* For use with the protection functions we also need the key as an
      canonical encoded S-expression in a buffer.  Create this buffer
      now.  */
-  err = make_canon_sexp (s_skey, &buf, &len);
-  if (err)
-    return err;
+  rc = make_canon_sexp (s_skey, &buf, &len);
+  if (rc)
+    return rc;
 
   switch (agent_private_key_type (buf))
     {
@@ -887,10 +673,10 @@ agent_key_from_file (ctrl_t ctrl, const char *cache_nonce,
         unsigned char *buf_new;
         size_t buf_newlen;
 
-        err = agent_unprotect (ctrl, buf, "", NULL, &buf_new, &buf_newlen);
-        if (err)
+        rc = agent_unprotect (ctrl, buf, "", NULL, &buf_new, &buf_newlen);
+        if (rc)
           log_error ("failed to convert unprotected openpgp key: %s\n",
-                     gpg_strerror (err));
+                     gpg_strerror (rc));
         else
           {
             xfree (buf);
@@ -917,17 +703,17 @@ agent_key_from_file (ctrl_t ctrl, const char *cache_nonce,
 
         desc_text_final = NULL;
 	if (desc_text)
-          err = agent_modify_description (desc_text, comment, s_skey,
-                                          &desc_text_final);
+          rc = modify_description (desc_text, comment? comment:"", s_skey,
+                                   &desc_text_final);
         gcry_free (comment);
 
-	if (!err)
+	if (!rc)
 	  {
-	    err = unprotect (ctrl, cache_nonce, desc_text_final, &buf, grip,
+	    rc = unprotect (ctrl, cache_nonce, desc_text_final, &buf, grip,
                             cache_mode, lookup_ttl, r_passphrase);
-	    if (err)
+	    if (rc)
 	      log_error ("failed to unprotect the secret key: %s\n",
-			 gpg_strerror (err));
+			 gpg_strerror (rc));
 	  }
 
 	xfree (desc_text_final);
@@ -939,34 +725,34 @@ agent_key_from_file (ctrl_t ctrl, const char *cache_nonce,
           const unsigned char *s;
           size_t n;
 
-          err = agent_get_shadow_info (buf, &s);
-          if (!err)
+          rc = agent_get_shadow_info (buf, &s);
+          if (!rc)
             {
               n = gcry_sexp_canon_len (s, 0, NULL,NULL);
-              log_assert (n);
+              assert (n);
               *shadow_info = xtrymalloc (n);
               if (!*shadow_info)
-                err = out_of_core ();
+                rc = out_of_core ();
               else
                 {
                   memcpy (*shadow_info, s, n);
-                  err = 0;
+                  rc = 0;
                 }
             }
-          if (err)
-            log_error ("get_shadow_info failed: %s\n", gpg_strerror (err));
+          if (rc)
+            log_error ("get_shadow_info failed: %s\n", gpg_strerror (rc));
         }
       else
-        err = gpg_error (GPG_ERR_UNUSABLE_SECKEY);
+        rc = gpg_error (GPG_ERR_UNUSABLE_SECKEY);
       break;
     default:
       log_error ("invalid private key format\n");
-      err = gpg_error (GPG_ERR_BAD_SECKEY);
+      rc = gpg_error (GPG_ERR_BAD_SECKEY);
       break;
     }
   gcry_sexp_release (s_skey);
   s_skey = NULL;
-  if (err)
+  if (rc)
     {
       xfree (buf);
       if (r_passphrase)
@@ -974,23 +760,23 @@ agent_key_from_file (ctrl_t ctrl, const char *cache_nonce,
           xfree (*r_passphrase);
           *r_passphrase = NULL;
         }
-      return err;
+      return rc;
     }
 
   buflen = gcry_sexp_canon_len (buf, 0, NULL, NULL);
-  err = gcry_sexp_sscan (&s_skey, &erroff, (char*)buf, buflen);
+  rc = gcry_sexp_sscan (&s_skey, &erroff, (char*)buf, buflen);
   wipememory (buf, buflen);
   xfree (buf);
-  if (err)
+  if (rc)
     {
       log_error ("failed to build S-Exp (off=%u): %s\n",
-                 (unsigned int)erroff, gpg_strerror (err));
+                 (unsigned int)erroff, gpg_strerror (rc));
       if (r_passphrase)
         {
           xfree (*r_passphrase);
           *r_passphrase = NULL;
         }
-      return err;
+      return rc;
     }
 
   *result = s_skey;
@@ -1265,7 +1051,7 @@ agent_public_key_from_file (ctrl_t ctrl,
   /* FIXME: The following thing is pretty ugly code; we should
      investigate how to make it cleaner.  Probably code to handle
      canonical S-expressions in a memory buffer is better suited for
-     such a task.  After all that is what we do in protect.c.  Need
+     such a task.  After all that is what we do in protect.c.  Neeed
      to find common patterns and write a straightformward API to use
      them.  */
   assert (sizeof (size_t) <= sizeof (void*));
@@ -1332,7 +1118,7 @@ agent_public_key_from_file (ctrl_t ctrl,
 
 
 
-/* Check whether the secret key identified by GRIP is available.
+/* Check whether the the secret key identified by GRIP is available.
    Returns 0 is the key is available.  */
 int
 agent_key_available (const unsigned char *grip)
@@ -1344,8 +1130,7 @@ agent_key_available (const unsigned char *grip)
   bin2hex (grip, 20, hexgrip);
   strcpy (hexgrip+40, ".key");
 
-  fname = make_filename (gnupg_homedir (), GNUPG_PRIVATE_KEYS_DIR,
-                         hexgrip, NULL);
+  fname = make_filename (opt.homedir, GNUPG_PRIVATE_KEYS_DIR, hexgrip, NULL);
   result = !access (fname, R_OK)? 0 : -1;
   xfree (fname);
   return result;
@@ -1434,20 +1219,18 @@ agent_key_info_from_file (ctrl_t ctrl, const unsigned char *grip,
 
 
 /* Delete the key with GRIP from the disk after having asked for
- * confirmation using DESC_TEXT.  If FORCE is set the function won't
- * require a confirmation via Pinentry or warns if the key is also
- * used by ssh.  If ONLY_STUBS is set only stub keys (references to
- * smartcards) will be affected.
- *
- * Common error codes are:
- *   GPG_ERR_NO_SECKEY
- *   GPG_ERR_KEY_ON_CARD
- *   GPG_ERR_NOT_CONFIRMED
- *   GPG_ERR_FORBIDDEN     - Not a stub key and ONLY_STUBS requested.
- */
+   confirmation using DESC_TEXT.  If FORCE is set the function won't
+   require a confirmation via Pinentry or warns if the key is also
+   used by ssh.
+
+   Common error codes are:
+     GPG_ERR_NO_SECKEY
+     GPG_ERR_KEY_ON_CARD
+     GPG_ERR_NOT_CONFIRMED
+*/
 gpg_error_t
 agent_delete_key (ctrl_t ctrl, const char *desc_text,
-                  const unsigned char *grip, int force, int only_stubs)
+                  const unsigned char *grip, int force)
 {
   gpg_error_t err;
   gcry_sexp_t s_skey = NULL;
@@ -1458,7 +1241,6 @@ agent_delete_key (ctrl_t ctrl, const char *desc_text,
   ssh_control_file_t cf = NULL;
   char hexgrip[40+4+1];
   char *default_desc = NULL;
-  int key_type;
 
   err = read_key_file (grip, &s_skey);
   if (gpg_err_code (err) == GPG_ERR_ENOENT)
@@ -1470,14 +1252,7 @@ agent_delete_key (ctrl_t ctrl, const char *desc_text,
   if (err)
     goto leave;
 
-  key_type = agent_private_key_type (buf);
-  if (only_stubs && key_type != PRIVATE_KEY_SHADOWED)
-    {
-      err  = gpg_error (GPG_ERR_FORBIDDEN);
-      goto leave;
-    }
-
-  switch (key_type)
+  switch (agent_private_key_type (buf))
     {
     case PRIVATE_KEY_CLEAR:
     case PRIVATE_KEY_OPENPGP_NONE:
@@ -1506,8 +1281,8 @@ agent_delete_key (ctrl_t ctrl, const char *desc_text,
           }
 
           if (desc_text)
-            err = agent_modify_description (desc_text, comment, s_skey,
-                                            &desc_text_final);
+            err = modify_description (desc_text, comment? comment:"", s_skey,
+                                      &desc_text_final);
           if (err)
             goto leave;
 
@@ -1536,7 +1311,7 @@ agent_delete_key (ctrl_t ctrl, const char *desc_text,
       break;
 
     case PRIVATE_KEY_SHADOWED:
-      err = remove_key_file (grip);
+      err = gpg_error (GPG_ERR_KEY_ON_CARD);
       break;
 
     default:
@@ -1554,39 +1329,3 @@ agent_delete_key (ctrl_t ctrl, const char *desc_text,
   gcry_sexp_release (s_skey);
   return err;
 }
-
-
-/* Write an S-expression formatted shadow key to our key storage.
-   Shadow key is created by an S-expression public key in PKBUF and
-   card's SERIALNO and the IDSTRING.  With FORCE passed as true an
-   existing key with the given GRIP will get overwritten.  */
-gpg_error_t
-agent_write_shadow_key (const unsigned char *grip,
-                        const char *serialno, const char *keyid,
-                        const unsigned char *pkbuf, int force)
-{
-  gpg_error_t err;
-  unsigned char *shadow_info;
-  unsigned char *shdkey;
-  size_t len;
-
-  shadow_info = make_shadow_info (serialno, keyid);
-  if (!shadow_info)
-    return gpg_error_from_syserror ();
-
-  err = agent_shadow_key (pkbuf, shadow_info, &shdkey);
-  xfree (shadow_info);
-  if (err)
-    {
-      log_error ("shadowing the key failed: %s\n", gpg_strerror (err));
-      return err;
-    }
-
-  len = gcry_sexp_canon_len (shdkey, 0, NULL, NULL);
-  err = agent_write_private_key (grip, shdkey, len, force);
-  xfree (shdkey);
-  if (err)
-    log_error ("error writing key: %s\n", gpg_strerror (err));
-
-  return err;
-}
