@@ -23,7 +23,7 @@
  ************************************************************
  * The code for the splash screen has been taken from the Splash
  * plugin of the NSIS 2.04 distribution.  That code comes without
- * explicit copyright notices in the source files or author names, it
+ * explicit copyright notices in tyhe source files or author names, it
  * seems that it has been written by Justin Frankel; not sure about
  * the year, though. [wk 2005-11-28]
  *
@@ -70,12 +70,12 @@ dummy (HWND hwndParent, int string_size, char *variables,
   // do your stuff here
   {
     char buf[1024];
-    snprintf (buf, sizeof buf, "$R0=%s\r\n$R1=%s\r\n",
+    snprintf (buf, sizeof buf - 1, "$R0=%s\r\n$R1=%s\r\n",
               getuservariable(INST_R0),
               getuservariable(INST_R1));
     MessageBox (g_hwndParent,buf,0,MB_OK);
 
-    snprintf (buf, sizeof buf,
+    snprintf (buf, sizeof buf - 1,
              "autoclose    =%d\r\n"
              "all_user_var =%d\r\n"
              "exec_error   =%d\r\n"
@@ -278,7 +278,7 @@ void
 service_error (const char *str)
 {
   char buf[1024];
-  snprintf (buf, sizeof (buf), "error: %s: ec=%d\r\n", str,
+  snprintf (buf, sizeof (buf) - 1, "error: %s: ec=%d\r\n", str,
 	    GetLastError ());
   MessageBox(g_hwndParent, buf, 0, MB_OK);
 
@@ -575,7 +575,7 @@ service_stop (HWND hwndParent, int string_size, char *variables,
       if (GetTickCount () - start_time > timeout)
 	{
 	  char buf[1024];
-	  snprintf (buf, sizeof (buf),
+	  snprintf (buf, sizeof (buf) - 1,
 		    "time out waiting for service %s to stop\r\n",
 		    service_name);
 	  MessageBox (g_hwndParent, buf, 0, MB_OK);
@@ -964,8 +964,10 @@ read_w32_registry_string (HKEY root, const char *dir, const char *name)
 #define ENV_REG "SYSTEM\\CurrentControlSet\\Control\\" \
     "Session Manager\\Environment"
   /* The following setting can be used for a per-user setting.  */
-#define ENV_HK_USER HKEY_CURRENT_USER
-#define ENV_REG_USER "Environment"
+#if 0
+#define ENV_HK HKEY_CURRENT_USER
+#define ENV_REG "Environment"
+#endif
 /* Due to a bug in Windows7 (kb 2685893) we better put a lower limit
    than 8191 on the maximum length of the PATH variable.  Note, that
    depending on the used toolchain we used to had a 259 byte limit in
@@ -977,16 +979,12 @@ path_add (HWND hwndParent, int string_size, char *variables,
 	  stack_t **stacktop, extra_parameters_t *extra)
 {
   char dir[PATH_LENGTH_LIMIT];
-  char is_user_install[2];
   char *path;
   char *path_new;
   int path_new_size;
   char *comp;
   const char delims[] = ";";
-  int is_user;
   HKEY key_handle = 0;
-  HKEY root_key;
-  const char *env_reg;
 
   g_hwndParent = hwndParent;
   EXDLL_INIT();
@@ -999,26 +997,13 @@ path_add (HWND hwndParent, int string_size, char *variables,
   if (popstring (dir, sizeof (dir)))
     return;
 
-  /* The expected stack layout: HKEY component.  */
-  if (popstring (is_user_install, sizeof (is_user_install)))
-    return;
-
-  if (!strcmp(is_user_install, "1"))
-    {
-      root_key = ENV_HK_USER;
-      env_reg = ENV_REG_USER;
-    }
-  else
-    {
-      root_key = ENV_HK;
-      env_reg = ENV_REG;
-    }
-
-  path = read_w32_registry_string (root_key, env_reg, "Path");
+/*   MessageBox (g_hwndParent, "XXX 2", 0, MB_OK); */
 
+  path = read_w32_registry_string (ENV_HK, ENV_REG, "Path");
   if (! path)
     {
-      path = strdup ("");
+      MessageBox (g_hwndParent, "No PATH variable found", 0, MB_OK);
+      return;
     }
 
 /*   MessageBox (g_hwndParent, "XXX 3", 0, MB_OK); */
@@ -1056,8 +1041,6 @@ path_add (HWND hwndParent, int string_size, char *variables,
   do
     {
 /*       MessageBox (g_hwndParent, comp, 0, MB_OK); */
-      if (!comp)
-        break;
 
       if (!strcmp (comp, dir))
 	{
@@ -1070,8 +1053,10 @@ path_add (HWND hwndParent, int string_size, char *variables,
   while (comp);
   free (path);
 
-  /* Update the path key.  */
-  RegCreateKey (root_key, env_reg, &key_handle);
+/*   MessageBox (g_hwndParent, "XXX 8", 0, MB_OK); */
+
+  /* Set a key for our CLSID.  */
+  RegCreateKey (ENV_HK, ENV_REG, &key_handle);
   RegSetValueEx (key_handle, "Path", 0, REG_EXPAND_SZ,
 		 path_new, path_new_size);
   RegCloseKey (key_handle);
@@ -1089,7 +1074,6 @@ path_remove (HWND hwndParent, int string_size, char *variables,
 	     stack_t **stacktop, extra_parameters_t *extra)
 {
   char dir[PATH_LENGTH_LIMIT];
-  char is_user_install[2];
   char *path;
   char *path_new;
   int path_new_size;
@@ -1098,8 +1082,6 @@ path_remove (HWND hwndParent, int string_size, char *variables,
   HKEY key_handle = 0;
   int changed = 0;
   int count = 0;
-  HKEY root_key;
-  const char *env_reg;
 
   g_hwndParent = hwndParent;
   EXDLL_INIT();
@@ -1110,25 +1092,7 @@ path_remove (HWND hwndParent, int string_size, char *variables,
   if (popstring (dir, sizeof (dir)))
     return;
 
-  /* The expected stack layout: HKEY component.  */
-  if (popstring (is_user_install, sizeof (is_user_install)))
-    return;
-
-  if (!strcmp(is_user_install, "1"))
-    {
-      root_key = ENV_HK_USER;
-      env_reg = ENV_REG_USER;
-    }
-  else
-    {
-      root_key = ENV_HK;
-      env_reg = ENV_REG;
-    }
-
-  path = read_w32_registry_string (root_key, env_reg, "Path");
-
-  if (!path)
-    return;
+  path = read_w32_registry_string (ENV_HK, ENV_REG, "Path");
   /* Old path plus semicolon plus dir plus terminating nul.  */
   path_new_size = strlen (path) + 1;
   path_new = malloc (path_new_size);
@@ -1162,7 +1126,7 @@ path_remove (HWND hwndParent, int string_size, char *variables,
     return;
 
   /* Set a key for our CLSID.  */
-  RegCreateKey (root_key, env_reg, &key_handle);
+  RegCreateKey (ENV_HK, ENV_REG, &key_handle);
   RegSetValueEx (key_handle, "Path", 0, REG_EXPAND_SZ,
 		 path_new, path_new_size);
   RegCloseKey (key_handle);
