@@ -65,7 +65,7 @@
 #include "exechelp.h"
 
 /* Define to 1 do enable debugging.  */
-#define DEBUG_W32_SPAWN 0
+#define DEBUG_W32_SPAWN 1
 
 
 /* It seems Vista doesn't grok X_OK and so fails access() tests.
@@ -233,41 +233,45 @@ build_w32_commandline (const char *pgmname, const char * const *argv,
 }
 
 
-#define INHERIT_READ	1
-#define INHERIT_WRITE	2
-#define INHERIT_BOTH	(INHERIT_READ|INHERIT_WRITE)
-
-/* Create pipe.  FLAGS indicates which ends are inheritable.  */
+/* Create pipe where one end is inheritable: With an INHERIT_IDX of 0
+   the read end is inheritable, with 1 the write end is inheritable.  */
 static int
-create_inheritable_pipe (HANDLE filedes[2], int flags)
+create_inheritable_pipe (HANDLE filedes[2], int inherit_idx)
 {
-  HANDLE r, w;
+  HANDLE r, w, h;
   SECURITY_ATTRIBUTES sec_attr;
 
   memset (&sec_attr, 0, sizeof sec_attr );
   sec_attr.nLength = sizeof sec_attr;
-  sec_attr.bInheritHandle = TRUE;
+  sec_attr.bInheritHandle = FALSE;
 
   if (!CreatePipe (&r, &w, &sec_attr, 0))
     return -1;
 
-  if ((flags & INHERIT_READ) == 0)
-    if (! SetHandleInformation (r, HANDLE_FLAG_INHERIT, 0))
-      goto fail;
+  if (!DuplicateHandle (GetCurrentProcess(), inherit_idx? w : r,
+                        GetCurrentProcess(), &h, 0,
+                        TRUE, DUPLICATE_SAME_ACCESS ))
+    {
+      log_error ("DuplicateHandle failed: %s\n", w32_strerror (-1));
+      CloseHandle (r);
+      CloseHandle (w);
+      return -1;
+    }
 
-  if ((flags & INHERIT_WRITE) == 0)
-    if (! SetHandleInformation (w, HANDLE_FLAG_INHERIT, 0))
-      goto fail;
+  if (inherit_idx)
+    {
+      CloseHandle (w);
+      w = h;
+    }
+  else
+    {
+      CloseHandle (r);
+      r = h;
+    }
 
   filedes[0] = r;
   filedes[1] = w;
   return 0;
-
- fail:
-  log_error ("SetHandleInformation failed: %s\n", w32_strerror (-1));
-  CloseHandle (r);
-  CloseHandle (w);
-  return -1;
 }
 
 
@@ -287,16 +291,16 @@ w32_open_null (int for_write)
 
 
 static gpg_error_t
-do_create_pipe (int filedes[2], int flags)
+do_create_pipe (int filedes[2], int inherit_idx)
 {
   gpg_error_t err = 0;
   HANDLE fds[2];
 
   filedes[0] = filedes[1] = -1;
   err = gpg_error (GPG_ERR_GENERAL);
-  if (!create_inheritable_pipe (fds, flags))
+  if (!create_inheritable_pipe (fds, inherit_idx))
     {
-      filedes[0] = _open_osfhandle (handle_to_fd (fds[0]), O_RDONLY);
+      filedes[0] = _open_osfhandle (handle_to_fd (fds[0]), 0);
       if (filedes[0] == -1)
         {
           log_error ("failed to translate osfhandle %p\n", fds[0]);
@@ -304,7 +308,7 @@ do_create_pipe (int filedes[2], int flags)
         }
       else
         {
-          filedes[1] = _open_osfhandle (handle_to_fd (fds[1]), O_APPEND);
+          filedes[1] = _open_osfhandle (handle_to_fd (fds[1]), 1);
           if (filedes[1] == -1)
             {
               log_error ("failed to translate osfhandle %p\n", fds[1]);
@@ -320,44 +324,28 @@ do_create_pipe (int filedes[2], int flags)
 }
 
 /* Portable function to create a pipe.  Under Windows the write end is
-   inheritable.  If R_FP is not NULL, an estream is created for the
-   read end and stored at R_FP.  */
+   inheritable.  */
 gpg_error_t
-gnupg_create_inbound_pipe (int filedes[2], estream_t *r_fp, int nonblock)
+gnupg_create_inbound_pipe (int filedes[2])
 {
-  if (r_fp)
-    return gpg_error (GPG_ERR_NOT_IMPLEMENTED);
-  else
-    return do_create_pipe (filedes, INHERIT_WRITE);
+  return do_create_pipe (filedes, 1);
 }
 
 
 /* Portable function to create a pipe.  Under Windows the read end is
-   inheritable.  If R_FP is not NULL, an estream is created for the
-   write end and stored at R_FP.  */
-gpg_error_t
-gnupg_create_outbound_pipe (int filedes[2], estream_t *r_fp, int nonblock)
-{
-  if (r_fp)
-    return gpg_error (GPG_ERR_NOT_IMPLEMENTED);
-  else
-    return do_create_pipe (filedes, INHERIT_READ);
-}
-
-
-/* Portable function to create a pipe.  Under Windows both ends are
    inheritable.  */
 gpg_error_t
-gnupg_create_pipe (int filedes[2])
+gnupg_create_outbound_pipe (int filedes[2])
 {
-  return do_create_pipe (filedes, INHERIT_BOTH);
+  return do_create_pipe (filedes, 0);
 }
 
 
 /* Fork and exec the PGMNAME, see exechelp.h for details.  */
 gpg_error_t
 gnupg_spawn_process (const char *pgmname, const char *argv[],
-                     int *except, void (*preexec)(void), unsigned int flags,
+                     gpg_err_source_t errsource,
+                     void (*preexec)(void), unsigned int flags,
                      estream_t *r_infp,
                      estream_t *r_outfp,
                      estream_t *r_errfp,
@@ -386,9 +374,6 @@ gnupg_spawn_process (const char *pgmname, const char *argv[],
                       INVALID_HANDLE_VALUE};
   int i;
   es_syshd_t syshd;
-  gpg_err_source_t errsource = default_errsource;
-
-  (void)except; /* Not yet used.  */
 
   if (r_infp)
     *r_infp = NULL;
@@ -398,9 +383,9 @@ gnupg_spawn_process (const char *pgmname, const char *argv[],
     *r_errfp = NULL;
   *pid = (pid_t)(-1); /* Always required.  */
 
-  if (r_infp)
+  if (infp)
     {
-      if (create_inheritable_pipe (inpipe, INHERIT_READ))
+      if (create_inheritable_pipe (inpipe, 0))
         {
           err = gpg_err_make (errsource, GPG_ERR_GENERAL);
           log_error (_("error creating a pipe: %s\n"), gpg_strerror (err));
@@ -424,7 +409,7 @@ gnupg_spawn_process (const char *pgmname, const char *argv[],
 
   if (r_outfp)
     {
-      if (create_inheritable_pipe (outpipe, INHERIT_WRITE))
+      if (create_inheritable_pipe (outpipe, 1))
         {
           err = gpg_err_make (errsource, GPG_ERR_GENERAL);
           log_error (_("error creating a pipe: %s\n"), gpg_strerror (err));
@@ -445,7 +430,7 @@ gnupg_spawn_process (const char *pgmname, const char *argv[],
           if (infp)
             es_fclose (infp);
           else if (inpipe[1] != INVALID_HANDLE_VALUE)
-            CloseHandle (inpipe[1]);
+            CloseHandle (outpipe[1]);
           if (inpipe[0] != INVALID_HANDLE_VALUE)
             CloseHandle (inpipe[0]);
           return err;
@@ -454,7 +439,7 @@ gnupg_spawn_process (const char *pgmname, const char *argv[],
 
   if (r_errfp)
     {
-      if (create_inheritable_pipe (errpipe, INHERIT_WRITE))
+      if (create_inheritable_pipe (errpipe, 1))
         {
           err = gpg_err_make (errsource, GPG_ERR_GENERAL);
           log_error (_("error creating a pipe: %s\n"), gpg_strerror (err));
@@ -481,7 +466,7 @@ gnupg_spawn_process (const char *pgmname, const char *argv[],
           if (infp)
             es_fclose (infp);
           else if (inpipe[1] != INVALID_HANDLE_VALUE)
-            CloseHandle (inpipe[1]);
+            CloseHandle (outpipe[1]);
           if (inpipe[0] != INVALID_HANDLE_VALUE)
             CloseHandle (inpipe[0]);
           return err;
@@ -498,12 +483,12 @@ gnupg_spawn_process (const char *pgmname, const char *argv[],
   if (err)
     return err;
 
-  if (inpipe[0] == INVALID_HANDLE_VALUE)
+  if (inpipe[0] != INVALID_HANDLE_VALUE)
     nullhd[0] = w32_open_null (0);
-  if (outpipe[1] == INVALID_HANDLE_VALUE)
-    nullhd[1] = w32_open_null (1);
-  if (errpipe[1] == INVALID_HANDLE_VALUE)
-    nullhd[2] = w32_open_null (1);
+  if (outpipe[1] != INVALID_HANDLE_VALUE)
+    nullhd[1] = w32_open_null (0);
+  if (errpipe[1] != INVALID_HANDLE_VALUE)
+    nullhd[2] = w32_open_null (0);
 
   /* Start the process.  Note that we can't run the PREEXEC function
      because this might change our own environment. */
@@ -692,86 +677,63 @@ gnupg_spawn_process_fd (const char *pgmname, const char *argv[],
 gpg_error_t
 gnupg_wait_process (const char *pgmname, pid_t pid, int hang, int *r_exitcode)
 {
-  return gnupg_wait_processes (&pgmname, &pid, 1, hang, r_exitcode);
-}
-
-/* See exechelp.h for a description.  */
-gpg_error_t
-gnupg_wait_processes (const char **pgmnames, pid_t *pids, size_t count,
-                      int hang, int *r_exitcodes)
-{
-  gpg_err_code_t ec = 0;
-  size_t i;
-  HANDLE *procs;
+  gpg_err_code_t ec;
+  HANDLE proc = fd_to_handle (pid);
   int code;
+  DWORD exc;
 
-  procs = xtrycalloc (count, sizeof *procs);
-  if (procs == NULL)
-    return gpg_error_from_syserror ();
+  if (r_exitcode)
+    *r_exitcode = -1;
 
-  for (i = 0; i < count; i++)
-    {
-      if (r_exitcodes)
-        r_exitcodes[i] = -1;
-
-      if (pids[i] == (pid_t)(-1))
-        return gpg_error (GPG_ERR_INV_VALUE);
-
-      procs[i] = fd_to_handle (pids[i]);
-    }
+  if (pid == (pid_t)(-1))
+    return gpg_error (GPG_ERR_INV_VALUE);
 
   /* FIXME: We should do a pth_waitpid here.  However this has not yet
      been implemented.  A special W32 pth system call would even be
      better.  */
-  code = WaitForMultipleObjects (count, procs, TRUE, hang? INFINITE : 0);
+  code = WaitForSingleObject (proc, hang? INFINITE : 0);
   switch (code)
     {
     case WAIT_TIMEOUT:
       ec = GPG_ERR_TIMEOUT;
-      goto leave;
+      break;
 
     case WAIT_FAILED:
-      log_error (_("waiting for processes to terminate failed: %s\n"),
-                 w32_strerror (-1));
+      log_error (_("waiting for process %d to terminate failed: %s\n"),
+                 (int)pid, w32_strerror (-1));
       ec = GPG_ERR_GENERAL;
-      goto leave;
+      break;
 
     case WAIT_OBJECT_0:
-      for (i = 0; i < count; i++)
+      if (!GetExitCodeProcess (proc, &exc))
         {
-          DWORD exc;
-
-          if (! GetExitCodeProcess (procs[i], &exc))
-            {
-              log_error (_("error getting exit code of process %d: %s\n"),
-                         (int) pids[i], w32_strerror (-1) );
-              ec = GPG_ERR_GENERAL;
-            }
-          else if (exc)
-            {
-              if (!r_exitcodes)
-                log_error (_("error running '%s': exit status %d\n"),
-                           pgmnames[i], (int)exc);
-              else
-                r_exitcodes[i] = (int)exc;
-              ec = GPG_ERR_GENERAL;
-            }
-          else
-            {
-              if (r_exitcodes)
-                r_exitcodes[i] = 0;
-            }
+          log_error (_("error getting exit code of process %d: %s\n"),
+                     (int)pid, w32_strerror (-1) );
+          ec = GPG_ERR_GENERAL;
+        }
+      else if (exc)
+        {
+          log_error (_("error running '%s': exit status %d\n"),
+                     pgmname, (int)exc );
+          if (r_exitcode)
+            *r_exitcode = (int)exc;
+          ec = GPG_ERR_GENERAL;
+        }
+      else
+        {
+          if (r_exitcode)
+            *r_exitcode = 0;
+          ec = 0;
         }
       break;
 
     default:
-      log_error ("WaitForMultipleObjects returned unexpected "
-                 "code %d\n", code);
+      log_error ("WaitForSingleObject returned unexpected "
+                 "code %d for pid %d\n", code, (int)pid );
       ec = GPG_ERR_GENERAL;
       break;
     }
 
- leave:
   return gpg_err_make (GPG_ERR_SOURCE_DEFAULT, ec);
 }
 
@@ -867,7 +829,6 @@ gnupg_spawn_process_detached (const char *pgmname, const char *argv[],
 /*              (int) pi.dwProcessId, (int) pi.dwThreadId); */
 
   CloseHandle (pi.hThread);
-  CloseHandle (pi.hProcess);
 
   return 0;
 }
