@@ -26,7 +26,7 @@
  *
  * You should have received a copies of the GNU General Public License
  * and the GNU Lesser General Public License along with this program;
- * if not, see <https://www.gnu.org/licenses/>.
+ * if not, see <http://www.gnu.org/licenses/>.
  */
 
 #ifndef GNUPG_COMMON_LOGGING_H
@@ -42,7 +42,6 @@ int  log_get_errorcount (int clear);
 void log_inc_errorcount (void);
 void log_set_file( const char *name );
 void log_set_fd (int fd);
-void log_set_socket_dir_cb (const char *(*fnc)(void));
 void log_set_pid_suffix_cb (int (*cb)(unsigned long *r_value));
 void log_set_prefix (const char *text, unsigned int flags);
 const char *log_get_prefix (unsigned int *flags);
@@ -53,22 +52,11 @@ estream_t log_get_stream (void);
 #ifdef GPGRT_HAVE_MACRO_FUNCTION
   void bug_at (const char *file, int line, const char *func)
                GPGRT_ATTR_NORETURN;
-  void _log_assert (const char *expr, const char *file, int line,
-                    const char *func) GPGRT_ATTR_NORETURN;
-# define BUG() bug_at( __FILE__ , __LINE__, __FUNCTION__)
-# define log_assert(expr)    do {                               \
-    if (!(expr))                                                \
-      _log_assert (#expr, __FILE__, __LINE__, __FUNCTION__);    \
-  } while (0)
-#else /*!GPGRT_HAVE_MACRO_FUNCTION*/
-  void bug_at (const char *file, int line);
-  void _log_assert (const char *expr, const char *file, int line;
+# define BUG() bug_at( __FILE__ , __LINE__, __FUNCTION__ )
+#else
+  void bug_at( const char *file, int line );
 # define BUG() bug_at( __FILE__ , __LINE__ )
-# define log_assert(expr)    do {                               \
-    if (!(expr))                                                \
-      _log_assert (#expr, __FILE__, __LINE__);                  \
-  } while (0)
-#endif /*!GPGRT_HAVE_MACRO_FUNCTION*/
+#endif
 
 /* Flag values for log_set_prefix. */
 #define GPGRT_LOG_WITH_PREFIX  1
@@ -91,6 +79,8 @@ enum jnlib_log_levels {
 void log_log (int level, const char *fmt, ...) GPGRT_ATTR_PRINTF(2,3);
 void log_logv (int level, const char *fmt, va_list arg_ptr);
 void log_string (int level, const char *string);
+
+
 void log_bug (const char *fmt, ...)    GPGRT_ATTR_NR_PRINTF(1,2);
 void log_fatal (const char *fmt, ...)  GPGRT_ATTR_NR_PRINTF(1,2);
 void log_error (const char *fmt, ...)  GPGRT_ATTR_PRINTF(1,2);
