@@ -47,34 +47,6 @@ time_t timegm (struct tm *tm);
 #define DIM(v)		     (sizeof(v)/sizeof((v)[0]))
 #define DIMof(type,member)   DIM(((type *)0)->member)
 
-
-/* Replacements for macros not available with libgpg-error < 1.20.  */
-#ifndef GPGRT_GCC_VERSION
-
-# undef GPGRT_HAVE_PRAGMA_GCC_PUSH
-# if __GNUC__ > 2 || (__GNUC__ == 2 && __GNUC_MINOR__ >= 5 )
-#  define GPGRT_HAVE_MACRO_FUNCTION 1  /* __FUNCTION__ macro is available.  */
-#  define GPGRT_ATTR_NORETURN  __attribute__ ((noreturn))
-#  if __GNUC__ > 4 || (__GNUC__ == 4 && __GNUC_MINOR__ >= 4 )
-#    define GPGRT_HAVE_PRAGMA_GCC_PUSH 1
-#    define GPGRT_ATTR_PRINTF(f,a) \
-                  __attribute__ ((format (__gnu_printf__,f,a)))
-#    define GPGRT_ATTR_NR_PRINTF(f,a) \
-                  __attribute__ ((noreturn, format (__gnu_printf__,f,a)))
-#  else
-#    define GPGRT_ATTR_PRINTF(f, a) \
-                  __attribute__ ((format (printf,f,a)))
-#    define GPGRT_ATTR_NR_PRINTF(f, a) \
-                  __attribute__ ((noreturn, format (printf,f,a)))
-#  endif
-# else
-#  define GPGRT_ATTR_NORETURN
-#  define GPGRT_ATTR_PRINTF( f, a )
-#  define GPGRT_ATTR_NR_PRINTF( f, a )
-# endif
-
-#endif /*Older libgpg-error.  */
-
 /* To avoid that a compiler optimizes certain memset calls away, these
    macros may be used instead. */
 #define wipememory2(_ptr,_set,_len) do { \
