@@ -25,7 +25,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, see <https://www.gnu.org/licenses/>.
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
 
 #include <config.h>
@@ -35,7 +35,7 @@
 #include <assert.h>
 
 #include "util.h"
-#include "openpgpdefs.h"
+
 
 /* A table with all our supported OpenPGP curves.  */
 static struct {
@@ -43,11 +43,10 @@ static struct {
   const char *oidstr; /* IETF formatted OID.  */
   unsigned int nbits; /* Nominal bit length of the curve.  */
   const char *alias;  /* NULL or alternative name of the curve.  */
-  int pubkey_algo;    /* Required OpenPGP algo or 0 for ECDSA/ECDH.  */
 } oidtable[] = {
 
-  { "Curve25519", "1.3.6.1.4.1.3029.1.5.1", 255, "cv25519", PUBKEY_ALGO_ECDH },
-  { "Ed25519",    "1.3.6.1.4.1.11591.15.1", 255, "ed25519", PUBKEY_ALGO_EDDSA },
+  { "Curve25519",      "1.3.6.1.4.1.3029.1.5.1", 255, "cv25519" },
+  { "Ed25519",         "1.3.6.1.4.1.11591.15.1", 255, "ed25519" },
 
   { "NIST P-256",      "1.2.840.10045.3.1.7",    256, "nistp256" },
   { "NIST P-384",      "1.3.132.0.34",           384, "nistp384" },
@@ -68,7 +67,7 @@ static const char oid_ed25519[] =
   { 0x09, 0x2b, 0x06, 0x01, 0x04, 0x01, 0xda, 0x47, 0x0f, 0x01 };
 
 /* The OID for Curve25519 in OpenPGP format.  */
-static const char oid_cv25519[] =
+static const char oid_crv25519[] =
   { 0x0a, 0x2b, 0x06, 0x01, 0x04, 0x01, 0x97, 0x55, 0x01, 0x05, 0x01 };
 
 
@@ -298,7 +297,7 @@ openpgp_oid_is_ed25519 (gcry_mpi_t a)
 
 
 int
-openpgp_oid_is_cv25519 (gcry_mpi_t a)
+openpgp_oid_is_crv25519 (gcry_mpi_t a)
 {
   const unsigned char *buf;
   unsigned int nbits;
@@ -309,8 +308,8 @@ openpgp_oid_is_cv25519 (gcry_mpi_t a)
 
   buf = gcry_mpi_get_opaque (a, &nbits);
   n = (nbits+7)/8;
-  return (n == DIM (oid_cv25519)
-          && !memcmp (buf, oid_cv25519, DIM (oid_cv25519)));
+  return (n == DIM (oid_crv25519)
+          && !memcmp (buf, oid_crv25519, DIM (oid_crv25519)));
 }
 
 
@@ -409,35 +408,3 @@ openpgp_enum_curves (int *iterp)
   *iterp = idx;
   return NULL;
 }
-
-
-/* Return the Libgcrypt name for the gpg curve NAME if supported.  If
- * R_ALGO is not NULL the required OpenPGP public key algo or 0 is
- * stored at that address.  If R_NBITS is not NULL the nominal bitsize
- * of the curves is stored there.  NULL is returned if the curve is
- * not supported. */
-const char *
-openpgp_is_curve_supported (const char *name, int *r_algo,
-                            unsigned int *r_nbits)
-{
-  int idx;
-
-  if (r_algo)
-    *r_algo = 0;
-  if (r_nbits)
-    *r_nbits = 0;
-  for (idx = 0; idx < DIM (oidtable) && oidtable[idx].name; idx++)
-    {
-      if ((!strcmp (name, oidtable[idx].name)
-           || (oidtable[idx].alias && !strcmp (name, (oidtable[idx].alias))))
-          && curve_supported_p (oidtable[idx].name))
-        {
-          if (r_algo)
-            *r_algo = oidtable[idx].pubkey_algo;
-          if (r_nbits)
-            *r_nbits = oidtable[idx].nbits;
-          return oidtable[idx].name;
-        }
-    }
-  return NULL;
-}
