@@ -56,34 +56,6 @@ typedef enum
   }
 pkttype_t;
 
-static inline const char *
-pkttype_str (pkttype_t type)
-{
-  switch (type)
-    {
-    case PKT_PUBKEY_ENC: return "PUBKEY_ENC";
-    case PKT_SIGNATURE: return "SIGNATURE";
-    case PKT_SYMKEY_ENC: return "SYMKEY_ENC";
-    case PKT_ONEPASS_SIG: return "ONEPASS_SIG";
-    case PKT_SECRET_KEY: return "SECRET_KEY";
-    case PKT_PUBLIC_KEY: return "PUBLIC_KEY";
-    case PKT_SECRET_SUBKEY: return "SECRET_SUBKEY";
-    case PKT_COMPRESSED: return "COMPRESSED";
-    case PKT_ENCRYPTED: return "ENCRYPTED";
-    case PKT_MARKER: return "MARKER";
-    case PKT_PLAINTEXT: return "PLAINTEXT";
-    case PKT_RING_TRUST: return "RING_TRUST";
-    case PKT_USER_ID: return "USER_ID";
-    case PKT_PUBLIC_SUBKEY: return "PUBLIC_SUBKEY";
-    case PKT_OLD_COMMENT: return "OLD_COMMENT";
-    case PKT_ATTRIBUTE: return "ATTRIBUTE";
-    case PKT_ENCRYPTED_MDC: return "ENCRYPTED_MDC";
-    case PKT_MDC: return "MDC";
-    case PKT_COMMENT: return "COMMENT";
-    case PKT_GPG_CONTROL: return "GPG_CONTROL";
-    default: return "unknown packet type";
-    }
-}
 
 typedef enum
   {
@@ -106,7 +78,7 @@ typedef enum
     SIGSUBPKT_PREF_HASH     = 21, /* Preferred hash algorithms. */
     SIGSUBPKT_PREF_COMPR    = 22, /* Preferred compression algorithms. */
     SIGSUBPKT_KS_FLAGS	    = 23, /* Key server preferences. */
-    SIGSUBPKT_PREF_KS	    = 24, /* Preferred keyserver. */
+    SIGSUBPKT_PREF_KS	    = 24, /* Preferred key server. */
     SIGSUBPKT_PRIMARY_UID   = 25, /* Primary user id. */
     SIGSUBPKT_POLICY	    = 26, /* Policy URL. */
     SIGSUBPKT_KEY_FLAGS     = 27, /* Key flags. */
@@ -115,7 +87,6 @@ typedef enum
     SIGSUBPKT_FEATURES      = 30, /* Feature flags. */
 
     SIGSUBPKT_SIGNATURE     = 32, /* Embedded signature. */
-    SIGSUBPKT_ISSUER_FPR    = 33, /* EXPERIMENTAL: Issuer fingerprint. */
 
     SIGSUBPKT_FLAG_CRITICAL = 128
   }
