@@ -33,7 +33,6 @@
 #ifndef GNUPG_COMMON_STRINGHELP_H
 #define GNUPG_COMMON_STRINGHELP_H
 
-#include <stdint.h>
 #include "types.h"
 
 /*-- stringhelp.c --*/
@@ -60,9 +59,11 @@ char *make_absfilename_try (const char *first_part,
                             ...) GPGRT_ATTR_SENTINEL(0);
 int compare_filenames( const char *a, const char *b );
 
-uint64_t string_to_u64 (const char *string);
 int hextobyte (const char *s);
 
+char *sanitize_buffer (const void *p, size_t n, int delim);
+
+
 size_t utf8_charcount (const char *s, int len);
 
 
@@ -147,10 +148,6 @@ char **strsplit (char *string, char delim, char replacement, int *count);
 /* Tokenize STRING using the set of delimiters in DELIM.  */
 char **strtokenize (const char *string, const char *delim);
 
-/* Split STRING into space delimited fields and store them in the
- * provided ARRAY.  */
-int split_fields (char *string, char **array, int arraysize);
-
 /* Return True if MYVERSION is greater or equal than REQ_VERSION.  */
 int compare_version_strings (const char *my_version, const char *req_version);
 
