@@ -15,7 +15,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, see <https://www.gnu.org/licenses/>.
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
 
 /*
@@ -44,7 +44,7 @@
         Field 1: Constant "v"
         Field 2: Version number of this file.  Must be 1.
 
-        This record must be the first non-comment record and
+        This record must be the first non-comment record record and
         there shall only exist one record of this type.
 
    1.3. CRL cache record
@@ -115,7 +115,7 @@
 #include "cdb.h"
 
 /* Change this whenever the format changes */
-#define DBDIR_D "crls.d"
+#define DBDIR_D (opt.system_daemon? "crls.d" : "dirmngr-cache.d")
 #define DBDIRFILE "DIR.txt"
 #define DBDIRVERSION 1
 
@@ -127,7 +127,7 @@
 
 
 static const char oidstr_crlNumber[] = "2.5.29.20";
-/* static const char oidstr_issuingDistributionPoint[] = "2.5.29.28"; */
+static const char oidstr_issuingDistributionPoint[] = "2.5.29.28";
 static const char oidstr_authorityKeyIdentifier[] = "2.5.29.35";
 
 
@@ -1851,9 +1851,7 @@ crl_parse_insert (ctrl_t ctrl, ksba_crl_t crl,
 	    md = NULL;
 
             err = validate_cert_chain (ctrl, crlissuer_cert, NULL,
-                                       (VALIDATE_FLAG_TRUST_CONFIG
-                                        | VALIDATE_FLAG_CRL
-                                        | VALIDATE_FLAG_RECURSIVE),
+                                       VALIDATE_MODE_CRL_RECURSIVE,
                                        r_trust_anchor);
             if (err)
               {
@@ -2363,7 +2361,7 @@ list_one_crl_entry (crl_cache_t cache, crl_cache_entry_t e, estream_t fp)
       if (reason & KSBA_CRLREASON_AFFILIATION_CHANGED )
         es_fputs( "affiliation_changed ", fp ), any = 1;
       if (reason & KSBA_CRLREASON_SUPERSEDED )
-        es_fputs( "superseded", fp ), any = 1;
+        es_fputs( "superseeded", fp ), any = 1;
       if (reason & KSBA_CRLREASON_CESSATION_OF_OPERATION )
         es_fputs( "cessation_of_operation", fp ), any = 1;
       if (reason & KSBA_CRLREASON_CERTIFICATE_HOLD )
@@ -2410,7 +2408,7 @@ crl_cache_load (ctrl_t ctrl, const char *filename)
   estream_t fp;
   ksba_reader_t reader;
 
-  fp = es_fopen (filename, "rb");
+  fp = es_fopen (filename, "r");
   if (!fp)
     {
       err = gpg_error_from_errno (errno);
