@@ -15,7 +15,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, see <https://www.gnu.org/licenses/>.
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
 
 #include <config.h>
@@ -38,9 +38,9 @@
 #include "../common/strlist.h"
 #include "../common/asshelp.h"
 
-#include "../common/i18n.h"
-#include "../common/util.h"
-#include "../common/init.h"
+#include "i18n.h"
+#include "util.h"
+#include "init.h"
 
 
 /* Constants for the options.  */
@@ -116,25 +116,6 @@ static unsigned char bintoasc[] = "ABCDEFGHIJKLMNOPQRSTUVWXYZ"
 static unsigned char asctobin[256]; /* runtime initialized */
 
 
-/* Build the helptable for radix64 to bin conversion. */
-static void
-init_asctobin (void)
-{
-  static int initialized;
-  int i;
-  unsigned char *s;
-
-  if (initialized)
-    return;
-  initialized = 1;
-
-  for (i=0; i < 256; i++ )
-    asctobin[i] = 255; /* Used to detect invalid characters. */
-  for (s=bintoasc, i=0; *s; s++, i++)
-    asctobin[*s] = i;
-}
-
-
 /* Prototypes.  */
 static gpg_error_t read_certificate (const char *fname,
                                      unsigned char **rbuf, size_t *rbuflen);
@@ -253,6 +234,19 @@ main (int argc, char **argv )
   if (log_get_errorcount (0))
     exit (2);
 
+  /* Build the helptable for radix64 to bin conversion. */
+  if (opt.pem)
+    {
+      int i;
+      unsigned char *s;
+
+      for (i=0; i < 256; i++ )
+        asctobin[i] = 255; /* Used to detect invalid characters. */
+      for (s=bintoasc, i=0; *s; s++, i++)
+        asctobin[*s] = i;
+    }
+
+
   if (cmd_ping)
     err = 0;
   else if (cmd_lookup || cmd_loadcrl)
@@ -298,6 +292,7 @@ main (int argc, char **argv )
 
   err = start_new_dirmngr (&ctx,
                            GPG_ERR_SOURCE_DEFAULT,
+                           default_homedir (),
                            opt.dirmngr_program
                              ? opt.dirmngr_program
                              : gnupg_module_name (GNUPG_MODULE_NAME_DIRMNGR),
@@ -466,8 +461,6 @@ read_pem_certificate (const char *fname, unsigned char **rbuf, size_t *rbuflen)
     s_waitend
   } state = s_init;
 
-  init_asctobin ();
-
   fp = fname? fopen (fname, "r") : stdin;
   if (!fp)
     return gpg_error_from_errno (errno);
@@ -619,15 +612,6 @@ read_certificate (const char *fname, unsigned char **rbuf, size_t *rbuflen)
 
   if (opt.pem)
     return read_pem_certificate (fname, rbuf, rbuflen);
-  else if (fname)
-    {
-      /* A filename has been given.  Let's just assume it is in PEM
-         format and decode it, and fall back to interpreting it as
-         binary certificate if that fails.  */
-      err = read_pem_certificate (fname, rbuf, rbuflen);
-      if (! err)
-        return 0;
-    }
 
   fp = fname? fopen (fname, "rb") : stdin;
   if (!fp)
