@@ -15,7 +15,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, see <https://www.gnu.org/licenses/>.
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
 
 #include <config.h>
@@ -56,8 +56,8 @@
 #include "../common/mischelp.h"
 #include "../common/strlist.h"
 
-#include "../common/i18n.h"
-#include "../common/util.h"
+#include "i18n.h"
+#include "util.h"
 #include "../common/init.h"
 
 /* With the ldap wrapper, there is no need for the npth_unprotect and leave
@@ -150,7 +150,7 @@ static ARGPARSE_OPTS opts[] = {
   { oAttr,     "attr",      2, N_("|STRING|return the attribute STRING")},
   { oOnlySearchTimeout, "only-search-timeout", 0, "@"},
   { oLogWithPID,"log-with-pid", 0, "@"},
-  ARGPARSE_end ()
+  { 0, NULL, 0, NULL }
 };
 
 
@@ -641,23 +641,10 @@ fetch_ldap (my_opt_t myopt, const char *url, const LDAPURLDesc *ludp)
   /* Fixme:  Can we use MYOPT->user or is it shared with other theeads?.  */
   ret = my_ldap_simple_bind_s (ld, myopt->user, myopt->pass);
   npth_protect ();
-#ifdef LDAP_VERSION3
-  if (ret == LDAP_PROTOCOL_ERROR)
-    {
-      /* Protocol error could mean that the server only supports v3. */
-      int version = LDAP_VERSION3;
-      if (myopt->verbose)
-        log_info ("protocol error; retrying bind with v3 protocol\n");
-      npth_unprotect ();
-      ldap_set_option (ld, LDAP_OPT_PROTOCOL_VERSION, &version);
-      ret = my_ldap_simple_bind_s (ld, myopt->user, myopt->pass);
-      npth_protect ();
-    }
-#endif
   if (ret)
     {
       log_error (_("binding to '%s:%d' failed: %s\n"),
-                 host, port, ldap_err2string (ret));
+                 host, port, strerror (errno));
       ldap_unbind (ld);
       return -1;
     }
