@@ -15,7 +15,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, see <https://www.gnu.org/licenses/>.
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
 
 #include <config.h>
@@ -37,7 +37,7 @@
 
 #include "dirmngr.h"
 #include "misc.h"
-#include "../common/userids.h"
+#include "userids.h"
 #include "dns-stuff.h"
 #include "ks-engine.h"
 
@@ -82,13 +82,16 @@ struct hostinfo_s
   unsigned int v6:1; /* Host supports AF_INET6.  */
   unsigned int onion:1;/* NAME is an onion (Tor HS) address.  */
   unsigned int dead:1; /* Host is currently unresponsive.  */
-  unsigned int iporname_valid:1;  /* The field IPORNAME below is valid */
-                                  /* (but may be NULL) */
   time_t died_at;    /* The time the host was marked dead.  If this is
                         0 the host has been manually marked dead.  */
   char *cname;       /* Canonical name of the host.  Only set if this
-                        is a pool or NAME has a numerical IP address.  */
-  char *iporname;    /* Numeric IP address or name for printing.  */
+                        is a pool.  */
+  char *v4addr;      /* A string with the v4 IP address of the host.
+                        NULL if NAME has a numeric IP address or no v4
+                        address is available.  */
+  char *v6addr;      /* A string with the v6 IP address of the host.
+                        NULL if NAME has a numeric IP address or no v6
+                        address is available.  */
   unsigned short port; /* The port used by the host, 0 if unknown.  */
   char name[1];      /* The hostname.  */
 };
@@ -125,10 +128,10 @@ create_new_hostinfo (const char *name)
   hi->v6 = 0;
   hi->onion = 0;
   hi->dead = 0;
-  hi->iporname_valid = 0;
   hi->died_at = 0;
   hi->cname = NULL;
-  hi->iporname = NULL;
+  hi->v4addr = NULL;
+  hi->v6addr = NULL;
   hi->port = 0;
 
   /* Add it to the hosttable. */
@@ -255,31 +258,6 @@ arecords_is_pool (dns_addrinfo_t aibuf)
 }
 
 
-/* Print a warninng iff Tor is not running but Tor has been requested.
- * Also return true if it is not running.  */
-static int
-tor_not_running_p (ctrl_t ctrl)
-{
-  assuan_fd_t sock;
-
-  if (!dirmngr_use_tor ())
-    return 0;
-
-  sock = assuan_sock_connect_byname (NULL, 0, 0, NULL, ASSUAN_SOCK_TOR);
-  if (sock != ASSUAN_INVALID_FD)
-    {
-      assuan_sock_close (sock);
-      return 0;
-    }
-
-  log_info ("(it seems Tor is not running)\n");
-  dirmngr_status (ctrl, "WARNING", "tor_not_running 0",
-                  "Tor is enabled but the local Tor daemon"
-                  " seems to be down", NULL);
-  return 1;
-}
-
-
 /* Add the host AI under the NAME into the HOSTTABLE.  If PORT is not
    zero, it specifies which port to use to talk to the host.  If NAME
    specifies a pool (as indicated by IS_POOL), update the given
@@ -292,20 +270,15 @@ add_host (const char *name, int is_pool,
   gpg_error_t tmperr;
   char *tmphost;
   int idx, tmpidx;
+  int is_numeric = 0;
   int i;
 
   idx = find_hostinfo (name);
 
-  if (is_pool)
+  if (!is_pool && !is_ip_address (name))
     {
-      /* For a pool immediately convert the address to a string.  */
-      tmperr = resolve_dns_addr (ai->addr, ai->addrlen,
-                                 (DNS_NUMERICHOST | DNS_WITHBRACKET), &tmphost);
-    }
-  else if (!is_ip_address (name))
-    {
-      /* This is a hostname.  Use the name as given without going
-       * through resolve_dns_addr.  */
+      /* This is a hostname but not a pool.  Use the name
+         as given without going through resolve_dns_addr.  */
       tmphost = xtrystrdup (name);
       if (!tmphost)
         tmperr = gpg_error_from_syserror ();
@@ -314,10 +287,10 @@ add_host (const char *name, int is_pool,
     }
   else
     {
-      /* Do a PTR lookup on AI.  If a name was not found the function
-       * returns the numeric address (with brackets).  */
       tmperr = resolve_dns_addr (ai->addr, ai->addrlen,
                                  DNS_WITHBRACKET, &tmphost);
+      if (tmphost && is_ip_address (tmphost))
+        is_numeric = 1;
     }
 
   if (tmperr)
@@ -346,21 +319,45 @@ add_host (const char *name, int is_pool,
 
       if (tmpidx == -1)
         {
-          log_error ("map_host for '%s' problem: %s - '%s' [ignored]\n",
+          log_error ("map_host for '%s' problem: %s - '%s'"
+                     " [ignored]\n",
                      name, strerror (errno), tmphost);
         }
       else  /* Set or update the entry. */
         {
+          char *ipaddr = NULL;
+
           if (port)
             hosttable[tmpidx]->port = port;
 
+          if (!is_numeric)
+            {
+              xfree (tmphost);
+              tmperr = resolve_dns_addr (ai->addr, ai->addrlen,
+                                         (DNS_NUMERICHOST
+                                          | DNS_WITHBRACKET),
+                                         &tmphost);
+              if (tmperr)
+                log_info ("resolve_dns_addr failed: %s\n",
+                          gpg_strerror (tmperr));
+              else
+                {
+                  ipaddr = tmphost;
+                  tmphost = NULL;
+                }
+            }
+
           if (ai->family == AF_INET6)
             {
               hosttable[tmpidx]->v6 = 1;
+              xfree (hosttable[tmpidx]->v6addr);
+              hosttable[tmpidx]->v6addr = ipaddr;
             }
           else if (ai->family == AF_INET)
             {
               hosttable[tmpidx]->v4 = 1;
+              xfree (hosttable[tmpidx]->v4addr);
+              hosttable[tmpidx]->v4addr = ipaddr;
             }
           else
             BUG ();
@@ -377,24 +374,21 @@ add_host (const char *name, int is_pool,
 
 
 /* Map the host name NAME to the actual to be used host name.  This
- * allows us to manage round robin DNS names.  We use our own strategy
- * to choose one of the hosts.  For example we skip those hosts which
- * failed for some time and we stick to one host for a time
- * independent of DNS retry times.  If FORCE_RESELECT is true a new
- * host is always selected.  If SRVTAG is NULL no service record
- * lookup will be done, if it is set that service name is used.  The
- * selected host is stored as a malloced string at R_HOST; on error
- * NULL is stored.  If we know the port used by the selected host from
- * a service record, a string representation is written to R_PORTSTR,
- * otherwise it is left untouched.  If R_HTTPFLAGS is not NULL it will
- * receive flags which are to be passed to http_open.  If R_HTTPHOST
- * is not NULL a malloced name of the host is stored there; this might
- * be different from R_HOST in case it has been selected from a
- * pool.  */
+   allows us to manage round robin DNS names.  We use our own strategy
+   to choose one of the hosts.  For example we skip those hosts which
+   failed for some time and we stick to one host for a time
+   independent of DNS retry times.  If FORCE_RESELECT is true a new
+   host is always selected.  The selected host is stored as a malloced
+   string at R_HOST; on error NULL is stored.  If we know the port
+   used by the selected host, a string representation is written to
+   R_PORTSTR, otherwise it is left untouched.  If R_HTTPFLAGS is not
+   NULL it will receive flags which are to be passed to http_open.  If
+   R_POOLNAME is not NULL a malloced name of the pool is stored or
+   NULL if it is not a pool. */
 static gpg_error_t
-map_host (ctrl_t ctrl, const char *name, const char *srvtag, int force_reselect,
+map_host (ctrl_t ctrl, const char *name, int force_reselect,
           char **r_host, char *r_portstr,
-          unsigned int *r_httpflags, char **r_httphost)
+          unsigned int *r_httpflags, char **r_poolname)
 {
   gpg_error_t err = 0;
   hostinfo_t hi;
@@ -403,8 +397,8 @@ map_host (ctrl_t ctrl, const char *name, const char *srvtag, int force_reselect,
   *r_host = NULL;
   if (r_httpflags)
     *r_httpflags = 0;
-  if (r_httphost)
-    *r_httphost = NULL;
+  if (r_poolname)
+    *r_poolname = NULL;
 
   /* No hostname means localhost.  */
   if (!name || !*name)
@@ -432,8 +426,11 @@ map_host (ctrl_t ctrl, const char *name, const char *srvtag, int force_reselect,
       int refidx;
       int is_pool = 0;
       char *cname;
+#ifdef	USE_DNS_SRV
+      char *srvrecord;
       struct srventry *srvs;
-      unsigned int srvscount;
+      int srvscount;
+#endif	/* USE_DNS_SRV */
 
       reftblsize = 100;
       reftbl = xtrymalloc (reftblsize * sizeof *reftbl);
@@ -450,39 +447,46 @@ map_host (ctrl_t ctrl, const char *name, const char *srvtag, int force_reselect,
         }
       hi = hosttable[idx];
 
-      if (srvtag && !is_ip_address (name))
+#ifdef	USE_DNS_SRV
+      /* Check for SRV records.  */
+      srvrecord = xtryasprintf ("_hkp._tcp.%s", name);
+      if (srvrecord == NULL)
         {
-          /* Check for SRV records.  */
-          err = get_dns_srv (name, srvtag, NULL, &srvs, &srvscount);
-          if (err)
-            {
-              xfree (reftbl);
-              if (gpg_err_code (err) == GPG_ERR_ECONNREFUSED)
-                tor_not_running_p (ctrl);
-              return err;
-            }
-
-          if (srvscount > 0)
-            {
-              int i;
-              is_pool = srvscount > 1;
-
-              for (i = 0; i < srvscount; i++)
-                {
-                  err = resolve_dns_name (srvs[i].target, 0,
-                                          AF_UNSPEC, SOCK_STREAM,
-                                          &ai, &cname);
-                  if (err)
-                    continue;
-                  dirmngr_tick (ctrl);
-                  add_host (name, is_pool, ai, srvs[i].port,
-                            reftbl, reftblsize, &refidx);
-                }
-
-              xfree (srvs);
-            }
+          err = gpg_error_from_syserror ();
+          xfree (reftbl);
+          return err;
         }
 
+      srvscount = getsrv (srvrecord, &srvs);
+      xfree (srvrecord);
+      if (srvscount < 0)
+        {
+          err = gpg_error_from_syserror ();
+          xfree (reftbl);
+          return err;
+        }
+
+      if (srvscount > 0)
+        {
+          int i;
+          is_pool = srvscount > 1;
+
+          for (i = 0; i < srvscount; i++)
+            {
+              err = resolve_dns_name (srvs[i].target, 0,
+                                      AF_UNSPEC, SOCK_STREAM,
+                                      &ai, &cname);
+              if (err)
+                continue;
+              dirmngr_tick (ctrl);
+              add_host (name, is_pool, ai, srvs[i].port,
+                        reftbl, reftblsize, &refidx);
+            }
+
+          xfree (srvs);
+        }
+#endif	/* USE_DNS_SRV */
+
       /* Find all A records for this entry and put them into the pool
          list - if any.  */
       err = resolve_dns_name (name, 0, 0, SOCK_STREAM, &aibuf, &cname);
@@ -510,10 +514,6 @@ map_host (ctrl_t ctrl, const char *name, const char *srvtag, int force_reselect,
             {
               if (ai->family != AF_INET && ai->family != AF_INET6)
                 continue;
-              if (opt.disable_ipv4 && ai->family == AF_INET)
-                continue;
-              if (opt.disable_ipv6 && ai->family == AF_INET6)
-                continue;
               dirmngr_tick (ctrl);
 
               add_host (name, is_pool, ai, 0, reftbl, reftblsize, &refidx);
@@ -545,10 +545,10 @@ map_host (ctrl_t ctrl, const char *name, const char *srvtag, int force_reselect,
   if (hi->pool)
     {
       /* Deal with the pool name before selecting a host. */
-      if (r_httphost)
+      if (r_poolname)
         {
-          *r_httphost = xtrystrdup (hi->cname? hi->cname : hi->name);
-          if (!*r_httphost)
+          *r_poolname = xtrystrdup (hi->cname? hi->cname : hi->name);
+          if (!*r_poolname)
             return gpg_error_from_syserror ();
         }
 
@@ -567,10 +567,10 @@ map_host (ctrl_t ctrl, const char *name, const char *srvtag, int force_reselect,
           if (hi->poolidx == -1)
             {
               log_error ("no alive host found in pool '%s'\n", name);
-              if (r_httphost)
+              if (r_poolname)
                 {
-                  xfree (*r_httphost);
-                  *r_httphost = NULL;
+                  xfree (*r_poolname);
+                  *r_poolname = NULL;
                 }
               return gpg_error (GPG_ERR_NO_KEYSERVER);
             }
@@ -580,43 +580,14 @@ map_host (ctrl_t ctrl, const char *name, const char *srvtag, int force_reselect,
       hi = hosttable[hi->poolidx];
       assert (hi);
     }
-  else if (r_httphost && is_ip_address (hi->name))
-    {
-      /* This is a numerical IP address and not a pool.  We want to
-       * find the canonical name so that it can be used in the HTTP
-       * Host header.  Fixme: We should store that name in the
-       * hosttable. */
-      dns_addrinfo_t aibuf, ai;
-      char *host;
-
-      err = resolve_dns_name (hi->name, 0, 0, SOCK_STREAM, &aibuf, NULL);
-      if (!err)
-        {
-          for (ai = aibuf; ai; ai = ai->next)
-            {
-              if ((!opt.disable_ipv6 && ai->family == AF_INET6)
-                  || (!opt.disable_ipv4 && ai->family == AF_INET))
-                {
-                  err = resolve_dns_addr (ai->addr, ai->addrlen, 0, &host);
-                  if (!err)
-                    {
-                      /* Okay, we return the first found name.  */
-                      *r_httphost = host;
-                      break;
-                    }
-                }
-            }
-        }
-      free_dns_addrinfo (aibuf);
-    }
 
   if (hi->dead)
     {
       log_error ("host '%s' marked as dead\n", hi->name);
-      if (r_httphost)
+      if (r_poolname)
         {
-          xfree (*r_httphost);
-          *r_httphost = NULL;
+          xfree (*r_poolname);
+          *r_poolname = NULL;
         }
       return gpg_error (GPG_ERR_NO_KEYSERVER);
     }
@@ -643,10 +614,10 @@ map_host (ctrl_t ctrl, const char *name, const char *srvtag, int force_reselect,
   if (!*r_host)
     {
       err = gpg_error_from_syserror ();
-      if (r_httphost)
+      if (r_poolname)
         {
-          xfree (*r_httphost);
-          *r_httphost = NULL;
+          xfree (*r_poolname);
+          *r_poolname = NULL;
         }
       return err;
     }
@@ -799,7 +770,6 @@ ks_hkp_print_hosttable (ctrl_t ctrl)
   if (err)
     return err;
 
-  /* FIXME: We need a lock for the hosttable.  */
   curtime = gnupg_get_time ();
   for (idx=0; idx < hosttable_size; idx++)
     if ((hi=hosttable[idx]))
@@ -811,82 +781,16 @@ ks_hkp_print_hosttable (ctrl_t ctrl)
           }
         else
           diedstr = died = NULL;
-
-        if (!hi->iporname_valid)
-          {
-            char *canon = NULL;
-
-            xfree (hi->iporname);
-            hi->iporname = NULL;
-
-            /* Do a lookup just for the display purpose.  */
-            if (hi->onion || hi->pool)
-              ;
-            else if (is_ip_address (hi->name))
-              {
-                dns_addrinfo_t aibuf, ai;
-
-                /* Turn the numerical IP address string into an AI and
-                 * then do a DNS PTR lookup.  */
-                if (!resolve_dns_name (hi->name, 0, 0,
-                                       SOCK_STREAM,
-                                       &aibuf, &canon))
-                  {
-                    if (canon && is_ip_address (canon))
-                      {
-                        xfree (canon);
-                        canon = NULL;
-                      }
-                    for (ai = aibuf; !canon && ai; ai = ai->next)
-                      {
-                        resolve_dns_addr (ai->addr, ai->addrlen,
-                                          DNS_WITHBRACKET, &canon);
-                        if (canon && is_ip_address (canon))
-                          {
-                            /* We already have the numeric IP - no need to
-                             * display it a second time.  */
-                            xfree (canon);
-                            canon = NULL;
-                          }
-                      }
-                  }
-                free_dns_addrinfo (aibuf);
-              }
-            else
-              {
-                dns_addrinfo_t aibuf, ai;
-
-                /* Get the IP address as a string from a name.  Note
-                 * that resolve_dns_addr allocates CANON on success
-                 * and thus terminates the loop. */
-                if (!resolve_dns_name (hi->name, 0,
-                                       hi->v6? AF_INET6 : AF_INET,
-                                       SOCK_STREAM,
-                                       &aibuf, NULL))
-                  {
-                    for (ai = aibuf; !canon && ai; ai = ai->next)
-                      {
-                        resolve_dns_addr (ai->addr, ai->addrlen,
-                                          DNS_NUMERICHOST|DNS_WITHBRACKET,
-                                          &canon);
-                      }
-                  }
-                free_dns_addrinfo (aibuf);
-              }
-
-            hi->iporname = canon;
-            hi->iporname_valid = 1;
-          }
-
-        err = ks_printf_help (ctrl, "%3d %s %s %s %s%s%s%s%s%s%s\n",
+        err = ks_printf_help (ctrl, "%3d %s %s %s %s%s%s%s%s%s%s%s\n",
                               idx,
                               hi->onion? "O" : hi->v6? "6":" ",
                               hi->v4? "4":" ",
                               hi->dead? "d":" ",
                               hi->name,
-                              hi->iporname? " (":"",
-                              hi->iporname? hi->iporname : "",
-                              hi->iporname? ")":"",
+                              hi->v6addr? " v6=":"",
+                              hi->v6addr? hi->v6addr:"",
+                              hi->v4addr? " v4=":"",
+                              hi->v4addr? hi->v4addr:"",
                               diedstr? "  (":"",
                               diedstr? diedstr:"",
                               diedstr? ")":""   );
@@ -928,7 +832,7 @@ ks_hkp_print_hosttable (ctrl_t ctrl)
 gpg_error_t
 ks_hkp_help (ctrl_t ctrl, parsed_uri_t uri)
 {
-  const char data[] =
+  const char const data[] =
     "Handler for HKP URLs:\n"
     "  hkp://\n"
 #if  HTTP_USE_GNUTLS || HTTP_USE_NTBTLS
@@ -956,65 +860,55 @@ ks_hkp_help (ctrl_t ctrl, parsed_uri_t uri)
 
 
 /* Build the remote part of the URL from SCHEME, HOST and an optional
- * PORT.  If NO_SRV is set no SRV record lookup will be done.  Returns
- * an allocated string at R_HOSTPORT or NULL on failure.  If
- * R_HTTPHOST is not NULL it receives a malloced string with the
- * hostname; this may be different from HOST if HOST is selected from
- * a pool.  */
+   PORT.  Returns an allocated string at R_HOSTPORT or NULL on failure
+   If R_POOLNAME is not NULL it receives a malloced string with the
+   poolname.  */
 static gpg_error_t
 make_host_part (ctrl_t ctrl,
                 const char *scheme, const char *host, unsigned short port,
-                int force_reselect, int no_srv,
-                char **r_hostport, unsigned int *r_httpflags, char **r_httphost)
+                int force_reselect,
+                char **r_hostport, unsigned int *r_httpflags, char **r_poolname)
 {
   gpg_error_t err;
-  const char *srvtag;
   char portstr[10];
   char *hostname;
 
   *r_hostport = NULL;
 
+  portstr[0] = 0;
+  err = map_host (ctrl, host, force_reselect,
+                  &hostname, portstr, r_httpflags, r_poolname);
+  if (err)
+    return err;
+
+  /* Map scheme and port.  */
   if (!strcmp (scheme, "hkps") || !strcmp (scheme,"https"))
     {
       scheme = "https";
-      srvtag = no_srv? NULL : "pgpkey-https";
+      if (! *portstr)
+        strcpy (portstr, "443");
     }
   else /* HKP or HTTP.  */
     {
       scheme = "http";
-      srvtag = no_srv? NULL : "pgpkey-http";
+      if (! *portstr)
+        strcpy (portstr, "11371");
+    }
+  if (port)
+    snprintf (portstr, sizeof portstr, "%hu", port);
+  else
+    {
+      /*fixme_do_srv_lookup ()*/
     }
 
-  portstr[0] = 0;
-  err = map_host (ctrl, host, srvtag, force_reselect,
-                  &hostname, portstr, r_httpflags, r_httphost);
-  if (err)
-    return err;
-
-  /* If map_host did not return a port (from a SRV record) but a port
-   * has been specified (implicitly or explicitly) then use that port.
-   * In the case that a port was not specified (which is probably a
-   * bug in https.c) we will set up defaults.  */
-  if (*portstr)
-    ;
-  else if (!*portstr && port)
-    snprintf (portstr, sizeof portstr, "%hu", port);
-  else if (!strcmp (scheme,"https"))
-    strcpy (portstr, "443");
-  else
-    strcpy (portstr, "11371");
-
-  if (*hostname != '[' && is_ip_address (hostname) == 6)
-    *r_hostport = strconcat (scheme, "://[", hostname, "]:", portstr, NULL);
-  else
-    *r_hostport = strconcat (scheme, "://", hostname, ":", portstr, NULL);
+  *r_hostport = strconcat (scheme, "://", hostname, ":", portstr, NULL);
   xfree (hostname);
   if (!*r_hostport)
     {
-      if (r_httphost)
+      if (r_poolname)
         {
-          xfree (*r_httphost);
-          *r_httphost = NULL;
+          xfree (*r_poolname);
+          *r_poolname = NULL;
         }
       return gpg_error_from_syserror ();
     }
@@ -1031,11 +925,7 @@ ks_hkp_resolve (ctrl_t ctrl, parsed_uri_t uri)
   gpg_error_t err;
   char *hostport = NULL;
 
-  /* NB: With an explicitly given port we do not want to consult a
-   * service record because that might be in conflict with the port
-   * from such a service record.  */
-  err = make_host_part (ctrl, uri->scheme, uri->host, uri->port,
-                        1, uri->explicit_port,
+  err = make_host_part (ctrl, uri->scheme, uri->host, uri->port, 1,
                         &hostport, NULL, NULL);
   if (err)
     {
@@ -1080,30 +970,6 @@ ks_hkp_housekeeping (time_t curtime)
 }
 
 
-/* Reload (SIGHUP) action for this module.  We mark all host alive
- * even those which have been manually shot.  */
-void
-ks_hkp_reload (void)
-{
-  int idx, count;
-  hostinfo_t hi;
-
-  for (idx=count=0; idx < hosttable_size; idx++)
-    {
-      hi = hosttable[idx];
-      if (!hi)
-        continue;
-      hi->iporname_valid = 0;
-      if (!hi->dead)
-        continue;
-      hi->dead = 0;
-      count++;
-    }
-  if (count)
-    log_info ("number of resurrected hosts: %d", count);
-}
-
-
 /* Send an HTTP request.  On success returns an estream object at
    R_FP.  HOSTPORTSTR is only used for diagnostics.  If HTTPHOST is
    not NULL it will be used as HTTP "Host" header.  If POST_CB is not
@@ -1125,10 +991,7 @@ send_request (ctrl_t ctrl, const char *request, const char *hostportstr,
 
   *r_fp = NULL;
 
-  err = http_session_new (&session, httphost,
-                          ((ctrl->http_no_crl? HTTP_FLAG_NO_CRL : 0)
-                           | HTTP_FLAG_TRUST_DEF),
-                          gnupg_http_tls_verify_cb, ctrl);
+  err = http_session_new (&session, NULL, httphost);
   if (err)
     goto leave;
   http_session_set_log_cb (session, cert_log_cb);
@@ -1141,9 +1004,7 @@ send_request (ctrl_t ctrl, const char *request, const char *hostportstr,
                    /* fixme: AUTH */ NULL,
                    (httpflags
                     |(opt.honor_http_proxy? HTTP_FLAG_TRY_PROXY:0)
-                    |(dirmngr_use_tor ()? HTTP_FLAG_FORCE_TOR:0)
-                    |(opt.disable_ipv4? HTTP_FLAG_IGNORE_IPv4 : 0)
-                    |(opt.disable_ipv6? HTTP_FLAG_IGNORE_IPv6 : 0)),
+                    |(opt.use_tor? HTTP_FLAG_FORCE_TOR:0)),
                    ctrl->http_proxy,
                    session,
                    NULL,
@@ -1261,32 +1122,23 @@ send_request (ctrl_t ctrl, const char *request, const char *hostportstr,
 }
 
 
-/* Helper to evaluate the error code ERR from a send_request() call
+/* Helper to evaluate the error code ERR form a send_request() call
    with REQUEST.  The function returns true if the caller shall try
    again.  TRIES_LEFT points to a variable to track the number of
    retries; this function decrements it and won't return true if it is
    down to zero. */
 static int
-handle_send_request_error (ctrl_t ctrl, gpg_error_t err, const char *request,
+handle_send_request_error (gpg_error_t err, const char *request,
                            unsigned int *tries_left)
 {
   int retry = 0;
 
-  /* Fixme: Should we disable all hosts of a protocol family if a
-   * request for an address of that familiy returned ENETDOWN?  */
-
   switch (gpg_err_code (err))
     {
     case GPG_ERR_ECONNREFUSED:
-      if (tor_not_running_p (ctrl))
-        break; /* A retry does not make sense.  */
-      /* Okay: Tor is up or --use-tor is not used.  */
-      /*FALLTHRU*/
     case GPG_ERR_ENETUNREACH:
-    case GPG_ERR_ENETDOWN:
     case GPG_ERR_UNKNOWN_HOST:
     case GPG_ERR_NETWORK:
-    case GPG_ERR_EIO:  /* Sometimes used by estream cookie functions.  */
       if (mark_host_dead (request) && *tries_left)
         retry = 1;
       break;
@@ -1297,17 +1149,6 @@ handle_send_request_error (ctrl_t ctrl, gpg_error_t err, const char *request,
           log_info ("selecting a different host due to a timeout\n");
           retry = 1;
         }
-      break;
-
-    case GPG_ERR_EACCES:
-      if (dirmngr_use_tor ())
-        {
-          log_info ("(Tor configuration problem)\n");
-          dirmngr_status (ctrl, "WARNING", "tor_config_problem 0",
-                          "Please check that the \"SocksPort\" flag "
-                          "\"IPv6Traffic\" is set in torrc", NULL);
-        }
-      break;
 
     default:
       break;
@@ -1366,16 +1207,12 @@ ks_hkp_search (ctrl_t ctrl, parsed_uri_t uri, const char *pattern,
       pattern = fprbuf;
       break;
     case KEYDB_SEARCH_MODE_FPR16:
-      fprbuf[0] = '0';
-      fprbuf[1] = 'x';
-      bin2hex (desc.u.fpr, 16, fprbuf+2);
+      bin2hex (desc.u.fpr, 16, fprbuf);
       pattern = fprbuf;
       break;
     case KEYDB_SEARCH_MODE_FPR20:
     case KEYDB_SEARCH_MODE_FPR:
-      fprbuf[0] = '0';
-      fprbuf[1] = 'x';
-      bin2hex (desc.u.fpr, 20, fprbuf+2);
+      bin2hex (desc.u.fpr, 20, fprbuf);
       pattern = fprbuf;
       break;
     default:
@@ -1390,8 +1227,7 @@ ks_hkp_search (ctrl_t ctrl, parsed_uri_t uri, const char *pattern,
 
     xfree (hostport); hostport = NULL;
     xfree (httphost); httphost = NULL;
-    err = make_host_part (ctrl, uri->scheme, uri->host, uri->port,
-                          reselect, uri->explicit_port,
+    err = make_host_part (ctrl, uri->scheme, uri->host, uri->port, reselect,
                           &hostport, &httpflags, &httphost);
     if (err)
       goto leave;
@@ -1419,7 +1255,7 @@ ks_hkp_search (ctrl_t ctrl, parsed_uri_t uri, const char *pattern,
   /* Send the request.  */
   err = send_request (ctrl, request, hostport, httphost, httpflags,
                       NULL, NULL, &fp, r_http_status);
-  if (handle_send_request_error (ctrl, err, request, &tries))
+  if (handle_send_request_error (err, request, &tries))
     {
       reselect = 1;
       goto again;
@@ -1515,7 +1351,6 @@ ks_hkp_get (ctrl_t ctrl, parsed_uri_t uri, const char *keyspec, estream_t *r_fp)
 
     case KEYDB_SEARCH_MODE_FPR16:
       log_error ("HKP keyservers do not support v3 fingerprints\n");
-      /* fall through */
     default:
       return gpg_error (GPG_ERR_INV_USER_ID);
     }
@@ -1533,8 +1368,7 @@ ks_hkp_get (ctrl_t ctrl, parsed_uri_t uri, const char *keyspec, estream_t *r_fp)
   /* Build the request string.  */
   xfree (hostport); hostport = NULL;
   xfree (httphost); httphost = NULL;
-  err = make_host_part (ctrl, uri->scheme, uri->host, uri->port,
-                        reselect, uri->explicit_port,
+  err = make_host_part (ctrl, uri->scheme, uri->host, uri->port, reselect,
                         &hostport, &httpflags, &httphost);
   if (err)
     goto leave;
@@ -1554,7 +1388,7 @@ ks_hkp_get (ctrl_t ctrl, parsed_uri_t uri, const char *keyspec, estream_t *r_fp)
   /* Send the request.  */
   err = send_request (ctrl, request, hostport, httphost, httpflags,
                       NULL, NULL, &fp, NULL);
-  if (handle_send_request_error (ctrl, err, request, &tries))
+  if (handle_send_request_error (err, request, &tries))
     {
       reselect = 1;
       goto again;
@@ -1646,8 +1480,7 @@ ks_hkp_put (ctrl_t ctrl, parsed_uri_t uri, const void *data, size_t datalen)
  again:
   xfree (hostport); hostport = NULL;
   xfree (httphost); httphost = NULL;
-  err = make_host_part (ctrl, uri->scheme, uri->host, uri->port,
-                        reselect, uri->explicit_port,
+  err = make_host_part (ctrl, uri->scheme, uri->host, uri->port, reselect,
                         &hostport, &httpflags, &httphost);
   if (err)
     goto leave;
@@ -1663,7 +1496,7 @@ ks_hkp_put (ctrl_t ctrl, parsed_uri_t uri, const void *data, size_t datalen)
   /* Send the request.  */
   err = send_request (ctrl, request, hostport, httphost, 0,
                       put_post_cb, &parm, &fp, NULL);
-  if (handle_send_request_error (ctrl, err, request, &tries))
+  if (handle_send_request_error (err, request, &tries))
     {
       reselect = 1;
       goto again;
