@@ -1,7 +1,7 @@
 /* server.c - LDAP and Keyserver access server
  * Copyright (C) 2002 Klarälvdalens Datakonsult AB
  * Copyright (C) 2003, 2004, 2005, 2007, 2008, 2009, 2011, 2015 g10 Code GmbH
- * Copyright (C) 2014, 2015, 2016 Werner Koch
+ * Copyright (C) 2014 Werner Koch
  *
  * This file is part of GnuPG.
  *
@@ -16,7 +16,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, see <https://www.gnu.org/licenses/>.
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
 
 #include <config.h>
@@ -53,7 +53,6 @@
 #include "dns-stuff.h"
 #include "mbox-util.h"
 #include "zb32.h"
-#include "server-help.h"
 
 /* To avoid DoS attacks we limit the size of a certificate to
    something reasonable.  The DoS was actually only an issue back when
@@ -95,13 +94,6 @@ struct server_local_s
 
   /* State variable private to is_tor_running.  */
   int tor_state;
-
-  /* If the first both flags are set the assuan logging of data lines
-   * is suppressed.  The count variable is used to show the number of
-   * non-logged bytes.  */
-  size_t inhibit_data_logging_count;
-  unsigned int inhibit_data_logging : 1;
-  unsigned int inhibit_data_logging_now : 1;
 };
 
 
@@ -182,14 +174,9 @@ leave_cmd (assuan_context_t ctx, gpg_error_t err)
 static gpg_error_t
 data_line_write (assuan_context_t ctx, const void *buffer_arg, size_t size)
 {
-  ctrl_t ctrl = assuan_get_pointer (ctx);
   const char *buffer = buffer_arg;
   gpg_error_t err;
 
-  /* If we do not want logging, enable it it here.  */
-  if (ctrl && ctrl->server_local && ctrl->server_local->inhibit_data_logging)
-    ctrl->server_local->inhibit_data_logging_now = 1;
-
   if (opt.verbose && buffer && size)
     {
       /* Ease reading of output by sending a physical line at each LF.  */
@@ -205,14 +192,14 @@ data_line_write (assuan_context_t ctx, const void *buffer_arg, size_t size)
           if (err)
             {
               gpg_err_set_errno (EIO);
-              goto leave;
+              return err;
             }
           buffer += n;
           nbytes -= n;
           if (nbytes && (err=assuan_send_data (ctx, NULL, 0))) /* Flush line. */
             {
               gpg_err_set_errno (EIO);
-              goto leave;
+              return err;
             }
         }
       while (nbytes);
@@ -223,18 +210,11 @@ data_line_write (assuan_context_t ctx, const void *buffer_arg, size_t size)
       if (err)
         {
           gpg_err_set_errno (EIO);  /* For use by data_line_cookie_write.  */
-          goto leave;
+          return err;
         }
     }
 
- leave:
-  if (ctrl && ctrl->server_local && ctrl->server_local->inhibit_data_logging)
-    {
-      ctrl->server_local->inhibit_data_logging_now = 0;
-      ctrl->server_local->inhibit_data_logging_count += size;
-    }
-
-  return err;
+  return 0;
 }
 
 
@@ -256,16 +236,6 @@ data_line_cookie_close (void *cookie)
 {
   assuan_context_t ctx = cookie;
 
-  if (DBG_IPC)
-    {
-      ctrl_t ctrl = assuan_get_pointer (ctx);
-
-      if (ctrl && ctrl->server_local
-          && ctrl->server_local->inhibit_data_logging
-          && ctrl->server_local->inhibit_data_logging_count)
-        log_debug ("(%zu bytes sent via D lines not shown)\n",
-                   ctrl->server_local->inhibit_data_logging_count);
-    }
   if (assuan_send_data (ctx, NULL, 0))
     {
       gpg_err_set_errno (EIO);
@@ -303,12 +273,86 @@ strcpy_escaped_plus (char *d, const unsigned char *s)
 }
 
 
-/* This function returns true if a Tor server is running.  The sattus
-   is cached for the current connection.  */
+/* Check whether the option NAME appears in LINE */
+static int
+has_option (const char *line, const char *name)
+{
+  const char *s;
+  int n = strlen (name);
+
+  s = strstr (line, name);
+  return (s && (s == line || spacep (s-1)) && (!s[n] || spacep (s+n)));
+}
+
+/* Same as has_option but only considers options at the begin of the
+   line.  This is useful for commands which allow arbitrary strings on
+   the line.  */
+static int
+has_leading_option (const char *line, const char *name)
+{
+  const char *s;
+  int n;
+
+  if (name[0] != '-' || name[1] != '-' || !name[2] || spacep (name+2))
+    return 0;
+  n = strlen (name);
+  while ( *line == '-' && line[1] == '-' )
+    {
+      s = line;
+      while (*line && !spacep (line))
+        line++;
+      if (n == (line - s) && !strncmp (s, name, n))
+        return 1;
+      while (spacep (line))
+        line++;
+    }
+  return 0;
+}
+
+
+/* Same as has_option but does only test for the name of the option
+   and ignores an argument, i.e. with NAME being "--hash" it would
+   return a pointer for "--hash" as well as for "--hash=foo".  If
+   thhere is no such option NULL is returned.  The pointer returned
+   points right behind the option name, this may be an equal sign, Nul
+   or a space.  */
+/* static const char * */
+/* has_option_name (const char *line, const char *name) */
+/* { */
+/*   const char *s; */
+/*   int n = strlen (name); */
+
+/*   s = strstr (line, name); */
+/*   return (s && (s == line || spacep (s-1)) */
+/*           && (!s[n] || spacep (s+n) || s[n] == '=')) ? (s+n) : NULL; */
+/* } */
+
+
+/* Skip over options.  It is assumed that leading spaces have been
+   removed (this is the case for lines passed to a handler from
+   assuan).  Blanks after the options are also removed. */
+static char *
+skip_options (char *line)
+{
+  while ( *line == '-' && line[1] == '-' )
+    {
+      while (*line && !spacep (line))
+        line++;
+      while (spacep (line))
+        line++;
+    }
+  return line;
+}
+
+
+/* This fucntion returns true if a Tor server is running.  The sattus
+   is cached for the current conenction.  */
 static int
 is_tor_running (ctrl_t ctrl)
 {
-  /* Check whether we can connect to the proxy.  */
+#if ASSUAN_VERSION_NUMBER >= 0x020402
+  /* Check whether we can connect to the proxy.  We use a
+     special feature introduced with libassuan 2.4.2.  */
 
   if (!ctrl || !ctrl->server_local)
     return 0; /* Ooops.  */
@@ -327,6 +371,9 @@ is_tor_running (ctrl_t ctrl)
         }
     }
   return (ctrl->server_local->tor_state > 0);
+#else /* Libassuan < 2.4.2 */
+  return 0;  /* We don't know.  */
+#endif
 }
 
 
@@ -645,7 +692,7 @@ static const char hlp_dns_cert[] =
   "  *     Return the first record of any supported subtype\n"
   "  PGP   Return the first record of subtype PGP (3)\n"
   "  IPGP  Return the first record of subtype IPGP (6)\n"
-  "If the content of a certificate is available (PGP) it is returned\n"
+  "If the content of a certifciate is available (PGP) it is returned\n"
   "by data lines.  Fingerprints and URLs are returned via status lines.\n"
   "In --pka mode the fingerprint and if available an URL is returned.\n"
   "In --dane mode the key is returned from RR type 61";
@@ -821,103 +868,6 @@ cmd_dns_cert (assuan_context_t ctx, char *line)
 }
 
 
-
-static const char hlp_wkd_get[] =
-  "WKD_GET [--submission-address|--policy-flags] <user_id>\n"
-  "\n"
-  "Return the key or other info for <user_id>\n"
-  "from the Web Key Directory.";
-static gpg_error_t
-cmd_wkd_get (assuan_context_t ctx, char *line)
-{
-  ctrl_t ctrl = assuan_get_pointer (ctx);
-  gpg_error_t err = 0;
-  char *mbox = NULL;
-  char *domain;     /* Points to mbox.  */
-  char sha1buf[20];
-  char *uri = NULL;
-  char *encodedhash = NULL;
-  int opt_submission_addr;
-  int opt_policy_flags;
-  int no_log = 0;
-
-  opt_submission_addr = has_option (line, "--submission-address");
-  opt_policy_flags = has_option (line, "--policy-flags");
-  line = skip_options (line);
-
-  mbox = mailbox_from_userid (line);
-  if (!mbox || !(domain = strchr (mbox, '@')))
-    {
-      err = set_error (GPG_ERR_INV_USER_ID, "no mailbox in user id");
-      goto leave;
-    }
-  *domain++ = 0;
-
-  gcry_md_hash_buffer (GCRY_MD_SHA1, sha1buf, mbox, strlen (mbox));
-  encodedhash = zb32_encode (sha1buf, 8*20);
-  if (!encodedhash)
-    {
-      err = gpg_error_from_syserror ();
-      goto leave;
-    }
-
-  if (opt_submission_addr)
-    {
-      uri = strconcat ("https://",
-                       domain,
-                       "/.well-known/openpgpkey/submission-address",
-                       NULL);
-    }
-  else if (opt_policy_flags)
-    {
-      uri = strconcat ("https://",
-                       domain,
-                       "/.well-known/openpgpkey/policy",
-                       NULL);
-    }
-  else
-    {
-      uri = strconcat ("https://",
-                       domain,
-                       "/.well-known/openpgpkey/hu/",
-                       encodedhash,
-                       NULL);
-      no_log = 1;
-    }
-  if (!uri)
-    {
-      err = gpg_error_from_syserror ();
-      goto leave;
-    }
-
-  /* Setup an output stream and perform the get.  */
-  {
-    estream_t outfp;
-
-    outfp = es_fopencookie (ctx, "w", data_line_cookie_functions);
-    if (!outfp)
-      err = set_error (GPG_ERR_ASS_GENERAL,
-                       "error setting up a data stream");
-    else
-      {
-        if (no_log)
-          ctrl->server_local->inhibit_data_logging = 1;
-        ctrl->server_local->inhibit_data_logging_now = 0;
-        ctrl->server_local->inhibit_data_logging_count = 0;
-        err = ks_action_fetch (ctrl, uri, outfp);
-        es_fclose (outfp);
-        ctrl->server_local->inhibit_data_logging = 0;
-      }
-  }
-
- leave:
-  xfree (uri);
-  xfree (encodedhash);
-  xfree (mbox);
-  return leave_cmd (ctx, err);
-}
-
-
 
 static const char hlp_ldapserver[] =
   "LDAPSERVER <data>\n"
@@ -1197,7 +1147,7 @@ static const char hlp_checkocsp[] =
   "Processing then takes place without further interaction; in\n"
   "particular dirmngr tries to locate other required certificates by\n"
   "its own mechanism which includes a local certificate store as well\n"
-  "as a list of trusted root certificates.\n"
+  "as a list of trusted root certifciates.\n"
   "\n"
   "If the option --force-default-responder is given, only the default\n"
   "OCSP responder will be used and any other methods of obtaining an\n"
@@ -1366,7 +1316,7 @@ lookup_cert_by_pattern (assuan_context_t ctx, char *line,
         }
     }
 
-  /* First look through the internal cache.  The certifcates returned
+  /* First look through the internal cache.  The certifcates retruned
      here are not counted towards the truncation limit.  */
   if (single && !cache_only)
     ; /* Do not read from the local cache in this case.  */
@@ -1810,11 +1760,7 @@ ensure_keyserver (ctrl_t ctrl)
   if (ctrl->server_local->keyservers)
     return 0; /* Already set for this session.  */
   if (!opt.keyserver)
-    {
-      /* No global option set.  Fall back to default:  */
-      return make_keyserver_item (DIRMNGR_DEFAULT_KEYSERVER,
-                                  &ctrl->server_local->keyservers);
-    }
+    return 0; /* No global option set.  */
 
   for (sl = opt.keyserver; sl; sl = sl->next)
     {
@@ -2119,12 +2065,8 @@ cmd_ks_get (assuan_context_t ctx, char *line)
     err = set_error (GPG_ERR_ASS_GENERAL, "error setting up a data stream");
   else
     {
-      ctrl->server_local->inhibit_data_logging = 1;
-      ctrl->server_local->inhibit_data_logging_now = 0;
-      ctrl->server_local->inhibit_data_logging_count = 0;
       err = ks_action_get (ctrl, ctrl->server_local->keyservers, list, outfp);
       es_fclose (outfp);
-      ctrl->server_local->inhibit_data_logging = 0;
     }
 
  leave:
@@ -2147,7 +2089,7 @@ cmd_ks_fetch (assuan_context_t ctx, char *line)
   /* No options for now.  */
   line = skip_options (line);
 
-  err = ensure_keyserver (ctrl);  /* FIXME: Why do we needs this here?  */
+  err = ensure_keyserver (ctrl);
   if (err)
     goto leave;
 
@@ -2157,12 +2099,8 @@ cmd_ks_fetch (assuan_context_t ctx, char *line)
     err = set_error (GPG_ERR_ASS_GENERAL, "error setting up a data stream");
   else
     {
-      ctrl->server_local->inhibit_data_logging = 1;
-      ctrl->server_local->inhibit_data_logging_now = 0;
-      ctrl->server_local->inhibit_data_logging_count = 0;
       err = ks_action_fetch (ctrl, line, outfp);
       es_fclose (outfp);
-      ctrl->server_local->inhibit_data_logging = 0;
     }
 
  leave:
@@ -2241,22 +2179,6 @@ cmd_ks_put (assuan_context_t ctx, char *line)
 }
 
 
-
-static const char hlp_loadswdb[] =
-  "LOADSWDB [--force]\n"
-  "\n"
-  "Load and verify the swdb.lst from the Net.";
-static gpg_error_t
-cmd_loadswdb (assuan_context_t ctx, char *line)
-{
-  ctrl_t ctrl = assuan_get_pointer (ctx);
-  gpg_error_t err;
-
-  err = dirmngr_load_swdb (ctrl, has_option (line, "--force"));
-
-  return leave_cmd (ctx, err);
-}
-
 
 
 static const char hlp_getinfo[] =
@@ -2290,8 +2212,15 @@ cmd_getinfo (assuan_context_t ctx, char *line)
     }
   else if (!strcmp (line, "socket_name"))
     {
-      const char *s = dirmngr_get_current_socket_name ();
-      err = assuan_send_data (ctx, s, strlen (s));
+      const char *s = dirmngr_user_socket_name ();
+
+      if (!s)
+        s = dirmngr_sys_socket_name ();
+
+      if (s)
+        err = assuan_send_data (ctx, s, strlen (s));
+      else
+        err = gpg_error (GPG_ERR_NO_DATA);
     }
   else if (!strcmp (line, "tor"))
     {
@@ -2309,19 +2238,13 @@ cmd_getinfo (assuan_context_t ctx, char *line)
     }
   else if (!strcmp (line, "dnsinfo"))
     {
-      if (standard_resolver_p ())
-        assuan_set_okay_line
-          (ctx, "- Forced use of System resolver (w/o Tor support)");
-      else
-        {
-#ifdef USE_LIBDNS
-          assuan_set_okay_line (ctx, (recursive_resolver_p ()
-                                      ? "- Libdns recursive resolver"
-                                      : "- Libdns stub resolver"));
+#if USE_ADNS && HAVE_ADNS_IF_TORMODE
+      assuan_set_okay_line (ctx, "- ADNS with Tor support");
+#elif USE_ADNS
+      assuan_set_okay_line (ctx, "- ADNS w/o Tor support");
 #else
-          assuan_set_okay_line (ctx, "- System resolver (w/o Tor support)");
+      assuan_set_okay_line (ctx, "- System resolver w/o Tor support");
 #endif
-        }
       err = 0;
     }
   else
@@ -2341,12 +2264,29 @@ static gpg_error_t
 cmd_killdirmngr (assuan_context_t ctx, char *line)
 {
   ctrl_t ctrl = assuan_get_pointer (ctx);
+  gpg_error_t err;
 
   (void)line;
 
-  ctrl->server_local->stopme = 1;
-  assuan_set_flag (ctx, ASSUAN_FORCE_CLOSE, 1);
-  return gpg_error (GPG_ERR_EOF);
+  if (opt.system_daemon)
+    {
+      if (opt.system_service)
+        err = set_error (GPG_ERR_NOT_SUPPORTED,
+                         "can't do that whilst running as system service");
+      else
+        err = check_owner_permission (ctx,
+                                      "no permission to kill this process");
+    }
+  else
+    err = 0;
+
+  if (!err)
+    {
+      ctrl->server_local->stopme = 1;
+      assuan_set_flag (ctx, ASSUAN_FORCE_CLOSE, 1);
+      err = gpg_error (GPG_ERR_EOF);
+    }
+  return err;
 }
 
 
@@ -2361,11 +2301,26 @@ cmd_reloaddirmngr (assuan_context_t ctx, char *line)
   (void)ctx;
   (void)line;
 
+ if (opt.system_daemon)
+    {
+#ifndef HAVE_W32_SYSTEM
+      {
+        gpg_error_t err;
+
+        err = check_owner_permission (ctx,
+                                      "no permission to reload this process");
+        if (err)
+          return err;
+      }
+#endif
+    }
+
   dirmngr_sighup_action ();
   return 0;
 }
 
 
+
 
 /* Tell the assuan library about our commands. */
 static int
@@ -2377,7 +2332,6 @@ register_commands (assuan_context_t ctx)
     const char * const help;
   } table[] = {
     { "DNS_CERT",   cmd_dns_cert,   hlp_dns_cert },
-    { "WKD_GET",    cmd_wkd_get,    hlp_wkd_get },
     { "LDAPSERVER", cmd_ldapserver, hlp_ldapserver },
     { "ISVALID",    cmd_isvalid,    hlp_isvalid },
     { "CHECKCRL",   cmd_checkcrl,   hlp_checkcrl },
@@ -2393,7 +2347,6 @@ register_commands (assuan_context_t ctx)
     { "KS_FETCH",   cmd_ks_fetch,   hlp_ks_fetch },
     { "KS_PUT",     cmd_ks_put,     hlp_ks_put },
     { "GETINFO",    cmd_getinfo,    hlp_getinfo },
-    { "LOADSWDB",   cmd_loadswdb,   hlp_loadswdb },
     { "KILLDIRMNGR",cmd_killdirmngr,hlp_killdirmngr },
     { "RELOADDIRMNGR",cmd_reloaddirmngr,hlp_reloaddirmngr },
     { NULL, NULL }
@@ -2426,30 +2379,6 @@ reset_notify (assuan_context_t ctx, char *line)
 }
 
 
-/* This function is called by our assuan log handler to test whether a
- * log message shall really be printed.  The function must return
- * false to inhibit the logging of MSG.  CAT gives the requested log
- * category.  MSG might be NULL. */
-int
-dirmngr_assuan_log_monitor (assuan_context_t ctx, unsigned int cat,
-                            const char *msg)
-{
-  ctrl_t ctrl = assuan_get_pointer (ctx);
-
-  (void)cat;
-  (void)msg;
-
-  if (!ctrl || !ctrl->server_local)
-    return 1; /* Can't decide - allow logging.  */
-
-  if (!ctrl->server_local->inhibit_data_logging)
-    return 1; /* Not requested - allow logging.  */
-
-  /* Disallow logging if *_now is true.  */
-  return !ctrl->server_local->inhibit_data_logging_now;
-}
-
-
 /* Startup the server and run the main command loop.  With FD = -1,
    use stdin/stdout. */
 void
@@ -2514,13 +2443,22 @@ start_command_handler (assuan_fd_t fd)
 
   if (!hello_line)
     {
-      hello_line = xtryasprintf
-        ("Home: %s\n"
-         "Config: %s\n"
-         "%s",
-         gnupg_homedir (),
-         opt.config_filename? opt.config_filename : "[none]",
-         hello);
+      size_t n;
+      const char *cfgname;
+
+      cfgname = opt.config_filename? opt.config_filename : "[none]";
+
+      n = (30 + strlen (opt.homedir) + strlen (cfgname)
+           + strlen (hello) + 1);
+      hello_line = xmalloc (n+1);
+      snprintf (hello_line, n,
+                "Home: %s\n"
+                "Config: %s\n"
+                "%s",
+                opt.homedir,
+                cfgname,
+                hello);
+      hello_line[n] = 0;
     }
 
   ctrl->server_local->assuan_ctx = ctx;
