@@ -26,7 +26,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, see <https://www.gnu.org/licenses/>.
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
 
 #include <config.h>
@@ -38,11 +38,11 @@
 #include <unistd.h>
 #include <assuan.h>
 
-#include "../common/util.h"
-#include "../common/logging.h"
+#include "util.h"
+#include "logging.h"
 #include "http.h"
 
-#include <ksba.h>
+
 #if HTTP_USE_NTBTLS
 # include <ntbtls.h>
 #elif HTTP_USE_GNUTLS
@@ -118,57 +118,6 @@ my_gnutls_log (int level, const char *text)
 }
 #endif
 
-#if HTTP_USE_NTBTLS
-static gpg_error_t
-my_http_tls_verify_cb (void *opaque,
-                       http_t http,
-                       http_session_t session,
-                       unsigned int http_flags,
-                       void *tls_context)
-{
-  gpg_error_t err;
-  int idx;
-  ksba_cert_t cert;
-  ksba_cert_t hostcert = NULL;
-
-  (void)opaque;
-  (void)http;
-  (void)session;
-  (void)http_flags;
-
-  /* Get the peer's certs fron ntbtls.  */
-  for (idx = 0;
-       (cert = ntbtls_x509_get_peer_cert (tls_context, idx)); idx++)
-    {
-      if (!idx)
-        {
-          log_info ("Received host certificate\n");
-          hostcert = cert;
-        }
-      else
-        {
-
-          log_info ("Received additional certificate\n");
-          ksba_cert_release (cert);
-        }
-    }
-  if (!idx)
-    {
-      err  = gpg_error (GPG_ERR_MISSING_CERT);
-      goto leave;
-    }
-
-  err = 0;
-
- leave:
-  ksba_cert_release (hostcert);
-  log_info ("my_http_tls_verify_cb returns: %s\n", gpg_strerror (err));
-  return err;
-}
-#endif /*HTTP_USE_NTBTLS*/
-
-
-
 /* Prepend FNAME with the srcdir environment variable's value and
    return an allocated filename. */
 static char *
@@ -193,19 +142,19 @@ main (int argc, char **argv)
 {
   int last_argc = -1;
   gpg_error_t err;
-  int rc;  parsed_uri_t uri;
+  int rc;
+  parsed_uri_t uri;
   uri_tuple_t r;
   http_t hd;
   int c;
   unsigned int my_http_flags = 0;
   int no_out = 0;
   int tls_dbg = 0;
-  int no_crl = 0;
   const char *cafile = NULL;
   http_session_t session = NULL;
 
   gpgrt_init ();
-  log_set_prefix (PGM, GPGRT_LOG_WITH_PREFIX | GPGRT_LOG_WITH_PID);
+  log_set_prefix (PGM, 1 | 4);
   if (argc)
     { argc--; argv++; }
   while (argc && last_argc != argc )
@@ -222,13 +171,12 @@ main (int argc, char **argv)
                  "Options:\n"
                  "  --verbose         print timings etc.\n"
                  "  --debug           flyswatter\n"
-                 "  --tls-debug N     use TLS debug level N\n"
+                 "  --gnutls-debug N  use GNUTLS debug level N\n"
                  "  --cacert FNAME    expect CA certificate in file FNAME\n"
                  "  --no-verify       do not verify the certificate\n"
                  "  --force-tls       use HTTP_FLAG_FORCE_TLS\n"
                  "  --force-tor       use HTTP_FLAG_FORCE_TOR\n"
-                 "  --no-out          do not print the content\n"
-                 "  --no-crl          do not consuilt a CRL\n",
+                 "  --no-out          do not print the content\n",
                  stdout);
           exit (0);
         }
@@ -243,7 +191,7 @@ main (int argc, char **argv)
           debug++;
           argc--; argv++;
         }
-      else if (!strcmp (*argv, "--tls-debug"))
+      else if (!strcmp (*argv, "--gnutls-debug"))
         {
           argc--; argv++;
           if (argc)
@@ -281,11 +229,6 @@ main (int argc, char **argv)
           no_out = 1;
           argc--; argv++;
         }
-      else if (!strcmp (*argv, "--no-crl"))
-        {
-          no_crl = 1;
-          argc--; argv++;
-        }
       else if (!strncmp (*argv, "--", 2))
         {
           fprintf (stderr, PGM ": unknown option '%s'\n", *argv);
@@ -305,13 +248,9 @@ main (int argc, char **argv)
   assuan_sock_init ();
 
 #if HTTP_USE_NTBTLS
-  log_info ("new session.\n");
-  err = http_session_new (&session, NULL,
-                          ((no_crl? HTTP_FLAG_NO_CRL : 0)
-                           | HTTP_FLAG_TRUST_DEF),
-                          my_http_tls_verify_cb, NULL);
-  if (err)
-    log_error ("http_session_new failed: %s\n", gpg_strerror (err));
+
+  (void)err;
+
   ntbtls_set_debug (tls_dbg, NULL, NULL);
 
 #elif HTTP_USE_GNUTLS
@@ -323,10 +262,7 @@ main (int argc, char **argv)
   http_register_tls_callback (verify_callback);
   http_register_tls_ca (cafile);
 
-  err = http_session_new (&session, NULL,
-                          ((no_crl? HTTP_FLAG_NO_CRL : 0)
-                           | HTTP_FLAG_TRUST_DEF),
-                          NULL, NULL);
+  err = http_session_new (&session, NULL, NULL);
   if (err)
     log_error ("http_session_new failed: %s\n", gpg_strerror (err));
 
@@ -347,10 +283,6 @@ main (int argc, char **argv)
   if (tls_dbg)
     gnutls_global_set_log_level (tls_dbg);
 
-#else
-  (void)err;
-  (void)tls_dbg;
-  (void)no_crl;
 #endif /*HTTP_USE_GNUTLS*/
 
   rc = http_parse_uri (&uri, *argv, 1);
