@@ -15,13 +15,14 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, see <https://www.gnu.org/licenses/>.
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
 
 #include <config.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <assert.h>
 #include <ctype.h>
 
 #include "gpg.h"
@@ -53,29 +54,6 @@ static int write_sign_packet_header( IOBUF out, int ctb, u32 len );
 static int write_header2( IOBUF out, int ctb, u32 len, int hdrlen );
 static int write_new_header( IOBUF out, int ctb, u32 len, int hdrlen );
 
-/* Returns 1 if CTB is a new format ctb and 0 if CTB is an old format
-   ctb.  */
-static int
-ctb_new_format_p (int ctb)
-{
-  /* Bit 7 must always be set.  */
-  log_assert ((ctb & (1 << 7)));
-  /* Bit 6 indicates whether the packet is a new format packet.  */
-  return (ctb & (1 << 6));
-}
-
-/* Extract the packet type from a CTB.  */
-static int
-ctb_pkttype (int ctb)
-{
-  if (ctb_new_format_p (ctb))
-    /* Bits 0 through 5 are the packet type.  */
-    return (ctb & ((1 << 6) - 1));
-  else
-    /* Bits 2 through 5 are the packet type.  */
-    return (ctb & ((1 << 6) - 1)) >> 2;
-}
-
 /****************
  * Build a packet and write it to INP
  * Returns: 0 := okay
@@ -90,7 +68,7 @@ build_packet( IOBUF out, PACKET *pkt )
 
     if( DBG_PACKET )
 	log_debug("build_packet() type=%d\n", pkt->pkttype );
-    log_assert( pkt->pkt.generic );
+    assert( pkt->pkt.generic );
 
     switch ((pkttype = pkt->pkttype))
       {
@@ -265,7 +243,7 @@ calc_packet_length( PACKET *pkt )
     u32 n=0;
     int new_ctb = 0;
 
-    log_assert (pkt->pkt.generic);
+    assert( pkt->pkt.generic );
     switch( pkt->pkttype ) {
       case PKT_PLAINTEXT:
 	n = calc_plaintext( pkt->pkt.plaintext );
@@ -301,8 +279,6 @@ write_fake_data (IOBUF out, gcry_mpi_t a)
 
   if (!a)
     return 0;
-  if (!gcry_mpi_get_flag (a, GCRYMPI_FLAG_OPAQUE))
-    return 0; /* e.g. due to generating a key with wrong usage.  */
   p = gcry_mpi_get_opaque ( a, &n);
   if (!p)
     return 0; /* For example due to a read error in
@@ -311,19 +287,11 @@ write_fake_data (IOBUF out, gcry_mpi_t a)
 }
 
 
-/* Serialize the user id (RFC 4880, Section 5.11) or the user
-   attribute UID (Section 5.12) and write it to OUT.
-
-   CTB is the serialization's CTB.  It specifies the header format and
-   the packet's type.  The header length must not be set.  */
 static int
 do_user_id( IOBUF out, int ctb, PKT_user_id *uid )
 {
   int rc;
 
-  log_assert (ctb_pkttype (ctb) == PKT_USER_ID
-              || ctb_pkttype (ctb) == PKT_ATTRIBUTE);
-
   if (uid->attrib_data)
     {
       write_header(out, ctb, uid->attrib_len);
@@ -331,44 +299,19 @@ do_user_id( IOBUF out, int ctb, PKT_user_id *uid )
     }
   else
     {
-      write_header2( out, ctb, uid->len, 0 );
+      write_header2( out, ctb, uid->len, 2 );
       rc = iobuf_write( out, uid->name, uid->len );
     }
   return rc;
 }
 
 
-/* Serialize the key (RFC 4880, Section 5.5) described by PK and write
-   it to OUT.
-
-   This function serializes both primary keys and subkeys with or
-   without a secret part.
-
-   CTB is the serialization's CTB.  It specifies the header format and
-   the packet's type.  The header length must not be set.
-
-   PK->VERSION specifies the serialization format.  A value of 0 means
-   to use the default version.  Currently, only version 4 packets are
-   supported.
- */
 static int
 do_key (iobuf_t out, int ctb, PKT_public_key *pk)
 {
   gpg_error_t err = 0;
-  /* The length of the body is stored in the packet's header, which
-     occurs before the body.  Unfortunately, we don't know the length
-     of the packet's body until we've written all of the data!  To
-     work around this, we first write the data into this temporary
-     buffer, then generate the header, and finally copy the contents
-     of this buffer to OUT.  */
-  iobuf_t a = iobuf_temp();
   int i, nskey, npkey;
-
-  log_assert (pk->version == 0 || pk->version == 4);
-  log_assert (ctb_pkttype (ctb) == PKT_PUBLIC_KEY
-              || ctb_pkttype (ctb) == PKT_PUBLIC_SUBKEY
-              || ctb_pkttype (ctb) == PKT_SECRET_KEY
-              || ctb_pkttype (ctb) == PKT_SECRET_SUBKEY);
+  iobuf_t a = iobuf_temp(); /* Build in a self-enlarging buffer.  */
 
   /* Write the version number - if none is specified, use 4 */
   if ( !pk->version )
@@ -380,7 +323,7 @@ do_key (iobuf_t out, int ctb, PKT_public_key *pk)
   iobuf_put (a, pk->pubkey_algo );
 
   /* Get number of secret and public parameters.  They are held in one
-     array: the public ones followed by the secret ones.  */
+     array first the public ones, then the secret ones.  */
   nskey = pubkey_get_nskey (pk->pubkey_algo);
   npkey = pubkey_get_npkey (pk->pubkey_algo);
 
@@ -392,7 +335,7 @@ do_key (iobuf_t out, int ctb, PKT_public_key *pk)
       write_fake_data (a, pk->pkey[0]);
       goto leave;
     }
-  log_assert (npkey < nskey);
+  assert (npkey < nskey);
 
   for (i=0; i < npkey; i++ )
     {
@@ -466,7 +409,7 @@ do_key (iobuf_t out, int ctb, PKT_public_key *pk)
           byte *p;
           unsigned int ndatabits;
 
-          log_assert (gcry_mpi_get_flag (pk->pkey[npkey], GCRYMPI_FLAG_OPAQUE));
+          assert (gcry_mpi_get_flag (pk->pkey[npkey], GCRYMPI_FLAG_OPAQUE));
           p = gcry_mpi_get_opaque (pk->pkey[npkey], &ndatabits);
           if (p)
             iobuf_write (a, p, (ndatabits+7)/8 );
@@ -487,8 +430,8 @@ do_key (iobuf_t out, int ctb, PKT_public_key *pk)
       /* Build the header of the packet - which we must do after
          writing all the other stuff, so that we know the length of
          the packet */
-      write_header2 (out, ctb, iobuf_get_temp_length(a), 0);
-       /* And finally write it out to the real stream. */
+      write_header2 (out, ctb, iobuf_get_temp_length(a), pk->hdrbytes);
+      /* And finally write it out to the real stream. */
       err = iobuf_write_temp (out, a);
     }
 
@@ -496,36 +439,16 @@ do_key (iobuf_t out, int ctb, PKT_public_key *pk)
   return err;
 }
 
-/* Serialize the symmetric-key encrypted session key packet (RFC 4880,
-   5.3) described by ENC and write it to OUT.
-
-   CTB is the serialization's CTB.  It specifies the header format and
-   the packet's type.  The header length must not be set.  */
 static int
 do_symkey_enc( IOBUF out, int ctb, PKT_symkey_enc *enc )
 {
     int rc = 0;
     IOBUF a = iobuf_temp();
 
-    log_assert (ctb_pkttype (ctb) == PKT_SYMKEY_ENC);
-
-    /* The only acceptable version.  */
-    log_assert( enc->version == 4 );
-
-    /* RFC 4880, Section 3.7.  */
-    switch( enc->s2k.mode )
-      {
-      /* Simple S2K.  */
-      case 0:
-      /* Salted S2K.  */
-      case 1:
-      /* Iterated and salted S2K.  */
-      case 3:
-        /* Reasonable values.  */
-        break;
-
-      default:
-        log_bug("do_symkey_enc: s2k=%d\n", enc->s2k.mode );
+    assert( enc->version == 4 );
+    switch( enc->s2k.mode ) {
+      case 0: case 1: case 3: break;
+      default: log_bug("do_symkey_enc: s2k=%d\n", enc->s2k.mode );
     }
     iobuf_put( a, enc->version );
     iobuf_put( a, enc->cipher_algo );
@@ -547,11 +470,6 @@ do_symkey_enc( IOBUF out, int ctb, PKT_symkey_enc *enc )
 }
 
 
-/* Serialize the public-key encrypted session key packet (RFC 4880,
-   5.1) described by ENC and write it to OUT.
-
-   CTB is the serialization's CTB.  It specifies the header format and
-   the packet's type.  The header length must not be set.  */
 static int
 do_pubkey_enc( IOBUF out, int ctb, PKT_pubkey_enc *enc )
 {
@@ -559,8 +477,6 @@ do_pubkey_enc( IOBUF out, int ctb, PKT_pubkey_enc *enc )
   int n, i;
   IOBUF a = iobuf_temp();
 
-  log_assert (ctb_pkttype (ctb) == PKT_PUBKEY_ENC);
-
   iobuf_put (a, 3); /* Version.  */
 
   if ( enc->throw_keyid )
@@ -596,8 +512,6 @@ do_pubkey_enc( IOBUF out, int ctb, PKT_pubkey_enc *enc )
 }
 
 
-/* Calculate the length of the serialized plaintext packet PT (RFC
-   4480, Section 5.9).  */
 static u32
 calc_plaintext( PKT_plaintext *pt )
 {
@@ -611,71 +525,48 @@ calc_plaintext( PKT_plaintext *pt )
   return pt->len? (1 + 1 + pt->namelen + 4 + pt->len) : 0;
 }
 
-/* Serialize the plaintext packet (RFC 4880, 5.9) described by PT and
-   write it to OUT.
-
-   The body of the message is stored in PT->BUF.  The amount of data
-   to write is PT->LEN.  (PT->BUF should be configured to return EOF
-   after this much data has been read.)  If PT->LEN is 0 and CTB
-   indicates that this is a new format packet, then partial block mode
-   is assumed to have been enabled on OUT.  On success, partial block
-   mode is disabled.
-
-   If PT->BUF is NULL, the the caller must write out the data.  In
-   this case, if PT->LEN was 0, then partial body length mode was
-   enabled and the caller must disable it by calling
-   iobuf_set_partial_body_length_mode (out, 0).  */
 static int
 do_plaintext( IOBUF out, int ctb, PKT_plaintext *pt )
 {
-    int rc = 0;
-    size_t nbytes;
-
-    log_assert (ctb_pkttype (ctb) == PKT_PLAINTEXT);
+    int i, rc = 0;
+    u32 n;
+    byte buf[1000]; /* this buffer has the plaintext! */
+    int nbytes;
 
     write_header(out, ctb, calc_plaintext( pt ) );
-    log_assert (pt->mode == 'b' || pt->mode == 't' || pt->mode == 'u'
-                || pt->mode == 'm'
-                || pt->mode == 'l' || pt->mode == '1');
     iobuf_put(out, pt->mode );
     iobuf_put(out, pt->namelen );
-    iobuf_write (out, pt->name, pt->namelen);
+    for(i=0; i < pt->namelen; i++ )
+	iobuf_put(out, pt->name[i] );
     rc = write_32(out, pt->timestamp );
     if (rc)
       return rc;
 
-    if (pt->buf)
-      {
-        nbytes = iobuf_copy (out, pt->buf);
-        if(ctb_new_format_p (ctb) && !pt->len)
-          /* Turn off partial body length mode.  */
-          iobuf_set_partial_body_length_mode (out, 0);
-        if( pt->len && nbytes != pt->len )
-          log_error("do_plaintext(): wrote %lu bytes but expected %lu bytes\n",
-                    (ulong)nbytes, (ulong)pt->len );
-      }
+    n = 0;
+    while( (nbytes=iobuf_read(pt->buf, buf, 1000)) != -1 ) {
+      rc = iobuf_write (out, buf, nbytes);
+      if (rc)
+        break;
+      n += nbytes;
+    }
+    wipememory(buf,1000); /* burn the buffer */
+    if( (ctb&0x40) && !pt->len )
+      iobuf_set_partial_block_mode(out, 0 ); /* turn off partial */
+    if( pt->len && n != pt->len )
+      log_error("do_plaintext(): wrote %lu bytes but expected %lu bytes\n",
+		(ulong)n, (ulong)pt->len );
 
     return rc;
 }
 
 
 
-/* Serialize the symmetrically encrypted data packet (RFC 4880,
-   Section 5.7) described by ED and write it to OUT.
-
-   Note: this only writes the packets header!  The call must then
-   follow up and write the initial random data and the body to OUT.
-   (If you use the encryption iobuf filter (cipher_filter), then this
-   is done automatically.)  */
 static int
 do_encrypted( IOBUF out, int ctb, PKT_encrypted *ed )
 {
     int rc = 0;
     u32 n;
 
-    log_assert (! ed->mdc_method);
-    log_assert (ctb_pkttype (ctb) == PKT_ENCRYPTED);
-
     n = ed->len ? (ed->len + ed->extralen) : 0;
     write_header(out, ctb, n );
 
@@ -684,22 +575,13 @@ do_encrypted( IOBUF out, int ctb, PKT_encrypted *ed )
     return rc;
 }
 
-/* Serialize the symmetrically encrypted integrity protected data
-   packet (RFC 4880, Section 5.13) described by ED and write it to
-   OUT.
-
-   Note: this only writes the packet's header!  The caller must then
-   follow up and write the initial random data, the body and the MDC
-   packet to OUT.  (If you use the encryption iobuf filter
-   (cipher_filter), then this is done automatically.)  */
 static int
 do_encrypted_mdc( IOBUF out, int ctb, PKT_encrypted *ed )
 {
     int rc = 0;
     u32 n;
 
-    log_assert (ed->mdc_method);
-    log_assert (ctb_pkttype (ctb) == PKT_ENCRYPTED_MDC);
+    assert( ed->mdc_method );
 
     /* Take version number and the following MDC packet in account. */
     n = ed->len ? (ed->len + ed->extralen + 1 + 22) : 0;
@@ -712,18 +594,11 @@ do_encrypted_mdc( IOBUF out, int ctb, PKT_encrypted *ed )
 }
 
 
-/* Serialize the compressed packet (RFC 4880, Section 5.6) described
-   by CD and write it to OUT.
-
-   Note: this only writes the packet's header!  The caller must then
-   follow up and write the body to OUT.  */
 static int
 do_compressed( IOBUF out, int ctb, PKT_compressed *cd )
 {
     int rc = 0;
 
-    log_assert (ctb_pkttype (ctb) == PKT_COMPRESSED);
-
     /* We must use the old convention and don't use blockmode for the
        sake of PGP 2 compatibility.  However if the new_ctb flag was
        set, CTB is already formatted as new style and write_header2
@@ -800,7 +675,7 @@ delete_sig_subpkt (subpktarea_t *area, sigsubpkttype_t reqtype )
 
     if (!okay)
         log_error ("delete_subpkt: buffer shorter than subpacket\n");
-    log_assert (unused <= area->len);
+    assert (unused <= area->len);
     area->len -= unused;
     return !!unused;
 }
@@ -973,44 +848,28 @@ build_sig_subpkt (PKT_signature *sig, sigsubpkttype_t type,
 	sig->unhashed = newarea;
 }
 
-/*
+/****************
  * Put all the required stuff from SIG into subpackets of sig.
- * PKSK is the signing key.
  * Hmmm, should we delete those subpackets which are in a wrong area?
  */
 void
-build_sig_subpkt_from_sig (PKT_signature *sig, PKT_public_key *pksk)
+build_sig_subpkt_from_sig( PKT_signature *sig )
 {
     u32  u;
-    byte buf[1+MAX_FINGERPRINT_LEN];
-    size_t fprlen;
+    byte buf[8];
 
-    /* For v4 keys we need to write the ISSUER subpacket.  We do not
-     * want that for a future v5 format.  */
-    if (pksk->version < 5)
-      {
-        u = sig->keyid[0];
-        buf[0] = (u >> 24) & 0xff;
-        buf[1] = (u >> 16) & 0xff;
-        buf[2] = (u >>  8) & 0xff;
-        buf[3] = u & 0xff;
-        u = sig->keyid[1];
-        buf[4] = (u >> 24) & 0xff;
-        buf[5] = (u >> 16) & 0xff;
-        buf[6] = (u >>  8) & 0xff;
-        buf[7] = u & 0xff;
-        build_sig_subpkt (sig, SIGSUBPKT_ISSUER, buf, 8);
-      }
+    u = sig->keyid[0];
+    buf[0] = (u >> 24) & 0xff;
+    buf[1] = (u >> 16) & 0xff;
+    buf[2] = (u >>  8) & 0xff;
+    buf[3] = u & 0xff;
+    u = sig->keyid[1];
+    buf[4] = (u >> 24) & 0xff;
+    buf[5] = (u >> 16) & 0xff;
+    buf[6] = (u >>  8) & 0xff;
+    buf[7] = u & 0xff;
+    build_sig_subpkt( sig, SIGSUBPKT_ISSUER, buf, 8 );
 
-    /* Write the new ISSUER_FPR subpacket.  */
-    fingerprint_from_pk (pksk, buf+1, &fprlen);
-    if (fprlen == 20)
-      {
-        buf[0] = pksk->version;
-        build_sig_subpkt (sig, SIGSUBPKT_ISSUER_FPR, buf, 21);
-      }
-
-    /* Write the timestamp.  */
     u = sig->timestamp;
     buf[0] = (u >> 24) & 0xff;
     buf[1] = (u >> 16) & 0xff;
@@ -1088,42 +947,6 @@ build_attribute_subpkt(PKT_user_id *uid,byte type,
   uid->attrib_len+=idx+headerlen+buflen;
 }
 
-/* Returns a human-readable string corresponding to the notation.
-   This ignores notation->value.  The caller must free the result.  */
-static char *
-notation_value_to_human_readable_string (struct notation *notation)
-{
-  if(notation->bdat)
-    /* Binary data.  */
-    {
-      size_t len = notation->blen;
-      int i;
-      char preview[20];
-
-      for (i = 0; i < len && i < sizeof (preview) - 1; i ++)
-        if (isprint (notation->bdat[i]))
-          preview[i] = notation->bdat[i];
-        else
-          preview[i] = '?';
-      preview[i] = 0;
-
-      return xasprintf (_("[ not human readable (%zu bytes: %s%s) ]"),
-                        len, preview, i < len ? "..." : "");
-    }
-  else
-    /* The value is human-readable.  */
-    return xstrdup (notation->value);
-}
-
-/* Turn the notation described by the string STRING into a notation.
-
-   STRING has the form:
-
-     - -name - Delete the notation.
-     - name@domain.name=value - Normal notation
-     - !name@domain.name=value - Notation with critical bit set.
-
-   The caller must free the result using free_notation().  */
 struct notation *
 string_to_notation(const char *string,int is_utf8)
 {
@@ -1214,101 +1037,14 @@ string_to_notation(const char *string,int is_utf8)
   return NULL;
 }
 
-/* Like string_to_notation, but store opaque data rather than human
-   readable data.  */
-struct notation *
-blob_to_notation(const char *name, const char *data, size_t len)
-{
-  const char *s;
-  int saw_at=0;
-  struct notation *notation;
-
-  notation=xmalloc_clear(sizeof(*notation));
-
-  if(*name=='-')
-    {
-      notation->flags.ignore=1;
-      name++;
-    }
-
-  if(*name=='!')
-    {
-      notation->flags.critical=1;
-      name++;
-    }
-
-  /* If and when the IETF assigns some official name tags, we'll have
-     to add them here. */
-
-  for( s=name ; *s; s++ )
-    {
-      if( *s=='@')
-	saw_at++;
-
-      /* -notationname is legal without an = sign */
-      if(!*s && notation->flags.ignore)
-	break;
-
-      if (*s == '=')
-        {
-          log_error(_("a notation name may not contain an '=' character\n"));
-          goto fail;
-        }
-
-      if (!isascii (*s) || (!isgraph(*s) && !isspace(*s)))
-	{
-	  log_error(_("a notation name must have only printable characters"
-		      " or spaces\n") );
-	  goto fail;
-	}
-    }
-
-  notation->name=xstrdup (name);
-
-  if(!saw_at && !opt.expert)
-    {
-      log_error(_("a user notation name must contain the '@' character\n"));
-      goto fail;
-    }
-
-  if (saw_at > 1)
-    {
-      log_error(_("a notation name must not contain more than"
-		  " one '@' character\n"));
-      goto fail;
-    }
-
-  notation->bdat = xmalloc (len);
-  memcpy (notation->bdat, data, len);
-  notation->blen = len;
-
-  notation->value = notation_value_to_human_readable_string (notation);
-
-  return notation;
-
- fail:
-  free_notation(notation);
-  return NULL;
-}
-
 struct notation *
 sig_to_notation(PKT_signature *sig)
 {
   const byte *p;
   size_t len;
-  int seq = 0;
-  int crit;
-  notation_t list = NULL;
+  int seq=0,crit;
+  struct notation *list=NULL;
 
-  /* See RFC 4880, 5.2.3.16 for the format of notation data.  In
-     short, a notation has:
-
-       - 4 bytes of flags
-       - 2 byte name length (n1)
-       - 2 byte value length (n2)
-       - n1 bytes of name data
-       - n2 bytes of value data
-   */
   while((p=enum_sig_subpkt(sig->hashed,SIGSUBPKT_NOTATION,&len,&seq,&crit)))
     {
       int n1,n2;
@@ -1320,9 +1056,7 @@ sig_to_notation(PKT_signature *sig)
 	  continue;
 	}
 
-      /* name length.  */
       n1=(p[4]<<8)|p[5];
-      /* value length.  */
       n2=(p[6]<<8)|p[7];
 
       if(8+n1+n2!=len)
@@ -1338,21 +1072,21 @@ sig_to_notation(PKT_signature *sig)
       n->name[n1]='\0';
 
       if(p[0]&0x80)
-        /* The value is human-readable.  */
 	{
 	  n->value=xmalloc(n2+1);
 	  memcpy(n->value,&p[8+n1],n2);
 	  n->value[n2]='\0';
-          n->flags.human = 1;
 	}
       else
-        /* Binary data.  */
 	{
 	  n->bdat=xmalloc(n2);
 	  n->blen=n2;
 	  memcpy(n->bdat,&p[8+n1],n2);
 
-          n->value = notation_value_to_human_readable_string (n);
+	  n->value=xmalloc(2+strlen(_("not human readable"))+2+1);
+	  strcpy(n->value,"[ ");
+	  strcat(n->value,_("not human readable"));
+	  strcat(n->value," ]");
 	}
 
       n->flags.critical=crit;
@@ -1364,9 +1098,6 @@ sig_to_notation(PKT_signature *sig)
   return list;
 }
 
-/* Release the resources associated with the *list* of notations.  To
-   release a single notation, make sure that notation->next is
-   NULL.  */
 void
 free_notation(struct notation *notation)
 {
@@ -1383,8 +1114,6 @@ free_notation(struct notation *notation)
     }
 }
 
-/* Serialize the signature packet (RFC 4880, Section 5.2) described by
-   SIG and write it to OUT.  */
 static int
 do_signature( IOBUF out, int ctb, PKT_signature *sig )
 {
@@ -1392,16 +1121,8 @@ do_signature( IOBUF out, int ctb, PKT_signature *sig )
   int n, i;
   IOBUF a = iobuf_temp();
 
-  log_assert (ctb_pkttype (ctb) == PKT_SIGNATURE);
-
-  if ( !sig->version || sig->version == 3)
-    {
-      iobuf_put( a, 3 );
-
-      /* Version 3 packets don't support subpackets.  */
-      log_assert (! sig->hashed);
-      log_assert (! sig->unhashed);
-    }
+  if ( !sig->version )
+    iobuf_put( a, 3 );
   else
     iobuf_put( a, sig->version );
   if ( sig->version < 4 )
@@ -1452,28 +1173,28 @@ do_signature( IOBUF out, int ctb, PKT_signature *sig )
 }
 
 
-/* Serialize the one-pass signature packet (RFC 4880, Section 5.4)
-   described by OPS and write it to OUT.  */
 static int
 do_onepass_sig( IOBUF out, int ctb, PKT_onepass_sig *ops )
 {
-    log_assert (ctb_pkttype (ctb) == PKT_ONEPASS_SIG);
+    int rc = 0;
+    IOBUF a = iobuf_temp();
 
-    write_header(out, ctb, 4 + 8 + 1);
+    iobuf_put (a, 3);  /* Version.  */
+    iobuf_put(a, ops->sig_class );
+    iobuf_put(a, ops->digest_algo );
+    iobuf_put(a, ops->pubkey_algo );
+    write_32(a, ops->keyid[0] );
+    write_32(a, ops->keyid[1] );
+    iobuf_put(a, ops->last );
 
-    iobuf_put (out, 3);  /* Version.  */
-    iobuf_put(out, ops->sig_class );
-    iobuf_put(out, ops->digest_algo );
-    iobuf_put(out, ops->pubkey_algo );
-    write_32(out, ops->keyid[0] );
-    write_32(out, ops->keyid[1] );
-    iobuf_put(out, ops->last );
+    write_header(out, ctb, iobuf_get_temp_length(a) );
+    rc = iobuf_write_temp( out, a );
 
-    return 0;
+    iobuf_close(a);
+    return rc;
 }
 
 
-/* Write a 16-bit quantity to OUT in big endian order.  */
 static int
 write_16(IOBUF out, u16 a)
 {
@@ -1483,7 +1204,6 @@ write_16(IOBUF out, u16 a)
     return 0;
 }
 
-/* Write a 32-bit quantity to OUT in big endian order.  */
 static int
 write_32(IOBUF out, u32 a)
 {
@@ -1495,18 +1215,14 @@ write_32(IOBUF out, u32 a)
 
 
 /****************
- * calculate the length of a header.
- *
- * LEN is the length of the packet's body.  NEW_CTB is whether we are
- * using a new or old format packet.
- *
- * This function does not handle indeterminate lengths or partial body
- * lengths.  (If you pass LEN as 0, then this function assumes you
- * really mean an empty body.)
+ * calculate the length of a header
  */
 static int
 calc_header_length( u32 len, int new_ctb )
 {
+    if( !len )
+	return 1; /* only the ctb */
+
     if( new_ctb ) {
 	if( len < 192 )
 	    return 2;
@@ -1547,73 +1263,36 @@ write_sign_packet_header (IOBUF out, int ctb, u32 len)
 }
 
 /****************
- * Write a packet header to OUT.
- *
- * CTB is the ctb.  It determines whether a new or old format packet
- * header should be written.  The length field is adjusted, but the
- * CTB is otherwise written out as is.
- *
- * LEN is the length of the packet's body.
- *
- * If HDRLEN is set, then we don't necessarily use the most efficient
- * encoding to store LEN, but the specified length.  (If this is not
- * possible, this is a bug.)  In this case, LEN=0 means a 0 length
- * packet.  Note: setting HDRLEN is only supported for old format
- * packets!
- *
- * If HDRLEN is not set, then the shortest encoding is used.  In this
- * case, LEN=0 means the body has an indeterminate length and a
- * partial body length header (if a new format packet) or an
- * indeterminate length header (if an old format packet) is written
- * out.  Further, if using partial body lengths, this enables partial
- * body length mode on OUT.
+ * If HDRLEN is > 0, try to build a header of this length.  We need
+ * this so that we can hash packets without reading them again.  If
+ * len is 0, write a partial or indeterminate length header, unless
+ * hdrlen is specified in which case write an actual zero length
+ * (using the specified hdrlen).
  */
 static int
 write_header2( IOBUF out, int ctb, u32 len, int hdrlen )
 {
-  if (ctb_new_format_p (ctb))
+  if( ctb & 0x40 )
     return write_new_header( out, ctb, len, hdrlen );
 
-  /* An old format packet.  Refer to RFC 4880, Section 4.2.1 to
-     understand how lengths are encoded in this case.  */
-
-  /* The length encoding is stored in the two least significant bits.
-     Make sure they are cleared.  */
-  log_assert ((ctb & 3) == 0);
-
-  log_assert (hdrlen == 0 || hdrlen == 2 || hdrlen == 3 || hdrlen == 5);
-
-  if (hdrlen)
-    /* Header length is given.  */
+  if( hdrlen )
     {
       if( hdrlen == 2 && len < 256 )
-        /* 00 => 1 byte length.  */
 	;
       else if( hdrlen == 3 && len < 65536 )
-        /* 01 => 2 byte length.  If len < 256, this is not the most
-           compact encoding, but it is a correct encoding.  */
 	ctb |= 1;
-      else if (hdrlen == 5)
-        /* 10 => 4 byte length.  If len < 65536, this is not the most
-           compact encoding, but it is a correct encoding.  */
-	ctb |= 2;
       else
-        log_bug ("Can't encode length=%d in a %d byte header!\n",
-                 len, hdrlen);
+	ctb |= 2;
     }
   else
     {
       if( !len )
-        /* 11 => Indeterminate length.  */
 	ctb |= 3;
       else if( len < 256 )
-        /* 00 => 1 byte length.  */
 	;
       else if( len < 65536 )
-        /* 01 => 2 byte length.  */
 	ctb |= 1;
       else
-        /* 10 => 4 byte length.  */
 	ctb |= 2;
     }
 
@@ -1642,20 +1321,6 @@ write_header2( IOBUF out, int ctb, u32 len, int hdrlen )
 }
 
 
-/* Write a new format header to OUT.
-
-   CTB is the ctb.
-
-   LEN is the length of the packet's body.  If LEN is 0, then enables
-   partial body length mode (i.e., the body is of an indeterminant
-   length) on OUT.  Note: this function cannot be used to generate a
-   header for a zero length packet.
-
-   HDRLEN is the length of the packet's header.  If HDRLEN is 0, the
-   shortest encoding is chosen based on the length of the packet's
-   body.  Currently, values other than 0 are not supported.
-
-   Returns 0 on success.  */
 static int
 write_new_header( IOBUF out, int ctb, u32 len, int hdrlen )
 {
@@ -1665,7 +1330,7 @@ write_new_header( IOBUF out, int ctb, u32 len, int hdrlen )
     if( iobuf_put(out, ctb ) )
 	return -1;
     if( !len ) {
-	iobuf_set_partial_body_length_mode(out, 512 );
+	iobuf_set_partial_block_mode(out, 512 );
     }
     else {
 	if( len < 192 ) {
