@@ -15,7 +15,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, see <https://www.gnu.org/licenses/>.
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
 
 #include <config.h>
@@ -25,24 +25,24 @@
 #include <errno.h>
 #include <unistd.h>
 #include <time.h>
+#include <assert.h>
 #ifdef HAVE_LOCALE_H
 # include <locale.h>
 #endif
 
 #include "gpg.h"
 #include <assuan.h>
-#include "../common/util.h"
-#include "../common/membuf.h"
+#include "util.h"
+#include "membuf.h"
 #include "options.h"
-#include "../common/i18n.h"
-#include "../common/asshelp.h"
-#include "../common/keyserver.h"
-#include "../common/status.h"
+#include "i18n.h"
+#include "asshelp.h"
+#include "keyserver.h"
+#include "status.h"
 #include "call-dirmngr.h"
 
 
-/* Parameter structure used to gather status info.  Note that it is
- * also used for WKD requests.  */
+/* Parameter structure used to gather status info.  */
 struct ks_status_parm_s
 {
   const char *keyword; /* Look for this keyword or NULL for "SOURCE". */
@@ -146,7 +146,7 @@ warn_version_mismatch (assuan_context_t ctx, const char *servername)
   if (err)
     log_error (_("error getting version from '%s': %s\n"),
                servername, gpg_strerror (err));
-  else if (compare_version_strings (serverversion, myversion) < 0)
+  else if (!compare_version_strings (serverversion, myversion))
     {
       char *warn;
 
@@ -178,6 +178,7 @@ create_context (ctrl_t ctrl, assuan_context_t *r_ctx)
   *r_ctx = NULL;
   err = start_new_dirmngr (&ctx,
                            GPG_ERR_SOURCE_DEFAULT,
+                           opt.homedir,
                            opt.dirmngr_program,
                            opt.autostart, opt.verbose, DBG_IPC,
                            NULL /*gpg_status2*/, ctrl);
@@ -260,7 +261,7 @@ open_context (ctrl_t ctrl, assuan_context_t *r_ctx)
       if (dml)
         {
           /* Found an inactive local session - return that.  */
-          log_assert (!dml->is_active);
+          assert (!dml->is_active);
 
           /* But first do the per session init if not yet done.  */
           if (!dml->set_keyservers_done)
@@ -369,18 +370,16 @@ clear_context_flags (ctrl_t ctrl, assuan_context_t ctx)
 
 
 
-/* Status callback for ks_list, ks_get, ks_search, and wkd_get  */
+/* Status callback for ks_list, ks_get and ks_search.  */
 static gpg_error_t
 ks_status_cb (void *opaque, const char *line)
 {
   struct ks_status_parm_s *parm = opaque;
   gpg_error_t err = 0;
-  const char *s, *s2;
-  const char *warn;
+  const char *s;
 
   if ((s = has_leading_keyword (line, parm->keyword? parm->keyword : "SOURCE")))
     {
-      /* Note that the arg for "S SOURCE" is the URL of a keyserver.  */
       if (!parm->source)
         {
           parm->source = xtrystrdup (s);
@@ -388,29 +387,6 @@ ks_status_cb (void *opaque, const char *line)
             err = gpg_error_from_syserror ();
         }
     }
-  else if ((s = has_leading_keyword (line, "WARNING")))
-    {
-      if ((s2 = has_leading_keyword (s, "tor_not_running")))
-        warn = _("Tor is not running");
-      else if ((s2 = has_leading_keyword (s, "tor_config_problem")))
-        warn = _("Tor is not properly configured");
-      else
-        warn = NULL;
-
-      if (warn)
-        {
-          log_info (_("WARNING: %s\n"), warn);
-          if (s2)
-            {
-              while (*s2 && !spacep (s2))
-                s2++;
-              while (*s2 && spacep (s2))
-                s2++;
-              if (*s2)
-                print_further_info ("%s", s2);
-            }
-        }
-    }
 
   return err;
 }
@@ -626,8 +602,6 @@ ks_get_data_cb (void *opaque, const void *data, size_t datalen)
    don't need to escape the patterns before sending them to the
    server.
 
-   If QUICK is set the dirmngr is advised to use a shorter timeout.
-
    If R_SOURCE is not NULL the source of the data is stored as a
    malloced string there.  If a source is not known NULL is stored.
 
@@ -637,7 +611,7 @@ ks_get_data_cb (void *opaque, const void *data, size_t datalen)
    are able to ask for (1000-10-1)/(2+8+1) = 90 keys at once.  */
 gpg_error_t
 gpg_dirmngr_ks_get (ctrl_t ctrl, char **pattern,
-                    keyserver_spec_t override_keyserver, int quick,
+                    keyserver_spec_t override_keyserver,
                     estream_t *r_fp, char **r_source)
 {
   gpg_error_t err;
@@ -683,7 +657,7 @@ gpg_dirmngr_ks_get (ctrl_t ctrl, char **pattern,
 
   /* Lump all patterns into one string.  */
   init_membuf (&mb, 1024);
-  put_membuf_str (&mb, quick? "KS_GET --quick --" : "KS_GET --");
+  put_membuf_str (&mb, "KS_GET --");
   for (idx=0; pattern[idx]; idx++)
     {
       put_membuf (&mb, " ", 1); /* Append Delimiter.  */
@@ -830,7 +804,7 @@ record_output (estream_t output,
       type_str = "sig";
       break;
     default:
-      log_assert (! "Unhandled type.");
+      assert (! "Unhandled type.");
     }
 
   if (pub_key_length > 0)
@@ -989,9 +963,9 @@ ks_put_inq_cb (void *opaque, const char *line)
 		    int i;
 
 		    i = 0;
-		    if (uid->flags.revoked)
+		    if (uid->is_revoked)
 		      validity[i ++] = 'r';
-		    if (uid->flags.expired)
+		    if (uid->is_expired)
 		      validity[i ++] = 'e';
 		    validity[i] = '\0';
 
@@ -1070,7 +1044,7 @@ gpg_dirmngr_ks_put (ctrl_t ctrl, void *data, size_t datalen, kbnode_t keyblock)
   /* We are going to parse the keyblock, thus we better make sure the
      all information is readily available.  */
   if (keyblock)
-    merge_keys_and_selfsig (ctrl, keyblock);
+    merge_keys_and_selfsig (keyblock);
 
   err = open_context (ctrl, &ctx);
   if (err)
@@ -1090,7 +1064,7 @@ gpg_dirmngr_ks_put (ctrl_t ctrl, void *data, size_t datalen, kbnode_t keyblock)
 
 
 
-/* Data callback for the DNS_CERT and WKD_GET commands. */
+/* Data callback for the DNS_CERT command. */
 static gpg_error_t
 dns_cert_data_cb (void *opaque, const void *data, size_t datalen)
 {
@@ -1313,76 +1287,3 @@ gpg_dirmngr_get_pka (ctrl_t ctrl, const char *userid,
   close_context (ctrl, ctx);
   return err;
 }
-
-
-
-/* Ask the dirmngr to retrieve a key via the Web Key Directory
- * protocol.  If QUICK is set the dirmngr is advised to use a shorter
- * timeout.  On success a new estream with the key stored at R_KEY and the
- * url of the lookup (if any) stored at R_URL.  Note that
- */
-gpg_error_t
-gpg_dirmngr_wkd_get (ctrl_t ctrl, const char *name, int quick,
-                     estream_t *r_key, char **r_url)
-{
-  gpg_error_t err;
-  assuan_context_t ctx;
-  struct ks_status_parm_s stparm = { NULL };
-  struct dns_cert_parm_s parm = { NULL };
-  char *line = NULL;
-
-  if (r_key)
-    *r_key = NULL;
-
-  if (r_url)
-    *r_url = NULL;
-
-  err = open_context (ctrl, &ctx);
-  if (err)
-    return err;
-
-  line = es_bsprintf ("WKD_GET%s -- %s", quick?" --quick":"", name);
-  if (!line)
-    {
-      err = gpg_error_from_syserror ();
-      goto leave;
-    }
-  if (strlen (line) + 2 >= ASSUAN_LINELENGTH)
-    {
-      err = gpg_error (GPG_ERR_TOO_LARGE);
-      goto leave;
-    }
-
-  parm.memfp = es_fopenmem (0, "rwb");
-  if (!parm.memfp)
-    {
-      err = gpg_error_from_syserror ();
-      goto leave;
-    }
-  err = assuan_transact (ctx, line, dns_cert_data_cb, &parm,
-                         NULL, NULL, ks_status_cb, &stparm);
-  if (err)
-    goto leave;
-
-  if (r_key)
-    {
-      es_rewind (parm.memfp);
-      *r_key = parm.memfp;
-      parm.memfp = NULL;
-    }
-
-  if (r_url)
-    {
-      *r_url = stparm.source;
-      stparm.source = NULL;
-    }
-
- leave:
-  xfree (stparm.source);
-  xfree (parm.fpr);
-  xfree (parm.url);
-  es_fclose (parm.memfp);
-  xfree (line);
-  close_context (ctrl, ctx);
-  return err;
-}
