@@ -15,20 +15,21 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, see <https://www.gnu.org/licenses/>.
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
 
 #include <config.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <assert.h>
 
 #include "gpg.h"
-#include "../common/util.h"
+#include "util.h"
 #include "packet.h"
 #include "options.h"
-#include "../common/i18n.h"
-#include "../common/status.h"
+#include "i18n.h"
+#include "status.h"
 
 
 static int mdc_decode_filter ( void *opaque, int control, IOBUF a,
@@ -56,7 +57,7 @@ release_dfx_context (decode_filter_ctx_t dfx)
   if (!dfx)
     return;
 
-  log_assert (dfx->refcount);
+  assert (dfx->refcount);
   if ( !--dfx->refcount )
     {
       gcry_cipher_close (dfx->cipher_hd);
@@ -222,7 +223,7 @@ decrypt_data (ctrl_t ctrl, void *procctx, PKT_encrypted *ed, DEK *dek)
 
   if (opt.unwrap_encryption)
     {
-      char *filename = NULL;
+      char *filename;
       estream_t fp;
       rc = get_output_file ("", 0, ed->buf, &filename, &fp);
       if (! rc)
@@ -248,7 +249,6 @@ decrypt_data (ctrl_t ctrl, void *procctx, PKT_encrypted *ed, DEK *dek)
           if (afx)
             release_armor_context (afx);
         }
-      xfree (filename);
     }
   else
     proc_packets (ctrl, procctx, ed->buf );
@@ -273,8 +273,8 @@ decrypt_data (ctrl_t ctrl, void *procctx, PKT_encrypted *ed, DEK *dek)
          bytes are appended.  */
       int datalen = gcry_md_get_algo_dlen (ed->mdc_method);
 
-      log_assert (dfx->cipher_hd);
-      log_assert (dfx->mdc_hash);
+      assert (dfx->cipher_hd);
+      assert (dfx->mdc_hash);
       gcry_cipher_decrypt (dfx->cipher_hd, dfx->defer, 22, NULL, 0);
       gcry_md_write (dfx->mdc_hash, dfx->defer, 2);
       gcry_md_final (dfx->mdc_hash);
@@ -320,8 +320,8 @@ mdc_decode_filter (void *opaque, int control, IOBUF a,
     }
   else if( control == IOBUFCTRL_UNDERFLOW )
     {
-      log_assert (a);
-      log_assert (size > 44); /* Our code requires at least this size.  */
+      assert (a);
+      assert (size > 44); /* Our code requires at least this size.  */
 
       /* Get at least 22 bytes and put it ahead in the buffer.  */
       if (dfx->partial)
@@ -414,7 +414,7 @@ mdc_decode_filter (void *opaque, int control, IOBUF a,
 	}
       else
         {
-          log_assert ( dfx->eof_seen );
+          assert ( dfx->eof_seen );
           rc = -1; /* Return EOF.  */
 	}
       *ret_len = n;
@@ -447,7 +447,7 @@ decode_filter( void *opaque, int control, IOBUF a, byte *buf, size_t *ret_len)
     }
   else if ( control == IOBUFCTRL_UNDERFLOW )
     {
-      log_assert (a);
+      assert(a);
 
       if (fc->partial)
         {
