@@ -1,7 +1,6 @@
 /* encrypt.c - Main encryption driver
  * Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
  *               2006, 2009 Free Software Foundation, Inc.
- * Copyright (C) 2016 g10 Code GmbH
  *
  * This file is part of GnuPG.
  *
@@ -16,7 +15,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, see <https://www.gnu.org/licenses/>.
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
 
 #include <config.h>
@@ -24,25 +23,25 @@
 #include <stdlib.h>
 #include <string.h>
 #include <errno.h>
+#include <assert.h>
 
 #include "gpg.h"
 #include "options.h"
 #include "packet.h"
-#include "../common/status.h"
-#include "../common/iobuf.h"
+#include "status.h"
+#include "iobuf.h"
 #include "keydb.h"
-#include "../common/util.h"
+#include "util.h"
 #include "main.h"
 #include "filter.h"
 #include "trustdb.h"
-#include "../common/i18n.h"
-#include "../common/status.h"
+#include "i18n.h"
+#include "status.h"
 #include "pkglue.h"
 
 
 static int encrypt_simple( const char *filename, int mode, int use_seskey );
-static int write_pubkey_enc_from_list (ctrl_t ctrl,
-                                       PK_LIST pk_list, DEK *dek, iobuf_t out);
+static int write_pubkey_enc_from_list( PK_LIST pk_list, DEK *dek, iobuf_t out );
 
 /****************
  * Encrypt FILENAME with only the symmetric cipher.  Take input from
@@ -66,24 +65,17 @@ encrypt_store (const char *filename)
 }
 
 
-/* *SESKEY contains the unencrypted session key ((*SESKEY)->KEY) and
-   the algorithm that will be used to encrypt the contents of the SED
-   packet ((*SESKEY)->ALGO).  If *SESKEY is NULL, then a random
-   session key that is appropriate for DEK->ALGO is generated and
-   stored there.
-
-   Encrypt that session key using DEK and store the result in ENCKEY,
-   which must be large enough to hold (*SESKEY)->KEYLEN + 1 bytes.  */
-void
+static void
 encrypt_seskey (DEK *dek, DEK **seskey, byte *enckey)
 {
   gcry_cipher_hd_t hd;
   byte buf[33];
 
-  log_assert ( dek->keylen <= 32 );
+  assert ( dek->keylen <= 32 );
   if (!*seskey)
     {
       *seskey=xmalloc_clear(sizeof(DEK));
+      (*seskey)->keylen=dek->keylen;
       (*seskey)->algo=dek->algo;
       make_session_key(*seskey);
       /*log_hexdump( "thekey", c->key, c->keylen );*/
@@ -222,7 +214,8 @@ encrypt_simple (const char *filename, int mode, int use_seskey)
       s2k = xmalloc_clear( sizeof *s2k );
       s2k->mode = opt.s2k_mode;
       s2k->hash_algo = S2K_DIGEST_ALGO;
-      cfx.dek = passphrase_to_dek (default_cipher_algo (), s2k, 1, 0,
+      cfx.dek = passphrase_to_dek (NULL, 0,
+                                   default_cipher_algo(), s2k, 4,
                                    NULL, &canceled);
       if ( !cfx.dek || !cfx.dek->keylen )
         {
@@ -320,7 +313,7 @@ encrypt_simple (const char *filename, int mode, int use_seskey)
            && !overflow && opt.verbose)
         log_info(_("WARNING: '%s' is an empty file\n"), filename );
       /* We can't encode the length of very large files because
-         OpenPGP uses only 32 bit for file sizes.  So if the
+         OpenPGP uses only 32 bit for file sizes.  So if the the
          size of a file is larger than 2^32 minus some bytes for
          packet headers, we switch to partial length encoding. */
       if ( tmpsize < (IOBUF_FILELENGTH_LIMIT - 65536) )
@@ -333,9 +326,9 @@ encrypt_simple (const char *filename, int mode, int use_seskey)
 
   if (!opt.no_literal)
     {
-      /* Note that PT has been initialized above in !no_literal mode.  */
+      /* Note that PT has been initialized above in no_literal mode.  */
       pt->timestamp = make_timestamp();
-      pt->mode = opt.mimemode? 'm' : opt.textmode? 't' : 'b';
+      pt->mode = opt.textmode? 't' : 'b';
       pt->len = filesize;
       pt->new_ctb = !pt->len;
       pt->buf = inp;
@@ -395,7 +388,7 @@ encrypt_simple (const char *filename, int mode, int use_seskey)
     }
   if (pt)
     pt->buf = NULL;
-  free_packet (&pkt, NULL);
+  free_packet (&pkt);
   xfree (cfx.dek);
   xfree (s2k);
   release_armor_context (afx);
@@ -413,8 +406,8 @@ setup_symkey (STRING2KEY **symkey_s2k,DEK **symkey_dek)
   (*symkey_s2k)->mode = opt.s2k_mode;
   (*symkey_s2k)->hash_algo = S2K_DIGEST_ALGO;
 
-  *symkey_dek = passphrase_to_dek (opt.s2k_cipher_algo,
-                                   *symkey_s2k, 1, 0, NULL, &canceled);
+  *symkey_dek=passphrase_to_dek(NULL,0,opt.s2k_cipher_algo,
+				*symkey_s2k, 4, NULL, &canceled);
   if(!*symkey_dek || !(*symkey_dek)->keylen)
     {
       xfree(*symkey_dek);
@@ -462,7 +455,7 @@ write_symkey_enc (STRING2KEY *symkey_s2k, DEK *symkey_dek, DEK *dek,
  * The caller may provide a checked list of public keys in
  * PROVIDED_PKS; if not the function builds a list of keys on its own.
  *
- * Note that FILEFD is currently only used by cmd_encrypt in the
+ * Note that FILEFD is currently only used by cmd_encrypt in the the
  * not yet finished server.c.
  */
 int
@@ -635,7 +628,7 @@ encrypt_crypt (ctrl_t ctrl, int filefd, const char *filename,
   if (DBG_CRYPTO)
     log_printhex ("DEK is: ", cfx.dek->key, cfx.dek->keylen );
 
-  rc = write_pubkey_enc_from_list (ctrl, pk_list, cfx.dek, out);
+  rc = write_pubkey_enc_from_list (pk_list, cfx.dek, out);
   if (rc)
     goto leave;
 
@@ -660,7 +653,7 @@ encrypt_crypt (ctrl_t ctrl, int filefd, const char *filename,
            && !overflow && opt.verbose)
         log_info(_("WARNING: '%s' is an empty file\n"), filename );
       /* We can't encode the length of very large files because
-         OpenPGP uses only 32 bit for file sizes.  So if the size
+         OpenPGP uses only 32 bit for file sizes.  So if the the size
          of a file is larger than 2^32 minus some bytes for packet
          headers, we switch to partial length encoding. */
       if (tmpsize < (IOBUF_FILELENGTH_LIMIT - 65536) )
@@ -674,7 +667,7 @@ encrypt_crypt (ctrl_t ctrl, int filefd, const char *filename,
   if (!opt.no_literal)
     {
       pt->timestamp = make_timestamp();
-      pt->mode = opt.mimemode? 'm' : opt.textmode ? 't' : 'b';
+      pt->mode = opt.textmode ? 't' : 'b';
       pt->len = filesize;
       pt->new_ctb = !pt->len;
       pt->buf = inp;
@@ -756,7 +749,7 @@ encrypt_crypt (ctrl_t ctrl, int filefd, const char *filename,
     }
   if (pt)
     pt->buf = NULL;
-  free_packet (&pkt, NULL);
+  free_packet (&pkt);
   xfree (cfx.dek);
   xfree (symkey_dek);
   xfree (symkey_s2k);
@@ -829,23 +822,22 @@ encrypt_filter (void *opaque, int control,
           if (DBG_CRYPTO)
             log_printhex ("DEK is: ", efx->cfx.dek->key, efx->cfx.dek->keylen);
 
-          rc = write_pubkey_enc_from_list (efx->ctrl,
-                                           efx->pk_list, efx->cfx.dek, a);
+          rc = write_pubkey_enc_from_list (efx->pk_list, efx->cfx.dek, a);
           if (rc)
             return rc;
 
-          if(efx->symkey_s2k && efx->symkey_dek)
-            {
-              rc=write_symkey_enc(efx->symkey_s2k,efx->symkey_dek,
-                                  efx->cfx.dek,a);
-              if(rc)
-                return rc;
-            }
+	    if(efx->symkey_s2k && efx->symkey_dek)
+	      {
+		rc=write_symkey_enc(efx->symkey_s2k,efx->symkey_dek,
+				    efx->cfx.dek,a);
+		if(rc)
+		  return rc;
+	      }
 
-          iobuf_push_filter (a, cipher_filter, &efx->cfx);
+	    iobuf_push_filter (a, cipher_filter, &efx->cfx);
 
-          efx->header_okay = 1;
-        }
+	    efx->header_okay = 1;
+	}
       rc = iobuf_write (a, buf, size);
 
     }
@@ -862,92 +854,82 @@ encrypt_filter (void *opaque, int control,
 }
 
 
-/*
- * Write a pubkey-enc packet for the public key PK to OUT.
- */
-int
-write_pubkey_enc (ctrl_t ctrl,
-                  PKT_public_key *pk, int throw_keyid, DEK *dek, iobuf_t out)
-{
-  PACKET pkt;
-  PKT_pubkey_enc *enc;
-  int rc;
-  gcry_mpi_t frame;
-
-  print_pubkey_algo_note ( pk->pubkey_algo );
-  enc = xmalloc_clear ( sizeof *enc );
-  enc->pubkey_algo = pk->pubkey_algo;
-  keyid_from_pk( pk, enc->keyid );
-  enc->throw_keyid = throw_keyid;
-
-  /* Okay, what's going on: We have the session key somewhere in
-   * the structure DEK and want to encode this session key in an
-   * integer value of n bits. pubkey_nbits gives us the number of
-   * bits we have to use.  We then encode the session key in some
-   * way and we get it back in the big intger value FRAME.  Then
-   * we use FRAME, the public key PK->PKEY and the algorithm
-   * number PK->PUBKEY_ALGO and pass it to pubkey_encrypt which
-   * returns the encrypted value in the array ENC->DATA.  This
-   * array has a size which depends on the used algorithm (e.g. 2
-   * for Elgamal).  We don't need frame anymore because we have
-   * everything now in enc->data which is the passed to
-   * build_packet().  */
-  frame = encode_session_key (pk->pubkey_algo, dek,
-                              pubkey_nbits (pk->pubkey_algo, pk->pkey));
-  rc = pk_encrypt (pk->pubkey_algo, enc->data, frame, pk, pk->pkey);
-  gcry_mpi_release (frame);
-  if (rc)
-    log_error ("pubkey_encrypt failed: %s\n", gpg_strerror (rc) );
-  else
-    {
-      if ( opt.verbose )
-        {
-          char *ustr = get_user_id_string_native (ctrl, enc->keyid);
-          log_info (_("%s/%s encrypted for: \"%s\"\n"),
-                    openpgp_pk_algo_name (enc->pubkey_algo),
-                    openpgp_cipher_algo_name (dek->algo),
-                    ustr );
-          xfree (ustr);
-        }
-      /* And write it. */
-      init_packet (&pkt);
-      pkt.pkttype = PKT_PUBKEY_ENC;
-      pkt.pkt.pubkey_enc = enc;
-      rc = build_packet (out, &pkt);
-      if (rc)
-        log_error ("build_packet(pubkey_enc) failed: %s\n",
-                   gpg_strerror (rc));
-    }
-  free_pubkey_enc(enc);
-  return rc;
-}
-
-
 /*
  * Write pubkey-enc packets from the list of PKs to OUT.
  */
 static int
-write_pubkey_enc_from_list (ctrl_t ctrl, PK_LIST pk_list, DEK *dek, iobuf_t out)
+write_pubkey_enc_from_list (PK_LIST pk_list, DEK *dek, iobuf_t out)
 {
-  if (opt.throw_keyids && (PGP6 || PGP7 || PGP8))
-    {
-      log_info(_("you may not use %s while in %s mode\n"),
-               "--throw-keyids",compliance_option_string());
-      compliance_failure();
-    }
+  PACKET pkt;
+  PKT_public_key *pk;
+  PKT_pubkey_enc  *enc;
+  int rc;
 
   for ( ; pk_list; pk_list = pk_list->next )
     {
-      PKT_public_key *pk = pk_list->pk;
-      int throw_keyid = (opt.throw_keyids || (pk_list->flags&1));
-      int rc = write_pubkey_enc (ctrl, pk, throw_keyid, dek, out);
+      gcry_mpi_t frame;
+
+      pk = pk_list->pk;
+
+      print_pubkey_algo_note ( pk->pubkey_algo );
+      enc = xmalloc_clear ( sizeof *enc );
+      enc->pubkey_algo = pk->pubkey_algo;
+      keyid_from_pk( pk, enc->keyid );
+      enc->throw_keyid = (opt.throw_keyids || (pk_list->flags&1));
+
+      if (opt.throw_keyids && (PGP6 || PGP7 || PGP8))
+        {
+          log_info(_("you may not use %s while in %s mode\n"),
+                   "--throw-keyids",compliance_option_string());
+          compliance_failure();
+        }
+
+      /* Okay, what's going on: We have the session key somewhere in
+       * the structure DEK and want to encode this session key in an
+       * integer value of n bits. pubkey_nbits gives us the number of
+       * bits we have to use.  We then encode the session key in some
+       * way and we get it back in the big intger value FRAME.  Then
+       * we use FRAME, the public key PK->PKEY and the algorithm
+       * number PK->PUBKEY_ALGO and pass it to pubkey_encrypt which
+       * returns the encrypted value in the array ENC->DATA.  This
+       * array has a size which depends on the used algorithm (e.g. 2
+       * for Elgamal).  We don't need frame anymore because we have
+       * everything now in enc->data which is the passed to
+       * build_packet().  */
+      frame = encode_session_key (pk->pubkey_algo, dek,
+                                  pubkey_nbits (pk->pubkey_algo, pk->pkey));
+      rc = pk_encrypt (pk->pubkey_algo, enc->data, frame, pk, pk->pkey);
+      gcry_mpi_release (frame);
+      if (rc)
+        log_error ("pubkey_encrypt failed: %s\n", gpg_strerror (rc) );
+      else
+        {
+          if ( opt.verbose )
+            {
+              char *ustr = get_user_id_string_native (enc->keyid);
+              log_info (_("%s/%s encrypted for: \"%s\"\n"),
+                        openpgp_pk_algo_name (enc->pubkey_algo),
+                        openpgp_cipher_algo_name (dek->algo),
+                        ustr );
+              xfree (ustr);
+	    }
+          /* And write it. */
+          init_packet (&pkt);
+          pkt.pkttype = PKT_PUBKEY_ENC;
+          pkt.pkt.pubkey_enc = enc;
+          rc = build_packet (out, &pkt);
+          if (rc)
+            log_error ("build_packet(pubkey_enc) failed: %s\n",
+                       gpg_strerror (rc));
+	}
+      free_pubkey_enc(enc);
       if (rc)
         return rc;
     }
-
   return 0;
 }
 
+
 void
 encrypt_crypt_files (ctrl_t ctrl, int nfiles, char **files, strlist_t remusr)
 {
