@@ -22,6 +22,7 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <assert.h>
 
 #include "gpg.h"
 #include "util.h"
@@ -82,7 +83,6 @@ free_seckey_enc( PKT_signature *sig )
       xfree (sig->pka_info->uri);
       xfree (sig->pka_info);
     }
-  xfree (sig->signers_uid);
 
   xfree(sig);
 }
@@ -259,8 +259,6 @@ copy_signature( PKT_signature *d, PKT_signature *s )
     d->pka_info = s->pka_info? cp_pka_info (s->pka_info) : NULL;
     d->hashed = cp_subpktarea (s->hashed);
     d->unhashed = cp_subpktarea (s->unhashed);
-    if (s->signers_uid)
-      d->signers_uid = xstrdup (s->signers_uid);
     if(s->numrevkeys)
       {
 	d->revkey=NULL;
@@ -304,14 +302,13 @@ free_attributes(PKT_user_id *uid)
 void
 free_user_id (PKT_user_id *uid)
 {
-    log_assert (uid->ref > 0);
+    assert (uid->ref > 0);
     if (--uid->ref)
         return;
 
     free_attributes(uid);
     xfree (uid->prefs);
     xfree (uid->namehash);
-    xfree (uid->mbox);
     xfree (uid);
 }
 
