@@ -15,7 +15,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, see <https://www.gnu.org/licenses/>.
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
 
 #include <config.h>
@@ -23,6 +23,7 @@
 #include <stdlib.h>
 #include <string.h>
 #include <errno.h>
+#include <assert.h>
 #include <sys/types.h>
 #ifdef HAVE_DOSISH_SYSTEM
 # include <fcntl.h> /* for setmode() */
@@ -108,25 +109,9 @@ get_output_file (const byte *embedded_name, int embedded_namelen,
     }
   else if (iobuf_is_pipe_filename (fname) || !*fname)
     {
-      /* Special file name, no filename, or "-" given; write to the
-       * file descriptor or to stdout. */
-      int fd;
-      char xname[64];
-
-      fd = check_special_filename (fname, 1, 0);
-      if (fd == -1)
-        {
-          /* Not a special filename, thus we want stdout.  */
-          fp = es_stdout;
-          es_set_binary (fp);
-        }
-      else if (!(fp = es_fdopen_nc (fd, "wb")))
-        {
-          err = gpg_error_from_syserror ();
-          snprintf (xname, sizeof xname, "[fd %d]", fd);
-          log_error (_("can't open '%s': %s\n"), xname, gpg_strerror (err));
-          goto leave;
-        }
+      /* No filename or "-" given; write to stdout. */
+      fp = es_stdout;
+      es_set_binary (fp);
     }
   else
     {
@@ -233,16 +218,11 @@ handle_plaintext (PKT_plaintext * pt, md_filter_context_t * mfx,
   static off_t count = 0;
   int err = 0;
   int c;
-  int convert;
+  int convert = (pt->mode == 't' || pt->mode == 'u');
 #ifdef __riscos__
   int filetype = 0xfff;
 #endif
 
-  if (pt->mode == 't' || pt->mode == 'u' || pt->mode == 'm')
-    convert = pt->mode;
-  else
-    convert = 0;
-
   /* Let people know what the plaintext info is. This allows the
      receiving program to try and do something different based on the
      format code (say, recode UTF-8 to local). */
@@ -300,10 +280,8 @@ handle_plaintext (PKT_plaintext * pt, md_filter_context_t * mfx,
 	      if (mfx->md)
 		gcry_md_putc (mfx->md, c);
 #ifndef HAVE_DOSISH_SYSTEM
-              /* Convert to native line ending. */
-              /* fixme: this hack might be too simple */
-	      if (c == '\r' && convert != 'm')
-		continue;
+	      if (c == '\r')	/* convert to native line ending */
+		continue;	/* fixme: this hack might be too simple */
 #endif
 	      if (fp)
 		{
@@ -377,7 +355,7 @@ handle_plaintext (PKT_plaintext * pt, md_filter_context_t * mfx,
 	      if (mfx->md)
 		gcry_md_putc (mfx->md, c);
 #ifndef HAVE_DOSISH_SYSTEM
-	      if (c == '\r' && convert != 'm')
+	      if (convert && c == '\r')
 		continue;	/* fixme: this hack might be too simple */
 #endif
 	      if (fp)
@@ -658,7 +636,7 @@ ask_for_detached_datafile (gcry_md_hd_t md, gcry_md_hd_t md2,
       if (opt.verbose)
 	log_info (_("reading stdin ...\n"));
       fp = iobuf_open (NULL);
-      log_assert (fp);
+      assert (fp);
     }
   do_hash (md, md2, fp, textmode);
   iobuf_close (fp);
