@@ -15,7 +15,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, see <https://www.gnu.org/licenses/>.
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
 
 #include <config.h>
@@ -23,15 +23,15 @@
 #include <stdlib.h>
 #include <string.h>
 #include <errno.h>
+#include <assert.h>
 
 #include "gpg.h"
 #include "options.h"
 #include "packet.h"
-#include "../common/status.h"
+#include "status.h"
 #include "keydb.h"
-#include "../common/util.h"
-#include "../common/i18n.h"
-#include "call-agent.h"
+#include "util.h"
+#include "i18n.h"
 
 
 /* Return true if Libgcrypt's RNG is in faked mode.  */
@@ -60,14 +60,14 @@ release_sk_list (SK_LIST sk_list)
  * the string "(insecure!)" or "not secure" or "do not use"
  * in one of the user ids.  */
 static int
-is_insecure (ctrl_t ctrl, PKT_public_key *pk)
+is_insecure (PKT_public_key *pk)
 {
   u32 keyid[2];
   KBNODE node = NULL, u;
   int insecure = 0;
 
   keyid_from_pk (pk, keyid);
-  node = get_pubkeyblock (ctrl, keyid);
+  node = get_pubkeyblock (keyid);
   for (u = node; u; u = u->next)
     {
       if (u->pkt->pkttype == PKT_USER_ID)
@@ -127,38 +127,13 @@ build_sk_list (ctrl_t ctrl,
      select the best key.  If a key specification is ambiguous and we
      are in batch mode, die.  */
 
-  if (!locusr) /* No user ids given - use the card key or the default key.  */
+  if (!locusr) /* No user ids given - use the default key.  */
     {
-      struct agent_card_info_s info;
       PKT_public_key *pk;
-      char *serialno;
 
-      memset (&info, 0, sizeof(info));
       pk = xmalloc_clear (sizeof *pk);
       pk->req_usage = use;
-
-      /* Check if a card is available.  If any, use it.  */
-      err = agent_scd_serialno (&serialno, NULL);
-      if (!err)
-        {
-          xfree (serialno);
-          err = agent_scd_getattr ("KEY-FPR", &info);
-          if (err)
-            log_error ("error retrieving key fingerprint from card: %s\n",
-                       gpg_strerror (err));
-          else if (info.fpr1valid)
-            {
-              if ((err = get_pubkey_byfprint (ctrl, pk, NULL, info.fpr1, 20)))
-                {
-                  info.fpr1valid = 0;
-                  log_error ("error on card key to sign: %s, try default\n",
-                             gpg_strerror (err));
-                }
-            }
-        }
-
-      if (!info.fpr1valid
-          && (err = getkey_byname (ctrl, NULL, pk, NULL, 1, NULL)))
+      if ((err = getkey_byname (ctrl, NULL, pk, NULL, 1, NULL)))
 	{
 	  free_public_key (pk);
 	  pk = NULL;
@@ -176,7 +151,7 @@ build_sk_list (ctrl_t ctrl,
 	{
 	  SK_LIST r;
 
-	  if (random_is_faked () && !is_insecure (ctrl, pk))
+	  if (random_is_faked () && !is_insecure (pk))
 	    {
 	      log_info (_("key is not flagged as insecure - "
 			  "can't use it with the faked RNG!\n"));
@@ -257,7 +232,7 @@ build_sk_list (ctrl_t ctrl,
 		     get_inv_recpsgnr_code (GPG_ERR_WRONG_KEY_USAGE),
 		     locusr->d, strlen (locusr->d), -1);
 		}
-	      else if (random_is_faked () && !is_insecure (ctrl, pk))
+	      else if (random_is_faked () && !is_insecure (pk))
 		{
 		  log_info (_("key is not flagged as insecure - "
 			      "can't use it with the faked RNG!\n"));
