@@ -15,7 +15,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, see <https://www.gnu.org/licenses/>.
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
 
 #include <config.h>
@@ -28,18 +28,18 @@
 
 #define INCLUDED_BY_MAIN_MODULE 1
 #include "gpg.h"
-#include "../common/util.h"
+#include "util.h"
 #include "packet.h"
-#include "../common/iobuf.h"
+#include "iobuf.h"
 #include "main.h"
 #include "options.h"
 #include "keydb.h"
 #include "trustdb.h"
 #include "filter.h"
-#include "../common/ttyio.h"
-#include "../common/i18n.h"
-#include "../common/sysutils.h"
-#include "../common/status.h"
+#include "ttyio.h"
+#include "i18n.h"
+#include "sysutils.h"
+#include "status.h"
 #include "call-agent.h"
 
 int g10_errors_seen;
@@ -58,20 +58,17 @@ g10_exit( int rc )
  * this utility assumes that all keys in the keyring are trustworthy
  */
 int
-check_signatures_trust (ctrl_t ctrl, PKT_signature *sig)
+check_signatures_trust( PKT_signature *sig )
 {
-  (void)ctrl;
   (void)sig;
   return 0;
 }
 
 void
-read_trust_options (ctrl_t ctrl,
-                    byte *trust_model, ulong *created, ulong *nextcheck,
-                    byte *marginals, byte *completes, byte *cert_depth,
-                    byte *min_cert_level)
+read_trust_options(byte *trust_model, ulong *created, ulong *nextcheck,
+		   byte *marginals, byte *completes, byte *cert_depth,
+		   byte *min_cert_level)
 {
-  (void)ctrl;
   (void)trust_model;
   (void)created;
   (void)nextcheck;
@@ -87,36 +84,29 @@ read_trust_options (ctrl_t ctrl,
  */
 
 int
-cache_disabled_value (ctrl_t ctrl, PKT_public_key *pk)
+cache_disabled_value(PKT_public_key *pk)
 {
-  (void)ctrl;
   (void)pk;
   return 0;
 }
 
 void
-check_trustdb_stale (ctrl_t ctrl)
+check_trustdb_stale(void)
 {
-  (void)ctrl;
 }
 
 int
-get_validity_info (ctrl_t ctrl, kbnode_t kb, PKT_public_key *pk,
-                   PKT_user_id *uid)
+get_validity_info (PKT_public_key *pk, PKT_user_id *uid)
 {
-  (void)ctrl;
-  (void)kb;
   (void)pk;
   (void)uid;
   return '?';
 }
 
 unsigned int
-get_validity (ctrl_t ctrl, kbnode_t kb, PKT_public_key *pk, PKT_user_id *uid,
-              PKT_signature *sig, int may_ask)
+get_validity (PKT_public_key *pk, PKT_user_id *uid, PKT_signature *sig,
+	      int may_ask)
 {
-  (void)ctrl;
-  (void)kb;
   (void)pk;
   (void)uid;
   (void)sig;
@@ -132,27 +122,23 @@ trust_value_to_string (unsigned int value)
 }
 
 const char *
-uid_trust_string_fixed (ctrl_t ctrl, PKT_public_key *key, PKT_user_id *uid)
+uid_trust_string_fixed (PKT_public_key *key, PKT_user_id *uid)
 {
-  (void)ctrl;
   (void)key;
   (void)uid;
   return "err";
 }
 
 int
-get_ownertrust_info (ctrl_t ctrl, PKT_public_key *pk, int no_create)
+get_ownertrust_info (PKT_public_key *pk)
 {
-  (void)ctrl;
   (void)pk;
-  (void)no_create;
   return '?';
 }
 
 unsigned int
-get_ownertrust (ctrl_t ctrl, PKT_public_key *pk)
+get_ownertrust (PKT_public_key *pk)
 {
-  (void)ctrl;
   (void)pk;
   return TRUST_UNKNOWN;
 }
@@ -178,23 +164,10 @@ keyserver_any_configured (ctrl_t ctrl)
 }
 
 int
-keyserver_import_keyid (u32 *keyid, void *dummy, int quick)
+keyserver_import_keyid (u32 *keyid, void *dummy)
 {
   (void)keyid;
   (void)dummy;
-  (void)quick;
-  return -1;
-}
-
-int
-keyserver_import_fprint (ctrl_t ctrl, const byte *fprint,size_t fprint_len,
-			 struct keyserver_spec *keyserver, int quick)
-{
-  (void)ctrl;
-  (void)fprint;
-  (void)fprint_len;
-  (void)keyserver;
-  (void)quick;
   return -1;
 }
 
@@ -213,18 +186,6 @@ keyserver_import_pka (const char *name,unsigned char *fpr)
   return -1;
 }
 
-gpg_error_t
-keyserver_import_wkd (ctrl_t ctrl, const char *name, int quick,
-                      unsigned char **fpr, size_t *fpr_len)
-{
-  (void)ctrl;
-  (void)name;
-  (void)quick;
-  (void)fpr;
-  (void)fpr_len;
-  return GPG_ERR_BUG;
-}
-
 int
 keyserver_import_name (const char *name,struct keyserver_spec *spec)
 {
@@ -240,15 +201,6 @@ keyserver_import_ldap (const char *name)
   return -1;
 }
 
-gpg_error_t
-read_key_from_file (ctrl_t ctrl, const char *fname, kbnode_t *r_keyblock)
-{
-  (void)ctrl;
-  (void)fname;
-  (void)r_keyblock;
-  return -1;
-}
-
 /* Stub:
  * No encryption here but mainproc links to these functions.
  */
@@ -306,13 +258,15 @@ check_secret_key (PKT_public_key *pk, int n)
  * No secret key, so no passphrase needed
  */
 DEK *
-passphrase_to_dek (int cipher_algo, STRING2KEY *s2k, int create, int nocache,
+passphrase_to_dek (u32 *keyid, int pubkey_algo,
+                   int cipher_algo, STRING2KEY *s2k, int mode,
                    const char *tmp, int *canceled)
 {
+  (void)keyid;
+  (void)pubkey_algo;
   (void)cipher_algo;
   (void)s2k;
-  (void)create;
-  (void)nocache;
+  (void)mode;
   (void)tmp;
 
   if (canceled)
@@ -321,9 +275,11 @@ passphrase_to_dek (int cipher_algo, STRING2KEY *s2k, int create, int nocache,
 }
 
 void
-passphrase_clear_cache (const char *cacheid)
+passphrase_clear_cache (u32 *keyid, const char *cacheid, int algo)
 {
+  (void)keyid;
   (void)cacheid;
+  (void)algo;
 }
 
 struct keyserver_spec *
@@ -443,12 +399,10 @@ agent_probe_any_secret_key (ctrl_t ctrl, kbnode_t keyblock)
 }
 
 gpg_error_t
-agent_get_keyinfo (ctrl_t ctrl, const char *hexkeygrip,
-                   char **r_serialno, int *r_cleartext)
+agent_get_keyinfo (ctrl_t ctrl, const char *hexkeygrip, char **r_serialno)
 {
   (void)ctrl;
   (void)hexkeygrip;
-  (void)r_cleartext;
   *r_serialno = NULL;
   return gpg_error (GPG_ERR_NO_SECKEY);
 }
@@ -486,21 +440,9 @@ export_pubkey_buffer (ctrl_t ctrl, const char *keyspec, unsigned int options,
 }
 
 gpg_error_t
-tofu_write_tfs_record (ctrl_t ctrl, estream_t fp,
-                       PKT_public_key *pk, const char *user_id)
-{
-  (void)ctrl;
-  (void)fp;
-  (void)pk;
-  (void)user_id;
-  return gpg_error (GPG_ERR_GENERAL);
-}
-
-gpg_error_t
-tofu_get_policy (ctrl_t ctrl, PKT_public_key *pk, PKT_user_id *user_id,
+tofu_get_policy (PKT_public_key *pk, PKT_user_id *user_id,
 		 enum tofu_policy *policy)
 {
-  (void)ctrl;
   (void)pk;
   (void)user_id;
   (void)policy;
@@ -516,22 +458,11 @@ tofu_policy_str (enum tofu_policy policy)
 }
 
 void
-tofu_begin_batch_update (ctrl_t ctrl)
+tofu_begin_batch_update (void)
 {
-  (void)ctrl;
 }
 
 void
-tofu_end_batch_update (ctrl_t ctrl)
+tofu_end_batch_update (void)
 {
-  (void)ctrl;
-}
-
-gpg_error_t
-tofu_notice_key_changed (ctrl_t ctrl, kbnode_t kb)
-{
-  (void) ctrl;
-  (void) kb;
-
-  return 0;
 }
