@@ -74,13 +74,10 @@ static int verbose;
 						\
     if (test_result == expected_result)		\
       {						\
-        if (verbose) printf (" ok.\n");         \
+        printf (" ok.\n");			\
       }						\
     else					\
       {						\
-        if (!verbose)                           \
-          printf ("%d. Checking %s...",         \
-                  tests, (description) ?: "");  \
 	printf (" failed.\n");			\
 	printf ("  %s == %s failed.\n",		\
 		STRINGIFY(test),		\
@@ -128,8 +125,7 @@ exit_tests (int force)
 {
   if (tests_failed == 0)
     {
-      if (verbose)
-        printf ("All %d tests passed.\n", tests);
+      printf ("All %d tests passed.\n", tests);
       exit (!!force);
     }
   else
