@@ -16,35 +16,36 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, see <https://www.gnu.org/licenses/>.
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
 
 #include <config.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <assert.h>
 
 #include "gpg.h"
 #include "keydb.h"
-#include "../common/util.h"
+#include "util.h"
 #include "options.h"
 #include "packet.h"
 #include "main.h"
-#include "../common/i18n.h"
+#include "i18n.h"
 #include "trustdb.h"
-#include "../common/host2net.h"
+#include "host2net.h"
 
 
 /* Return true if key is disabled.  Note that this is usually used via
    the pk_is_disabled macro.  */
 int
-cache_disabled_value (ctrl_t ctrl, PKT_public_key *pk)
+cache_disabled_value (PKT_public_key *pk)
 {
 #ifdef NO_TRUST_MODELS
   (void)pk;
   return 0;
 #else
-  return tdb_cache_disabled_value (ctrl, pk);
+  return tdb_cache_disabled_value (pk);
 #endif
 }
 
@@ -131,7 +132,7 @@ string_to_trust_value (const char *str)
 
 
 const char *
-uid_trust_string_fixed (ctrl_t ctrl, PKT_public_key *key, PKT_user_id *uid)
+uid_trust_string_fixed (PKT_public_key *key, PKT_user_id *uid)
 {
   if (!key && !uid)
     {
@@ -145,18 +146,17 @@ uid_trust_string_fixed (ctrl_t ctrl, PKT_public_key *key, PKT_user_id *uid)
          uid are both NULL, or neither are NULL. */
       return _("10 translator see trust.c:uid_trust_string_fixed");
     }
-  else if(uid->flags.revoked || (key && key->flags.revoked))
+  else if(uid->is_revoked || (key && key->flags.revoked))
     return                         _("[ revoked]");
-  else if(uid->flags.expired)
+  else if(uid->is_expired)
     return                         _("[ expired]");
   else if(key)
     {
-      switch (get_validity (ctrl, NULL, key, uid, NULL, 0) & TRUST_MASK)
+      switch (get_validity (key, uid, NULL, 0) & TRUST_MASK)
         {
         case TRUST_UNKNOWN:   return _("[ unknown]");
         case TRUST_EXPIRED:   return _("[ expired]");
         case TRUST_UNDEFINED: return _("[  undef ]");
-        case TRUST_NEVER:     return _("[  never ]");
         case TRUST_MARGINAL:  return _("[marginal]");
         case TRUST_FULLY:     return _("[  full  ]");
         case TRUST_ULTIMATE:  return _("[ultimate]");
@@ -173,24 +173,23 @@ uid_trust_string_fixed (ctrl_t ctrl, PKT_public_key *key, PKT_user_id *uid)
  * The key should be the primary key.
  */
 unsigned int
-get_ownertrust (ctrl_t ctrl, PKT_public_key *pk)
+get_ownertrust (PKT_public_key *pk)
 {
 #ifdef NO_TRUST_MODELS
   (void)pk;
   return TRUST_UNKNOWN;
 #else
-  return tdb_get_ownertrust (ctrl, pk, 0);
+  return tdb_get_ownertrust (pk);
 #endif
 }
 
 
 /*
  * Same as get_ownertrust but this takes the minimum ownertrust value
- * into account, and will bump up the value as needed.  NO_CREATE
- * inhibits creation of a trustdb it that does not yet exists.
+ * into into account, and will bump up the value as needed.
  */
 static int
-get_ownertrust_with_min (ctrl_t ctrl, PKT_public_key *pk, int no_create)
+get_ownertrust_with_min (PKT_public_key *pk)
 {
 #ifdef NO_TRUST_MODELS
   (void)pk;
@@ -198,15 +197,8 @@ get_ownertrust_with_min (ctrl_t ctrl, PKT_public_key *pk, int no_create)
 #else
   unsigned int otrust, otrust_min;
 
-  /* Shortcut instead of doing the same twice in the two tdb_get
-   * functions: If the caller asked not to create a trustdb we call
-   * init_trustdb directly and allow it to fail with an error code for
-   * a non-existing trustdb.  */
-  if (no_create && init_trustdb (ctrl, 1))
-    return TRUST_UNKNOWN;
-
-  otrust = (tdb_get_ownertrust (ctrl, pk, no_create) & TRUST_MASK);
-  otrust_min = tdb_get_min_ownertrust (ctrl, pk, no_create);
+  otrust = (tdb_get_ownertrust (pk) & TRUST_MASK);
+  otrust_min = tdb_get_min_ownertrust (pk);
   if (otrust < otrust_min)
     {
       /* If the trust that the user has set is less than the trust
@@ -225,25 +217,23 @@ get_ownertrust_with_min (ctrl_t ctrl, PKT_public_key *pk, int no_create)
 
 /*
  * Same as get_ownertrust but return a trust letter instead of an
- * value.  This takes the minimum ownertrust value into account.  If
- * NO_CREATE is set, no efforts for creating a trustdb will be taken.
+ * value.  This takes the minimum ownertrust value into account.
  */
 int
-get_ownertrust_info (ctrl_t ctrl, PKT_public_key *pk, int no_create)
+get_ownertrust_info (PKT_public_key *pk)
 {
-  return trust_letter (get_ownertrust_with_min (ctrl, pk, no_create));
+  return trust_letter (get_ownertrust_with_min (pk));
 }
 
 
 /*
  * Same as get_ownertrust but return a trust string instead of an
- * value.  This takes the minimum ownertrust value into account.  If
- * NO_CREATE is set, no efforts for creating a trustdb will be taken.
+ * value.  This takes the minimum ownertrust value into account.
  */
 const char *
-get_ownertrust_string (ctrl_t ctrl, PKT_public_key *pk, int no_create)
+get_ownertrust_string (PKT_public_key *pk)
 {
-  return trust_value_to_string (get_ownertrust_with_min (ctrl, pk, no_create));
+  return trust_value_to_string (get_ownertrust_with_min (pk));
 }
 
 
@@ -252,85 +242,70 @@ get_ownertrust_string (ctrl_t ctrl, PKT_public_key *pk, int no_create)
  * The key should be a primary one.
  */
 void
-update_ownertrust (ctrl_t ctrl, PKT_public_key *pk, unsigned int new_trust)
+update_ownertrust (PKT_public_key *pk, unsigned int new_trust)
 {
 #ifdef NO_TRUST_MODELS
   (void)pk;
   (void)new_trust;
 #else
-  tdb_update_ownertrust (ctrl, pk, new_trust);
+  tdb_update_ownertrust (pk, new_trust);
 #endif
 }
 
 
 int
-clear_ownertrusts (ctrl_t ctrl, PKT_public_key *pk)
+clear_ownertrusts (PKT_public_key *pk)
 {
 #ifdef NO_TRUST_MODELS
   (void)pk;
   return 0;
 #else
-  return tdb_clear_ownertrusts (ctrl, pk);
+  return tdb_clear_ownertrusts (pk);
 #endif
 }
 
 
 void
-revalidation_mark (ctrl_t ctrl)
+revalidation_mark (void)
 {
 #ifndef NO_TRUST_MODELS
-  tdb_revalidation_mark (ctrl);
+  tdb_revalidation_mark ();
 #endif
 }
 
 
 void
-check_trustdb_stale (ctrl_t ctrl)
+check_trustdb_stale (void)
 {
 #ifndef NO_TRUST_MODELS
-  tdb_check_trustdb_stale (ctrl);
-#else
-  (void)ctrl;
+  tdb_check_trustdb_stale ();
 #endif
 }
 
 
 void
-check_or_update_trustdb (ctrl_t ctrl)
+check_or_update_trustdb (void)
 {
 #ifndef NO_TRUST_MODELS
-  tdb_check_or_update (ctrl);
-#else
-  (void)ctrl;
+  tdb_check_or_update ();
 #endif
 }
 
 
 /*
- * Return the validity information for KB/PK (at least one must be
- * non-NULL).  If the namehash is not NULL, the validity of the
- * corresponding user ID is returned, otherwise, a reasonable value
- * for the entire key is returned.
+ * Return the validity information for PK.  If the namehash is not
+ * NULL, the validity of the corresponding user ID is returned,
+ * otherwise, a reasonable value for the entire key is returned.
  */
 unsigned int
-get_validity (ctrl_t ctrl, kbnode_t kb, PKT_public_key *pk, PKT_user_id *uid,
-              PKT_signature *sig, int may_ask)
+get_validity (PKT_public_key *pk, PKT_user_id *uid, PKT_signature *sig,
+	      int may_ask)
 {
   int rc;
   unsigned int validity;
   u32 kid[2];
   PKT_public_key *main_pk;
 
-  if (kb && pk)
-    log_assert (keyid_cmp (pk_main_keyid (pk),
-                           pk_main_keyid (kb->pkt->pkt.public_key)) == 0);
-
-  if (! pk)
-    {
-      log_assert (kb);
-      pk = kb->pkt->pkt.public_key;
-    }
-
   if (uid)
     namehash_from_uid (uid);
 
@@ -338,22 +313,17 @@ get_validity (ctrl_t ctrl, kbnode_t kb, PKT_public_key *pk, PKT_user_id *uid,
   if (pk->main_keyid[0] != kid[0] || pk->main_keyid[1] != kid[1])
     {
       /* This is a subkey - get the mainkey. */
-      if (kb)
-        main_pk = kb->pkt->pkt.public_key;
-      else
+      main_pk = xmalloc_clear (sizeof *main_pk);
+      rc = get_pubkey (main_pk, pk->main_keyid);
+      if (rc)
         {
-          main_pk = xmalloc_clear (sizeof *main_pk);
-          rc = get_pubkey (ctrl, main_pk, pk->main_keyid);
-          if (rc)
-            {
-              char *tempkeystr = xstrdup (keystr (pk->main_keyid));
-              log_error ("error getting main key %s of subkey %s: %s\n",
-                         tempkeystr, keystr (kid), gpg_strerror (rc));
-              xfree (tempkeystr);
-              validity = TRUST_UNKNOWN;
-              goto leave;
-            }
-        }
+	  char *tempkeystr = xstrdup (keystr (pk->main_keyid));
+          log_error ("error getting main key %s of subkey %s: %s\n",
+                     tempkeystr, keystr (kid), gpg_strerror (rc));
+	  xfree (tempkeystr);
+          validity = TRUST_UNKNOWN;
+          goto leave;
+	}
     }
   else
     main_pk = pk;
@@ -361,7 +331,7 @@ get_validity (ctrl_t ctrl, kbnode_t kb, PKT_public_key *pk, PKT_user_id *uid,
 #ifdef NO_TRUST_MODELS
   validity = TRUST_UNKNOWN;
 #else
-  validity = tdb_get_validity_core (ctrl, kb, pk, uid, main_pk, sig, may_ask);
+  validity = tdb_get_validity_core (pk, uid, main_pk, sig, may_ask);
 #endif
 
  leave:
@@ -376,28 +346,21 @@ get_validity (ctrl_t ctrl, kbnode_t kb, PKT_public_key *pk, PKT_user_id *uid,
     validity = ((validity & (~TRUST_MASK | TRUST_FLAG_PENDING_CHECK))
                 | TRUST_EXPIRED);
 
-  if (main_pk != pk && !kb)
+  if (main_pk != pk)
     free_public_key (main_pk);
   return validity;
 }
 
 
 int
-get_validity_info (ctrl_t ctrl, kbnode_t kb, PKT_public_key *pk,
-                   PKT_user_id *uid)
+get_validity_info (PKT_public_key *pk, PKT_user_id *uid)
 {
   int trustlevel;
 
-  if (kb && pk)
-    log_assert (keyid_cmp (pk_main_keyid (pk),
-                           pk_main_keyid (kb->pkt->pkt.public_key)) == 0);
-
-  if (! pk && kb)
-    pk = kb->pkt->pkt.public_key;
   if (!pk)
     return '?';  /* Just in case a NULL PK is passed.  */
 
-  trustlevel = get_validity (ctrl, kb, pk, uid, NULL, 0);
+  trustlevel = get_validity (pk, uid, NULL, 0);
   if ((trustlevel & TRUST_FLAG_REVOKED))
     return 'r';
   return trust_letter (trustlevel);
@@ -405,14 +368,14 @@ get_validity_info (ctrl_t ctrl, kbnode_t kb, PKT_public_key *pk,
 
 
 const char *
-get_validity_string (ctrl_t ctrl, PKT_public_key *pk, PKT_user_id *uid)
+get_validity_string (PKT_public_key *pk, PKT_user_id *uid)
 {
   int trustlevel;
 
   if (!pk)
     return "err";  /* Just in case a NULL PK is passed.  */
 
-  trustlevel = get_validity (ctrl, NULL, pk, uid, NULL, 0);
+  trustlevel = get_validity (pk, uid, NULL, 0);
   if ((trustlevel & TRUST_FLAG_REVOKED))
     return _("revoked");
   return trust_value_to_string (trustlevel);
@@ -430,7 +393,7 @@ get_validity_string (ctrl_t ctrl, PKT_public_key *pk, PKT_user_id *uid)
  * 9 and 10 are used for internal purposes.
  */
 void
-mark_usable_uid_certs (ctrl_t ctrl, kbnode_t keyblock, kbnode_t uidnode,
+mark_usable_uid_certs (kbnode_t keyblock, kbnode_t uidnode,
                        u32 *main_kid, struct key_item *klist,
                        u32 curtime, u32 *next_expire)
 {
@@ -444,8 +407,7 @@ mark_usable_uid_certs (ctrl_t ctrl, kbnode_t keyblock, kbnode_t uidnode,
 
       node->flag &= ~(1<<8 | 1<<9 | 1<<10 | 1<<11 | 1<<12);
       if (node->pkt->pkttype == PKT_USER_ID
-          || node->pkt->pkttype == PKT_PUBLIC_SUBKEY
-          || node->pkt->pkttype == PKT_SECRET_SUBKEY)
+          || node->pkt->pkttype == PKT_PUBLIC_SUBKEY)
         break; /* ready */
       if (node->pkt->pkttype != PKT_SIGNATURE)
         continue;
@@ -461,7 +423,7 @@ mark_usable_uid_certs (ctrl_t ctrl, kbnode_t keyblock, kbnode_t uidnode,
 		     invalid signature */
       if (klist && !is_in_klist (klist, sig))
         continue;  /* no need to check it then */
-      if ((rc=check_key_signature (ctrl, keyblock, node, NULL)))
+      if ((rc=check_key_signature (keyblock, node, NULL)))
 	{
 	  /* we ignore anything that won't verify, but tag the
 	     no_pubkey case */
@@ -487,8 +449,7 @@ mark_usable_uid_certs (ctrl_t ctrl, kbnode_t keyblock, kbnode_t uidnode,
       u32 kid[2];
       u32 sigdate;
 
-      if (node->pkt->pkttype == PKT_PUBLIC_SUBKEY
-          || node->pkt->pkttype == PKT_SECRET_SUBKEY)
+      if (node->pkt->pkttype == PKT_PUBLIC_SUBKEY)
         break;
       if ( !(node->flag & (1<<9)) )
         continue; /* not a node to look at */
@@ -503,8 +464,7 @@ mark_usable_uid_certs (ctrl_t ctrl, kbnode_t keyblock, kbnode_t uidnode,
       /* Now find the latest and greatest signature */
       for (n=uidnode->next; n; n = n->next)
         {
-          if (n->pkt->pkttype == PKT_PUBLIC_SUBKEY
-              || n->pkt->pkttype == PKT_SECRET_SUBKEY)
+          if (n->pkt->pkttype == PKT_PUBLIC_SUBKEY)
             break;
           if ( !(n->flag & (1<<9)) )
             continue;
@@ -594,22 +554,21 @@ mark_usable_uid_certs (ctrl_t ctrl, kbnode_t keyblock, kbnode_t uidnode,
 
 
 static int
-clean_sigs_from_uid (ctrl_t ctrl, kbnode_t keyblock, kbnode_t uidnode,
+clean_sigs_from_uid (kbnode_t keyblock, kbnode_t uidnode,
                      int noisy, int self_only)
 {
   int deleted = 0;
   kbnode_t node;
   u32 keyid[2];
 
-  log_assert (keyblock->pkt->pkttype == PKT_PUBLIC_KEY
-              || keyblock->pkt->pkttype == PKT_SECRET_KEY);
+  assert (keyblock->pkt->pkttype==PKT_PUBLIC_KEY);
 
   keyid_from_pk (keyblock->pkt->pkt.public_key, keyid);
 
   /* Passing in a 0 for current time here means that we'll never weed
      out an expired sig.  This is correct behavior since we want to
      keep the most recent expired sig in a series. */
-  mark_usable_uid_certs (ctrl, keyblock, uidnode, NULL, NULL, 0, NULL);
+  mark_usable_uid_certs (keyblock, uidnode, NULL, NULL, 0, NULL);
 
   /* What we want to do here is remove signatures that are not
      considered as part of the trust calculations.  Thus, all invalid
@@ -695,15 +654,14 @@ clean_uid_from_key (kbnode_t keyblock, kbnode_t uidnode, int noisy)
   PKT_user_id *uid = uidnode->pkt->pkt.user_id;
   int deleted = 0;
 
-  log_assert (keyblock->pkt->pkttype == PKT_PUBLIC_KEY
-              || keyblock->pkt->pkttype == PKT_SECRET_KEY);
-  log_assert (uidnode->pkt->pkttype==PKT_USER_ID);
+  assert (keyblock->pkt->pkttype==PKT_PUBLIC_KEY);
+  assert (uidnode->pkt->pkttype==PKT_USER_ID);
 
   /* Skip valid user IDs, compacted user IDs, and non-self-signed user
      IDs if --allow-non-selfsigned-uid is set. */
   if (uid->created
       || uid->flags.compacted
-      || (!uid->flags.expired && !uid->flags.revoked && opt.allow_non_selfsigned_uid))
+      || (!uid->is_expired && !uid->is_revoked && opt.allow_non_selfsigned_uid))
     return 0;
 
   for (node=uidnode->next;
@@ -723,9 +681,9 @@ clean_uid_from_key (kbnode_t keyblock, kbnode_t uidnode, int noisy)
       const char *reason;
       char *user = utf8_to_native (uid->name, uid->len, 0);
 
-      if (uid->flags.revoked)
+      if (uid->is_revoked)
 	reason = _("revoked");
-      else if (uid->flags.expired)
+      else if (uid->is_expired)
 	reason = _("expired");
       else
 	reason = _("invalid");
@@ -743,14 +701,13 @@ clean_uid_from_key (kbnode_t keyblock, kbnode_t uidnode, int noisy)
 
 /* Needs to be called after a merge_keys_and_selfsig() */
 void
-clean_one_uid (ctrl_t ctrl, kbnode_t keyblock, kbnode_t uidnode,
-               int noisy, int self_only, int *uids_cleaned, int *sigs_cleaned)
+clean_one_uid (kbnode_t keyblock, kbnode_t uidnode, int noisy, int self_only,
+               int *uids_cleaned, int *sigs_cleaned)
 {
   int dummy = 0;
 
-  log_assert (keyblock->pkt->pkttype == PKT_PUBLIC_KEY
-              || keyblock->pkt->pkttype == PKT_SECRET_KEY);
-  log_assert (uidnode->pkt->pkttype==PKT_USER_ID);
+  assert (keyblock->pkt->pkttype==PKT_PUBLIC_KEY);
+  assert (uidnode->pkt->pkttype==PKT_USER_ID);
 
   if (!uids_cleaned)
     uids_cleaned = &dummy;
@@ -762,46 +719,24 @@ clean_one_uid (ctrl_t ctrl, kbnode_t keyblock, kbnode_t uidnode,
      to bother with the other.  */
   *uids_cleaned += clean_uid_from_key (keyblock, uidnode, noisy);
   if (!uidnode->pkt->pkt.user_id->flags.compacted)
-    *sigs_cleaned += clean_sigs_from_uid (ctrl, keyblock, uidnode,
-                                          noisy, self_only);
+    *sigs_cleaned += clean_sigs_from_uid (keyblock, uidnode, noisy, self_only);
 }
 
 
-/* NB: This function marks the deleted nodes only and the caller is
- * responsible to skip or remove them.  */
 void
-clean_key (ctrl_t ctrl, kbnode_t keyblock, int noisy, int self_only,
+clean_key (kbnode_t keyblock, int noisy, int self_only,
            int *uids_cleaned, int *sigs_cleaned)
 {
-  kbnode_t node;
+  kbnode_t uidnode;
 
-  merge_keys_and_selfsig (ctrl, keyblock);
+  merge_keys_and_selfsig (keyblock);
 
-  for (node = keyblock->next;
-       node && !(node->pkt->pkttype == PKT_PUBLIC_SUBKEY
-                    || node->pkt->pkttype == PKT_SECRET_SUBKEY);
-       node = node->next)
+  for (uidnode = keyblock->next;
+       uidnode && uidnode->pkt->pkttype != PKT_PUBLIC_SUBKEY;
+       uidnode = uidnode->next)
     {
-      if (node->pkt->pkttype == PKT_USER_ID)
-        clean_one_uid (ctrl, keyblock, node, noisy, self_only,
+      if (uidnode->pkt->pkttype == PKT_USER_ID)
+        clean_one_uid (keyblock, uidnode,noisy, self_only,
                        uids_cleaned, sigs_cleaned);
     }
-
-  /* Remove bogus subkey binding signatures: The only signatures
-   * allowed are of class 0x18 and 0x28.  */
-  log_assert (!node || (node->pkt->pkttype == PKT_PUBLIC_SUBKEY
-                        || node->pkt->pkttype == PKT_SECRET_SUBKEY));
-  for (; node; node = node->next)
-    {
-      if (is_deleted_kbnode (node))
-        continue;
-      if (node->pkt->pkttype == PKT_SIGNATURE
-          && !(IS_SUBKEY_SIG (node->pkt->pkt.signature)
-                || IS_SUBKEY_REV (node->pkt->pkt.signature)))
-        {
-          delete_kbnode (node);
-          if (sigs_cleaned)
-            ++*sigs_cleaned;
-        }
-    }
 }
