@@ -42,10 +42,7 @@
 #include "runner.h"
 #include "create.h"
 #include "mount.h"
-#include "suspend.h"
 #include "mountinfo.h"
-#include "backend.h"
-#include "call-syshelp.h"
 
 
 enum cmd_and_opt_values {
@@ -59,10 +56,7 @@ enum cmd_and_opt_values {
   aCreate,
   aMount,
   aUmount,
-  aSuspend,
-  aResume,
   aServer,
-  aFindDevice,
 
   oOptions,
   oDebug,
@@ -79,7 +73,6 @@ enum cmd_and_opt_values {
 
   oAgentProgram,
   oGpgProgram,
-  oType,
 
   oDisplay,
   oTTYname,
@@ -113,10 +106,7 @@ static ARGPARSE_OPTS opts[] = {
   ARGPARSE_c (aCreate, "create", N_("Create a new file system container")),
   ARGPARSE_c (aMount,  "mount",  N_("Mount a file system container") ),
   ARGPARSE_c (aUmount, "umount", N_("Unmount a file system container") ),
-  ARGPARSE_c (aSuspend, "suspend", N_("Suspend a file system container") ),
-  ARGPARSE_c (aResume,  "resume",  N_("Resume a file system container") ),
   ARGPARSE_c (aServer, "server", N_("Run in server mode")),
-  ARGPARSE_c (aFindDevice, "find-device", "@"),
 
   ARGPARSE_c (aGPGConfList, "gpgconf-list", "@"),
   ARGPARSE_c (aGPGConfTest, "gpgconf-test", "@"),
@@ -124,7 +114,6 @@ static ARGPARSE_OPTS opts[] = {
   ARGPARSE_group (301, N_("@\nOptions:\n ")),
 
   ARGPARSE_s_s (oRecipient, "recipient", N_("|USER-ID|encrypt for USER-ID")),
-  ARGPARSE_s_s (oType, "type", N_("|NAME|use container format NAME")),
 
   ARGPARSE_s_s (oOutput, "output", N_("|FILE|write output to FILE")),
   ARGPARSE_s_n (oVerbose, "verbose", N_("verbose")),
@@ -249,7 +238,7 @@ my_strusage( int level )
       break;
 
     case 31: p = "\nHome: "; break;
-    case 32: p = gnupg_homedir (); break;
+    case 32: p = opt.homedir; break;
 
     default: p = NULL; break;
     }
@@ -366,7 +355,7 @@ main ( int argc, char **argv)
   set_strusage (my_strusage);
   gcry_control (GCRYCTL_SUSPEND_SECMEM_WARN);
 
-  log_set_prefix (G13_NAME, GPGRT_LOG_WITH_PREFIX);
+  log_set_prefix (G13_NAME, 1);
 
   /* Make sure that our subsystems are ready.  */
   i18n_init ();
@@ -374,6 +363,11 @@ main ( int argc, char **argv)
 
   npth_init ();
 
+  /* Check that the Libgcrypt is suitable.  */
+  if (!gcry_check_version (NEED_LIBGCRYPT_VERSION) )
+    log_fatal (_("%s is too old (need %s, have %s)\n"), "libgcrypt",
+               NEED_LIBGCRYPT_VERSION, gcry_check_version (NULL) );
+
   /* Take extra care of the random pool.  */
   gcry_control (GCRYCTL_USE_SECURE_RNDPOOL);
 
@@ -388,6 +382,8 @@ main ( int argc, char **argv)
     log_fatal ("error allocating session environment block: %s\n",
                strerror (errno));
 
+  opt.homedir = default_homedir ();
+
   /* First check whether we have a config file on the commandline.  */
   orig_argc = argc;
   orig_argv = argv;
@@ -407,7 +403,7 @@ main ( int argc, char **argv)
       else if (pargs.r_opt == oNoOptions)
         default_config = 0; /* --no-options */
       else if (pargs.r_opt == oHomedir)
-        gnupg_set_homedir (pargs.r.ret_str);
+        opt.homedir = pargs.r.ret_str;
     }
 
   /* Initialize the secure memory. */
@@ -441,7 +437,7 @@ main ( int argc, char **argv)
 
   /* Set the default option file */
   if (default_config )
-    configname = make_filename (gnupg_homedir (), G13_NAME".conf", NULL);
+    configname = make_filename (opt.homedir, G13_NAME".conf", NULL);
 
   argc        = orig_argc;
   argv        = orig_argv;
@@ -459,7 +455,7 @@ main ( int argc, char **argv)
           if (default_config)
             {
               if (parse_debug)
-                log_info (_("Note: no default option file '%s'\n"), configname);
+                log_info (_("NOTE: no default option file '%s'\n"), configname);
             }
           else
             {
@@ -490,10 +486,8 @@ main ( int argc, char **argv)
         case aServer:
         case aMount:
         case aUmount:
-        case aSuspend:
-        case aResume:
+          /* nokeysetup = 1; */
         case aCreate:
-        case aFindDevice:
           set_cmd (&cmd, pargs.r_opt);
           break;
 
@@ -548,7 +542,7 @@ main ( int argc, char **argv)
 	    }
           break;
 
-        case oHomedir: gnupg_set_homedir (pargs.r.ret_str); break;
+        case oHomedir: opt.homedir = pargs.r.ret_str; break;
 
         case oAgentProgram: opt.agent_program = pargs.r.ret_str;  break;
         case oGpgProgram: opt.gpg_program = pargs.r.ret_str;  break;
@@ -576,19 +570,6 @@ main ( int argc, char **argv)
           add_to_strlist (&recipients, pargs.r.ret_str);
           break;
 
-        case oType:
-          if (!strcmp (pargs.r.ret_str, "help"))
-            {
-              be_parse_conttype_name (NULL);
-              g13_exit (0);
-            }
-          cmdline_conttype = be_parse_conttype_name (pargs.r.ret_str);
-          if (!cmdline_conttype)
-            {
-              pargs.r_opt = ARGPARSE_INVALID_ARG;
-              pargs.err = ARGPARSE_PRINT_ERROR;
-            }
-          break;
 
         default:
           pargs.err = configfp? ARGPARSE_PRINT_WARNING:ARGPARSE_PRINT_ERROR;
@@ -619,8 +600,7 @@ main ( int argc, char **argv)
   configname = NULL;
 
   if (!opt.config_filename)
-    opt.config_filename = make_filename (gnupg_homedir (),
-                                         G13_NAME".conf", NULL);
+    opt.config_filename = make_filename (opt.homedir, G13_NAME".conf", NULL);
 
   if (log_get_errorcount(0))
     g13_exit(2);
@@ -628,8 +608,6 @@ main ( int argc, char **argv)
   /* Now that we have the options parsed we need to update the default
      control structure.  */
   g13_init_default_ctrl (&ctrl);
-  ctrl.recipients = recipients;
-  recipients = NULL;
 
   if (nogreeting)
     greeting = 0;
@@ -651,14 +629,14 @@ main ( int argc, char **argv)
 
       for (i=0; i < argc; i++)
         if (argv[i][0] == '-' && argv[i][1] == '-')
-          log_info (_("Note: '%s' is not considered an option\n"), argv[i]);
+          log_info (_("NOTE: '%s' is not considered an option\n"), argv[i]);
     }
 
 
   if (logfile)
     {
       log_set_file (logfile);
-      log_set_prefix (NULL, GPGRT_LOG_WITH_PREFIX | GPGRT_LOG_WITH_TIME | GPGRT_LOG_WITH_PID);
+      log_set_prefix (NULL, 1|2|4);
     }
 
   if (gnupg_faked_time_p ())
@@ -687,7 +665,7 @@ main ( int argc, char **argv)
   /* Set the standard GnuPG random seed file.  */
   if (use_random_seed)
     {
-      char *p = make_filename (gnupg_homedir (), "random_seed", NULL);
+      char *p = make_filename (opt.homedir, "random_seed", NULL);
       gcry_control (GCRYCTL_SET_RANDOM_SEED_FILE, p);
       xfree(p);
     }
@@ -703,7 +681,7 @@ main ( int argc, char **argv)
       strlist_t sl;
       int failed = 0;
 
-      for (sl = ctrl->recipients; sl; sl = sl->next)
+      for (sl = recipients; sl; sl = sl->next)
         if (check_encryption_key ())
           failed = 1;
       if (failed)
@@ -743,23 +721,7 @@ main ( int argc, char **argv)
           log_error ("server exited with error: %s <%s>\n",
                      gpg_strerror (err), gpg_strsource (err));
         else
-          g13_request_shutdown ();
-      }
-      break;
-
-    case aFindDevice:
-      {
-        char *blockdev;
-
-        if (argc != 1)
-          wrong_args ("--find-device name");
-
-        err = call_syshelp_find_device (&ctrl, argv[0], &blockdev);
-        if (err)
-          log_error ("error finding device '%s': %s <%s>\n",
-                     argv[0], gpg_strerror (err), gpg_strsource (err));
-        else
-          puts (blockdev);
+          shutdown_pending++;
       }
       break;
 
@@ -768,12 +730,12 @@ main ( int argc, char **argv)
         if (argc != 1)
           wrong_args ("--create filename");
         start_idle_task ();
-        err = g13_create_container (&ctrl, argv[0]);
+        err = g13_create_container (&ctrl, argv[0], recipients);
         if (err)
           log_error ("error creating a new container: %s <%s>\n",
                      gpg_strerror (err), gpg_strsource (err));
         else
-          g13_request_shutdown ();
+          shutdown_pending++;
       }
       break;
 
@@ -789,48 +751,11 @@ main ( int argc, char **argv)
       }
       break;
 
-    case aUmount: /* Unmount a mounted container.  */
-      {
-        if (argc != 1)
-          wrong_args ("--umount filename");
-        err = g13_umount_container (&ctrl, argv[0], NULL);
-        if (err)
-          log_error ("error unmounting container '%s': %s <%s>\n",
-                     *argv, gpg_strerror (err), gpg_strsource (err));
-      }
-      break;
-
-    case aSuspend: /* Suspend a container. */
-      {
-        /* Fixme: Should we add a suspend all container option?  */
-        if (argc != 1)
-          wrong_args ("--suspend filename");
-        err = g13_suspend_container (&ctrl, argv[0]);
-        if (err)
-          log_error ("error suspending container '%s': %s <%s>\n",
-                     *argv, gpg_strerror (err), gpg_strsource (err));
-      }
-      break;
-
-    case aResume: /* Resume a suspended container. */
-      {
-        /* Fixme: Should we add a resume all container option?  */
-        if (argc != 1)
-          wrong_args ("--resume filename");
-        err = g13_resume_container (&ctrl, argv[0]);
-        if (err)
-          log_error ("error resuming container '%s': %s <%s>\n",
-                     *argv, gpg_strerror (err), gpg_strsource (err));
-      }
-      break;
-
     default:
       log_error (_("invalid command (there is no implicit command)\n"));
       break;
     }
 
-  g13_deinit_default_ctrl (&ctrl);
-
   if (!err)
     join_idle_task ();
 
@@ -842,30 +767,12 @@ main ( int argc, char **argv)
 
 /* Store defaults into the per-connection CTRL object.  */
 void
-g13_init_default_ctrl (ctrl_t ctrl)
+g13_init_default_ctrl (struct server_control_s *ctrl)
 {
   ctrl->conttype = cmdline_conttype? cmdline_conttype : CONTTYPE_ENCFS;
 }
 
 
-/* Release remaining resources allocated in the CTRL object.  */
-void
-g13_deinit_default_ctrl (ctrl_t ctrl)
-{
-  call_syshelp_release (ctrl);
-  FREE_STRLIST (ctrl->recipients);
-}
-
-
-/* Request a shutdown.  This can be used when the process should
- * finish instead of running the idle task.  */
-void
-g13_request_shutdown (void)
-{
-  shutdown_pending++;
-}
-
-
 /* This function is called for each signal we catch.  It is run in the
    main context or the one of a NPth thread and thus it is not
    restricted in what it may do.  */
@@ -998,11 +905,9 @@ idle_task (void *dummy_arg)
 	}
 
       if (ret <= 0)
-        {
-          /* Interrupt or timeout.  Will be handled when calculating the
-             next timeout.  */
-          continue;
-        }
+	/* Interrupt or timeout.  Will be handled when calculating the
+	   next timeout.  */
+	continue;
 
       /* Here one would add processing of file descriptors.  */
     }
