@@ -25,9 +25,6 @@
 
 /* Forward declaration for an object defined in server.c.  */
 struct server_local_s;
-/* Forward declaration for an object defined in call-syshelp.c.  */
-struct call_syshelp_s;
-
 
 /* Session control object.  This object is passed down to most
    functions.  The default values for it are set by
@@ -37,7 +34,6 @@ struct server_control_s
   int no_server;      /* We are not running under server control */
   int  status_fd;     /* Only for non-server mode */
   struct server_local_s *server_local;
-  struct call_syshelp_s *syshelp_local;
 
   int agent_seen;     /* Flag indicating that the gpg-agent has been
                          accessed.  */
@@ -47,14 +43,10 @@ struct server_control_s
   /* Type of the current container.  See the CONTTYPE_ constants.  */
   int conttype;
 
-  strlist_t recipients; /* List of recipients.  */
-
 };
 
 
 /*-- g13.c --*/
-void g13_init_default_ctrl (ctrl_t ctrl);
-void g13_deinit_default_ctrl (ctrl_t ctrl);
-void g13_request_shutdown (void);
+void g13_init_default_ctrl (struct server_control_s *ctrl);
 
 #endif /*G13_H*/
