@@ -30,10 +30,9 @@
 #include "i18n.h"
 #include "keyblob.h"
 #include "server.h"
-#include "create.h"
 #include "mount.h"
-#include "suspend.h"
-#include "../common/server-help.h"
+#include "create.h"
+
 
 /* The filepointer for status message used in non-server mode */
 static FILE *statusfp;
@@ -46,6 +45,8 @@ struct server_local_s
   assuan_context_t assuan_ctx;
 
   char *containername;  /* Malloced active containername.  */
+
+  strlist_t recipients; /* List of recipients.  */
 };
 
 
@@ -65,6 +66,37 @@ static int command_has_option (const char *cmd, const char *cmdopt);
 #define set_error(e,t) assuan_set_error (ctx, gpg_error (e), (t))
 
 
+/* Skip over options.  Blanks after the options are also removed.  */
+static char *
+skip_options (const char *line)
+{
+  while (spacep (line))
+    line++;
+  while ( *line == '-' && line[1] == '-' )
+    {
+      while (*line && !spacep (line))
+        line++;
+      while (spacep (line))
+        line++;
+    }
+  return (char*)line;
+}
+
+
+/* Check whether the option NAME appears in LINE.  */
+/* static int */
+/* has_option (const char *line, const char *name) */
+/* { */
+/*   const char *s; */
+/*   int n = strlen (name); */
+
+/*   s = strstr (line, name); */
+/*   if (s && s >= skip_options (line)) */
+/*     return 0; */
+/*   return (s && (s == line || spacep (s-1)) && (!s[n] || spacep (s+n))); */
+/* } */
+
+
 /* Helper to print a message while leaving a command.  */
 static gpg_error_t
 leave_cmd (assuan_context_t ctx, gpg_error_t err)
@@ -162,7 +194,7 @@ reset_notify (assuan_context_t ctx, char *line)
   xfree (ctrl->server_local->containername);
   ctrl->server_local->containername = NULL;
 
-  FREE_STRLIST (ctrl->recipients);
+  FREE_STRLIST (ctrl->server_local->recipients);
 
   assuan_close_input_fd (ctx);
   assuan_close_output_fd (ctx);
@@ -326,56 +358,6 @@ cmd_umount (assuan_context_t ctx, char *line)
 }
 
 
-static const char hlp_suspend[] =
-  "SUSPEND\n"
-  "\n"
-  "Suspend the currently set device.";
-static gpg_error_t
-cmd_suspend (assuan_context_t ctx, char *line)
-{
-  ctrl_t ctrl = assuan_get_pointer (ctx);
-  gpg_error_t err;
-
-  line = skip_options (line);
-  if (*line)
-    {
-      err = gpg_error (GPG_ERR_ASS_SYNTAX);
-      goto leave;
-    }
-
-  /* Perform the suspend operation.  */
-  err = g13_suspend_container (ctrl, ctrl->server_local->containername);
-
- leave:
-  return leave_cmd (ctx, err);
-}
-
-
-static const char hlp_resume[] =
-  "RESUME\n"
-  "\n"
-  "Resume the currently set device.";
-static gpg_error_t
-cmd_resume (assuan_context_t ctx, char *line)
-{
-  ctrl_t ctrl = assuan_get_pointer (ctx);
-  gpg_error_t err;
-
-  line = skip_options (line);
-  if (*line)
-    {
-      err = gpg_error (GPG_ERR_ASS_SYNTAX);
-      goto leave;
-    }
-
-  /* Perform the suspend operation.  */
-  err = g13_resume_container (ctrl, ctrl->server_local->containername);
-
- leave:
-  return leave_cmd (ctx, err);
-}
-
-
 static const char hlp_recipient[] =
   "RECIPIENT <userID>\n"
   "\n"
@@ -390,7 +372,7 @@ cmd_recipient (assuan_context_t ctx, char *line)
 
   line = skip_options (line);
 
-  if (!add_to_strlist_try (&ctrl->recipients, line))
+  if (!add_to_strlist_try (&ctrl->server_local->recipients, line))
     err = gpg_error_from_syserror ();
 
   return leave_cmd (ctx, err);
@@ -456,11 +438,11 @@ cmd_create (assuan_context_t ctx, char *line)
     }
 
   /* Create container.  */
-  err = g13_create_container (ctrl, line);
+  err = g13_create_container (ctrl, line, ctrl->server_local->recipients);
 
   if (!err)
     {
-      FREE_STRLIST (ctrl->recipients);
+      FREE_STRLIST (ctrl->server_local->recipients);
 
       /* Store the filename.  */
       ctrl->server_local->containername = xtrystrdup (line);
@@ -563,8 +545,6 @@ register_commands (assuan_context_t ctx)
     { "OPEN",          cmd_open,   hlp_open },
     { "MOUNT",         cmd_mount,  hlp_mount},
     { "UMOUNT",        cmd_umount, hlp_umount },
-    { "SUSPEND",       cmd_suspend, hlp_suspend },
-    { "RESUME",        cmd_resume,  hlp_resume },
     { "RECIPIENT",     cmd_recipient, hlp_recipient },
     { "SIGNER",        cmd_signer, hlp_signer },
     { "CREATE",        cmd_create, hlp_create },
@@ -631,12 +611,12 @@ g13_server (ctrl_t ctrl)
 
   if (opt.verbose || opt.debug)
     {
-      char *tmp;
+      char *tmp = NULL;
 
       tmp = xtryasprintf ("Home: %s\n"
                           "Config: %s\n"
                           "%s",
-                          gnupg_homedir (),
+                          opt.homedir,
                           opt.config_filename,
                           hello);
       if (tmp)
