@@ -15,7 +15,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, see <https://www.gnu.org/licenses/>.
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
 
 #ifndef GPGSM_H
@@ -33,8 +33,6 @@
 #include "../common/status.h"
 #include "../common/audit.h"
 #include "../common/session-env.h"
-#include "../common/ksba-io-support.h"
-#include "../common/compliance.h"
 
 
 #define MAX_DIGEST_LEN 64
@@ -63,6 +61,7 @@ struct
   int dry_run;      /* don't change any persistent data */
   int no_homedir_creation;
 
+  const char *homedir;         /* Configuration directory name */
   const char *config_filename; /* Name of the used config file. */
   const char *agent_program;
 
@@ -85,8 +84,6 @@ struct
 
   int with_keygrip; /* Option --with-keygrip active.  */
 
-  int pinentry_mode;
-
   int armor;        /* force base64 armoring (see also ctrl.with_base64) */
   int no_armor;     /* don't try to figure out whether data is base64 armored*/
 
@@ -128,6 +125,7 @@ struct
   int no_policy_check;      /* ignore certificate policies */
   int no_chain_validation;  /* Bypass all cert chain validity tests */
   int ignore_expiration;    /* Ignore the notAfter validity checks. */
+  char *fixed_passphrase;   /* Passphrase used by regression tests.  */
 
   int auto_issuer_key_retrieve; /* try to retrieve a missing issuer key. */
 
@@ -145,7 +143,6 @@ struct
      OID per string.  */
   strlist_t ignored_cert_extensions;
 
-  enum gnupg_compliance_mode compliance;
 } opt;
 
 /* Debug values and macros.  */
@@ -208,6 +205,10 @@ struct server_control_s
 };
 
 
+/* Data structure used in base64.c. */
+typedef struct base64_context_s *Base64Context;
+
+
 /* An object to keep a list of certificates. */
 struct certlist_s
 {
@@ -243,8 +244,6 @@ gpg_error_t gpgsm_status (ctrl_t ctrl, int no, const char *text);
 gpg_error_t gpgsm_status2 (ctrl_t ctrl, int no, ...) GPGRT_ATTR_SENTINEL(0);
 gpg_error_t gpgsm_status_with_err_code (ctrl_t ctrl, int no, const char *text,
                                         gpg_err_code_t ec);
-gpg_error_t gpgsm_status_with_error (ctrl_t ctrl, int no, const char *text,
-                                     gpg_error_t err);
 gpg_error_t gpgsm_proxy_pinentry_notify (ctrl_t ctrl,
                                          const unsigned char *line);
 
@@ -261,6 +260,19 @@ int  gpgsm_get_key_algo_info (ksba_cert_t cert, unsigned int *nbits);
 char *gpgsm_get_certid (ksba_cert_t cert);
 
 
+/*-- base64.c --*/
+int  gpgsm_create_reader (Base64Context *ctx,
+                          ctrl_t ctrl, estream_t fp, int allow_multi_pem,
+                          ksba_reader_t *r_reader);
+int gpgsm_reader_eof_seen (Base64Context ctx);
+void gpgsm_destroy_reader (Base64Context ctx);
+int  gpgsm_create_writer (Base64Context *ctx,
+                          ctrl_t ctrl, estream_t stream,
+                          ksba_writer_t *r_writer);
+int  gpgsm_finish_writer (Base64Context ctx);
+void gpgsm_destroy_writer (Base64Context ctx);
+
+
 /*-- certdump.c --*/
 void gpgsm_print_serial (estream_t fp, ksba_const_sexp_t p);
 void gpgsm_print_time (estream_t fp, ksba_isotime_t t);
@@ -327,8 +339,7 @@ int gpgsm_add_cert_to_certlist (ctrl_t ctrl, ksba_cert_t cert,
 int gpgsm_add_to_certlist (ctrl_t ctrl, const char *name, int secret,
                            certlist_t *listaddr, int is_encrypt_to);
 void gpgsm_release_certlist (certlist_t list);
-int gpgsm_find_cert (ctrl_t ctrl, const char *name, ksba_sexp_t keyid,
-                     ksba_cert_t *r_cert);
+int gpgsm_find_cert (const char *name, ksba_sexp_t keyid, ksba_cert_t *r_cert);
 
 /*-- keylist.c --*/
 gpg_error_t gpgsm_list_keys (ctrl_t ctrl, strlist_t names,
