@@ -14,7 +14,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, see <https://www.gnu.org/licenses/>.
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
 
 #include <config.h>
@@ -27,10 +27,9 @@
 #include <unistd.h>
 #include <assert.h>
 
-#include "../common/i18n.h"
+#include "i18n.h"
 #include "../common/exectool.h"
 #include "../common/sysutils.h"
-#include "../common/ccparray.h"
 #include "gpgtar.h"
 
 
@@ -122,15 +121,11 @@ extract_directory (const char *dirname, tar_header_t hdr)
   if (fname[strlen (fname)-1] == '/')
     fname[strlen (fname)-1] = 0;
 
+ /* Note that we don't need to care about EEXIST because we always
+     extract into a new hierarchy.  */
   if (! opt.dry_run && gnupg_mkdir (fname, "-rwx------"))
     {
       err = gpg_error_from_syserror ();
-      if (gpg_err_code (err) == GPG_ERR_EEXIST)
-        {
-          /* Ignore existing directories while extracting.  */
-          err = 0;
-        }
-
       if (gpg_err_code (err) == GPG_ERR_ENOENT)
         {
           /* Try to create the directory with parents but keep the
@@ -286,7 +281,7 @@ gpgtar_extract (const char *filename, int decrypt)
   if (filename)
     {
       if (!strcmp (filename, "-"))
-        stream = es_stdin;
+        stream = es_stdout;
       else
         stream = es_fopen (filename, "rb");
       if (!stream)
@@ -304,8 +299,8 @@ gpgtar_extract (const char *filename, int decrypt)
 
   if (decrypt)
     {
+      int i;
       strlist_t arg;
-      ccparray_t ccp;
       const char **argv;
 
       cipher_stream = stream;
@@ -316,22 +311,22 @@ gpgtar_extract (const char *filename, int decrypt)
           goto leave;
         }
 
-      ccparray_init (&ccp, 0);
-
-      ccparray_put (&ccp, "--decrypt");
-      for (arg = opt.gpg_arguments; arg; arg = arg->next)
-        ccparray_put (&ccp, arg->d);
-
-      ccparray_put (&ccp, NULL);
-      argv = ccparray_get (&ccp, NULL);
-      if (!argv)
+      argv = xtrycalloc (strlist_length (opt.gpg_arguments) + 2,
+                         sizeof *argv);
+      if (argv == NULL)
         {
           err = gpg_error_from_syserror ();
           goto leave;
         }
+      i = 0;
+      argv[i++] = "--decrypt";
+      for (arg = opt.gpg_arguments; arg; arg = arg->next)
+        argv[i++] = arg->d;
+      argv[i++] = NULL;
+      assert (i == strlist_length (opt.gpg_arguments) + 2);
 
       err = gnupg_exec_tool_stream (opt.gpg_program, argv,
-                                    cipher_stream, NULL, stream, NULL, NULL);
+                                    cipher_stream, stream);
       xfree (argv);
       if (err)
         goto leave;
