@@ -1,20 +1,20 @@
 /* rfc822parse.c - Simple mail and MIME parser
- * Copyright (C) 1999, 2000 Werner Koch, Duesseldorf
- * Copyright (C) 2003, 2004 g10 Code GmbH
+ *	Copyright (C) 1999, 2000 Werner Koch, Duesseldorf
+ *      Copyright (C) 2003, 2004 g10 Code GmbH
  *
- * This file is free software; you can redistribute it and/or modify
- * it under the terms of the GNU Lesser General Public License as
- * published by the Free Software Foundation; either version 2.1 of
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public License
+ * as published by the Free Software Foundation; either version 3 of
  * the License, or (at your option) any later version.
  *
- * This file is distributed in the hope that it will be useful,
+ * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  * GNU Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU Lesser General Public License
- * along with this program; if not, see <https://www.gnu.org/licenses/>.
-g */
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this program; if not, see <http://www.gnu.org/licenses/>.
+ */
 
 
 /* According to RFC822 binary zeroes are allowed at many places. We do
@@ -155,7 +155,7 @@ capitalize_header_name (unsigned char *name)
 
 #ifndef HAVE_STPCPY
 static char *
-my_stpcpy (char *a,const char *b)
+stpcpy (char *a,const char *b)
 {
   while (*b)
     *a++ = *b++;
@@ -163,7 +163,6 @@ my_stpcpy (char *a,const char *b)
 
   return (char*)a;
 }
-#define stpcpy my_stpcpy
 #endif
 
 
@@ -1118,7 +1117,7 @@ dump_structure (rfc822parse_t msg, part_t part, int indent)
     {
       rfc822parse_field_t ctx;
       part_t save_part; /* ugly hack - we should have a function to
-                           get part information. */
+                           get part inforation. */
       const char *s;
 
       save_part = msg->current_part;
@@ -1178,13 +1177,8 @@ show_event (rfc822parse_event_t event)
     case RFC822PARSE_T2BODY: s= "T2Body"; break;
     case RFC822PARSE_FINISH: s= "Finish"; break;
     case RFC822PARSE_RCVD_SEEN: s= "Rcvd_Seen"; break;
-    case RFC822PARSE_LEVEL_DOWN: s= "Level_Down"; break;
-    case RFC822PARSE_LEVEL_UP:   s= "Level_Up"; break;
     case RFC822PARSE_BOUNDARY: s= "Boundary"; break;
     case RFC822PARSE_LAST_BOUNDARY: s= "Last_Boundary"; break;
-    case RFC822PARSE_BEGIN_HEADER: s= "Begin_Header"; break;
-    case RFC822PARSE_PREAMBLE: s= "Preamble"; break;
-    case RFC822PARSE_EPILOGUE: s= "Epilogue"; break;
     default: s= "***invalid event***"; break;
     }
   printf ("*** got RFC822 event %s\n", s);
